VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsAES"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
' This is a stripped down version of cRijndael - Advanced Encryption Standard (AES) Class
' by John Korejwa  <korejwa@tiac.net> 09 / July / 2006 Version:   1.0
' All comments from this point on are orignal authors
'--------------------------------------------------------------------
Option Explicit
Public Status As String

'Title:     cRijndael - Advanced Encryption Standard (AES) Class
'Author:    John Korejwa  <korejwa@tiac.net>
'Filename:  Rijndael.cls
'Date:      09 / July / 2006
'Version:   1.0

'Usage:
'Create an instance of the class, set the cipher key, encrypt/decrypt to your heart's content


#Const SUPPORT_LEVEL = 2     'Default=0
'SUPPORT_LEVEL = 0 [AES Compliant]   Blocksize = 128                        Keysize = 128, 192, 256
'SUPPORT_LEVEL = 1 [Common Sizes ]   Blocksize = 128, 192, 256              Keysize = 128, 192, 256
'SUPPORT_LEVEL = 2 [  All Sizes  ]   Blocksize = 128, 160, 192, 224, 256    Keysize = 128, 160, 192, 224, 256

#Const COMPILE_CONSTANTS = 0 'Default=0
'COMPILE_CONSTANTS = 0 [Fast compile, small exe size]  Calculate tables during Class initialization
'COMPILE_CONSTANTS = 1 [Fast run time initialization]  Compile tables of constants


'These are arrays of constants.  They are initialized with the Class and do not change.
Private Te0(255)      As Long
Private Te1(255)      As Long
Private Te2(255)      As Long
Private Te3(255)      As Long
Private Te4(255)      As Long

Private Td0(255)      As Long
Private Td1(255)      As Long
Private Td2(255)      As Long
Private Td3(255)      As Long
Private Td4(255)      As Long

#If SUPPORT_LEVEL Then
Private rco(28)       As Long
#Else
Private rco(9)        As Long
#End If

'Key schedule arrays
Private Nr            As Long 'Number of rounds [For 128 bit block, Nr = {10, 12, 14} for 128, 192, 256 bit cipher key]
#If SUPPORT_LEVEL Then
Private fkey(119)     As Long 'Nb*(Nr + 1)
Private rkey(119)     As Long 'Nb*(Nr + 1)
#Else
Private fkey(59)      As Long 'Nb*(Nr + 1)
Private rkey(59)      As Long 'Nb*(Nr + 1)
#End If

'For file encryption, this is the maximum amount of memory (in bytes) allowed for file data
Private Const MaxFileChunkSize As Long = 4000000

Private Declare Sub CopyMemory Lib "kernel32" Alias "RtlMoveMemory" (Destination As Any, Source As Any, ByVal Length As Long)


'Decryption Key Scheduler.  Calculate rkey() decryption keys based on fkey() and Nr.
'Nb is the number of 32 bit units in the block size.
Private Sub CreateDecryptionKeys(Nb As Long)
    Dim i    As Long
    Dim j    As Long
    Dim k    As Long
    Dim s(3) As Byte

    'Invert the order of the round keys
    i = 0
    j = Nb * Nr
    For k = 0 To Nr
        CopyMemory rkey(i), fkey(j), Nb * 4&
        i = i + Nb
        j = j - Nb
    Next k

    'Apply the inverse MixColumn transform to all round keys except the first and the last
    For i = Nb To Nb * Nr - 1
        CopyMemory s(0), rkey(i), 4&
        rkey(i) = Td0(Te4(s(0)) And &HFF&) Xor _
                  Td1(Te4(s(1)) And &HFF&) Xor _
                  Td2(Te4(s(2)) And &HFF&) Xor _
                  Td3(Te4(s(3)) And &HFF&)
    Next i
End Sub


'Key Scheduler. Expand the cipher key into the encryption key schedule.
'pass(0 ... n-1) contains the cipher key, where n = {16, 20, 24, 28, 32} , depending on KeyBits.
'If SUPPORT_LEVEL is non-zero, you must specify BlockBits for the block size you will be using.
'Returns zero on success.
#If SUPPORT_LEVEL Then
Public Function SetCipherKey(Pass() As Byte, KeyBits As Long, BlockBits As Long) As Long
#Else
Public Function SetCipherKey(Pass() As Byte, KeyBits As Long) As Long
#End If
    Dim i    As Long
    Dim j    As Long
    Dim s(3) As Byte

#If SUPPORT_LEVEL Then
    Select Case BlockBits
    Case 128
#End If

    '128 bit block size
    Select Case KeyBits
    Case 128
        i = 4
        CopyMemory fkey(0), Pass(0), 4& * i
        For j = 0 To 9
            CopyMemory s(0), fkey(i - 1), 4&
            fkey(i) = fkey(i - 4) Xor (Te4(s(0)) And &HFF000000) _
                                  Xor (Te4(s(3)) And &HFF0000) _
                                  Xor (Te4(s(2)) And &HFF00&) _
                                  Xor (Te4(s(1)) And &HFF&) _
                                  Xor rco(j)
            fkey(i + 1) = fkey(i - 3) Xor fkey(i)
            fkey(i + 2) = fkey(i - 2) Xor fkey(i + 1)
            fkey(i + 3) = fkey(i - 1) Xor fkey(i + 2)
            i = i + 4
        Next j
        Nr = 10
'Debug.Assert i = (Nr + 1) * 4
#If SUPPORT_LEVEL = 2 Then
    Case 160
        i = 5
        j = 0
        CopyMemory fkey(0), Pass(0), 4& * i
        Do
            CopyMemory s(0), fkey(i - 1), 4&
            fkey(i) = fkey(i - 5) Xor (Te4(s(0)) And &HFF000000) _
                                  Xor (Te4(s(3)) And &HFF0000) _
                                  Xor (Te4(s(2)) And &HFF00&) _
                                  Xor (Te4(s(1)) And &HFF&) _
                                  Xor rco(j)
            fkey(i + 1) = fkey(i - 4) Xor fkey(i)
            fkey(i + 2) = fkey(i - 3) Xor fkey(i + 1)
            If j = 8 Then Exit Do
            fkey(i + 3) = fkey(i - 2) Xor fkey(i + 2)
            fkey(i + 4) = fkey(i - 1) Xor fkey(i + 3)
            i = i + 5
            j = j + 1
        Loop
        Nr = 11
'Debug.Assert i + 3 = (Nr + 1) * 4
#End If
    Case 192
        i = 6
        j = 0
        CopyMemory fkey(0), Pass(0), 4& * i
        Do
            CopyMemory s(0), fkey(i - 1), 4&
            fkey(i) = fkey(i - 6) Xor (Te4(s(0)) And &HFF000000) _
                                  Xor (Te4(s(3)) And &HFF0000) _
                                  Xor (Te4(s(2)) And &HFF00&) _
                                  Xor (Te4(s(1)) And &HFF&) _
                                  Xor rco(j)
            fkey(i + 1) = fkey(i - 5) Xor fkey(i)
            fkey(i + 2) = fkey(i - 4) Xor fkey(i + 1)
            fkey(i + 3) = fkey(i - 3) Xor fkey(i + 2)
            If j = 7 Then Exit Do
            fkey(i + 4) = fkey(i - 2) Xor fkey(i + 3)
            fkey(i + 5) = fkey(i - 1) Xor fkey(i + 4)
            i = i + 6
            j = j + 1
        Loop
        Nr = 12
'Debug.Assert i + 4 = (Nr + 1) * 4
#If SUPPORT_LEVEL = 2 Then
    Case 224
        i = 7
        CopyMemory fkey(0), Pass(0), 4& * i
        For j = 0 To 6
            CopyMemory s(0), fkey(i - 1), 4&
            fkey(i) = fkey(i - 7) Xor (Te4(s(0)) And &HFF000000) _
                                  Xor (Te4(s(3)) And &HFF0000) _
                                  Xor (Te4(s(2)) And &HFF00&) _
                                  Xor (Te4(s(1)) And &HFF&) _
                                  Xor rco(j)
            fkey(i + 1) = fkey(i - 6) Xor fkey(i)
            fkey(i + 2) = fkey(i - 5) Xor fkey(i + 1)
            fkey(i + 3) = fkey(i - 4) Xor fkey(i + 2)
            CopyMemory s(0), fkey(i + 3), 4&
            fkey(i + 4) = fkey(i - 3) Xor (Te4(s(3)) And &HFF000000) _
                                      Xor (Te4(s(2)) And &HFF0000) _
                                      Xor (Te4(s(1)) And &HFF00&) _
                                      Xor (Te4(s(0)) And &HFF&)
            fkey(i + 5) = fkey(i - 2) Xor fkey(i + 4)
            fkey(i + 6) = fkey(i - 1) Xor fkey(i + 5)
            i = i + 7
        Next j
        Nr = 13
'Debug.Assert i = (Nr + 1) * 4
#End If
    Case 256
        i = 8
        j = 0
        CopyMemory fkey(0), Pass(0), 4& * i
        Do
            CopyMemory s(0), fkey(i - 1), 4&
            fkey(i) = fkey(i - 8) Xor (Te4(s(0)) And &HFF000000) _
                                  Xor (Te4(s(3)) And &HFF0000) _
                                  Xor (Te4(s(2)) And &HFF00&) _
                                  Xor (Te4(s(1)) And &HFF&) _
                                  Xor rco(j)
            fkey(i + 1) = fkey(i - 7) Xor fkey(i)
            fkey(i + 2) = fkey(i - 6) Xor fkey(i + 1)
            fkey(i + 3) = fkey(i - 5) Xor fkey(i + 2)
            If j = 6 Then Exit Do
            CopyMemory s(0), fkey(i + 3), 4&
            fkey(i + 4) = fkey(i - 4) Xor (Te4(s(3)) And &HFF000000) _
                                      Xor (Te4(s(2)) And &HFF0000) _
                                      Xor (Te4(s(1)) And &HFF00&) _
                                      Xor (Te4(s(0)) And &HFF&)
            fkey(i + 5) = fkey(i - 3) Xor fkey(i + 4)
            fkey(i + 6) = fkey(i - 2) Xor fkey(i + 5)
            fkey(i + 7) = fkey(i - 1) Xor fkey(i + 6)
            i = i + 8
            j = j + 1
        Loop
        Nr = 14
'Debug.Assert i + 4 = (Nr + 1) * 4
    Case Else
        Err.Raise 1, , "cRijndael.SetCipherKey - Illegal KeyBits Value"
        SetCipherKey = 1
        Exit Function
    End Select
    CreateDecryptionKeys 4

#If SUPPORT_LEVEL Then
#If SUPPORT_LEVEL = 2 Then
    Case 160

    '160 bit block size
    Select Case KeyBits
    Case 128
        i = 4
        CopyMemory fkey(0), Pass(0), 4& * i
        For j = 0 To 13
            CopyMemory s(0), fkey(i - 1), 4&
            fkey(i) = fkey(i - 4) Xor (Te4(s(0)) And &HFF000000) _
                                  Xor (Te4(s(3)) And &HFF0000) _
                                  Xor (Te4(s(2)) And &HFF00&) _
                                  Xor (Te4(s(1)) And &HFF&) _
                                  Xor rco(j)
            fkey(i + 1) = fkey(i - 3) Xor fkey(i)
            fkey(i + 2) = fkey(i - 2) Xor fkey(i + 1)
            fkey(i + 3) = fkey(i - 1) Xor fkey(i + 2)
            i = i + 4
        Next j
        Nr = 11
'Debug.Assert i = (Nr + 1) * 5
    Case 160
        i = 5
        CopyMemory fkey(0), Pass(0), 4& * i
        For j = 0 To 10
            CopyMemory s(0), fkey(i - 1), 4&
            fkey(i) = fkey(i - 5) Xor (Te4(s(0)) And &HFF000000) _
                                  Xor (Te4(s(3)) And &HFF0000) _
                                  Xor (Te4(s(2)) And &HFF00&) _
                                  Xor (Te4(s(1)) And &HFF&) _
                                  Xor rco(j)
            fkey(i + 1) = fkey(i - 4) Xor fkey(i)
            fkey(i + 2) = fkey(i - 3) Xor fkey(i + 1)
            fkey(i + 3) = fkey(i - 2) Xor fkey(i + 2)
            fkey(i + 4) = fkey(i - 1) Xor fkey(i + 3)
            i = i + 5
        Next j
        Nr = 11
'Debug.Assert i = (Nr + 1) * 5
    Case 192
        i = 6
        j = 0
        CopyMemory fkey(0), Pass(0), 4& * i
        Do
            CopyMemory s(0), fkey(i - 1), 4&
            fkey(i) = fkey(i - 6) Xor (Te4(s(0)) And &HFF000000) _
                                  Xor (Te4(s(3)) And &HFF0000) _
                                  Xor (Te4(s(2)) And &HFF00&) _
                                  Xor (Te4(s(1)) And &HFF&) _
                                  Xor rco(j)
            fkey(i + 1) = fkey(i - 5) Xor fkey(i)
            fkey(i + 2) = fkey(i - 4) Xor fkey(i + 1)
            fkey(i + 3) = fkey(i - 3) Xor fkey(i + 2)
            fkey(i + 4) = fkey(i - 2) Xor fkey(i + 3)
            If j = 9 Then Exit Do
            fkey(i + 5) = fkey(i - 1) Xor fkey(i + 4)
            i = i + 6
            j = j + 1
        Loop
        Nr = 12
'Debug.Assert i + 5 = (Nr + 1) * 5
    Case 224
        i = 7
        CopyMemory fkey(0), Pass(0), 4& * i
        For j = 0 To 8
            CopyMemory s(0), fkey(i - 1), 4&
            fkey(i) = fkey(i - 7) Xor (Te4(s(0)) And &HFF000000) _
                                  Xor (Te4(s(3)) And &HFF0000) _
                                  Xor (Te4(s(2)) And &HFF00&) _
                                  Xor (Te4(s(1)) And &HFF&) _
                                  Xor rco(j)
            fkey(i + 1) = fkey(i - 6) Xor fkey(i)
            fkey(i + 2) = fkey(i - 5) Xor fkey(i + 1)
            fkey(i + 3) = fkey(i - 4) Xor fkey(i + 2)
            CopyMemory s(0), fkey(i + 3), 4&
            fkey(i + 4) = fkey(i - 3) Xor (Te4(s(3)) And &HFF000000) _
                                      Xor (Te4(s(2)) And &HFF0000) _
                                      Xor (Te4(s(1)) And &HFF00&) _
                                      Xor (Te4(s(0)) And &HFF&)
            fkey(i + 5) = fkey(i - 2) Xor fkey(i + 4)
            fkey(i + 6) = fkey(i - 1) Xor fkey(i + 5)
            i = i + 7
        Next j
        Nr = 13
'Debug.Assert i = (Nr + 1) * 5
    Case 256
        i = 8
        j = 0
        CopyMemory fkey(0), Pass(0), 4& * i
        Do
            CopyMemory s(0), fkey(i - 1), 4&
            fkey(i) = fkey(i - 8) Xor (Te4(s(0)) And &HFF000000) _
                                  Xor (Te4(s(3)) And &HFF0000) _
                                  Xor (Te4(s(2)) And &HFF00&) _
                                  Xor (Te4(s(1)) And &HFF&) _
                                  Xor rco(j)
            fkey(i + 1) = fkey(i - 7) Xor fkey(i)
            fkey(i + 2) = fkey(i - 6) Xor fkey(i + 1)
            If j = 8 Then Exit Do
            fkey(i + 3) = fkey(i - 5) Xor fkey(i + 2)
            CopyMemory s(0), fkey(i + 3), 4&
            fkey(i + 4) = fkey(i - 4) Xor (Te4(s(3)) And &HFF000000) _
                                      Xor (Te4(s(2)) And &HFF0000) _
                                      Xor (Te4(s(1)) And &HFF00&) _
                                      Xor (Te4(s(0)) And &HFF&)
            fkey(i + 5) = fkey(i - 3) Xor fkey(i + 4)
            fkey(i + 6) = fkey(i - 2) Xor fkey(i + 5)
            fkey(i + 7) = fkey(i - 1) Xor fkey(i + 6)
            i = i + 8
            j = j + 1
        Loop
        Nr = 14
'Debug.Assert i + 3 = (Nr + 1) * 5
    Case Else
        Err.Raise 1, , "cRijndael.SetCipherKey - Illegal KeyBits Value"
        SetCipherKey = 1
        Exit Function
    End Select
    CreateDecryptionKeys 5
#End If
    Case 192

    '192 bit block size
    Select Case KeyBits
    Case 128
        i = 4
        j = 0
        CopyMemory fkey(0), Pass(0), 4& * i
        Do
            CopyMemory s(0), fkey(i - 1), 4&
            fkey(i) = fkey(i - 4) Xor (Te4(s(0)) And &HFF000000) _
                                  Xor (Te4(s(3)) And &HFF0000) _
                                  Xor (Te4(s(2)) And &HFF00&) _
                                  Xor (Te4(s(1)) And &HFF&) _
                                  Xor rco(j)
            fkey(i + 1) = fkey(i - 3) Xor fkey(i)
            If j = 18 Then Exit Do
            fkey(i + 2) = fkey(i - 2) Xor fkey(i + 1)
            fkey(i + 3) = fkey(i - 1) Xor fkey(i + 2)
            i = i + 4
            j = j + 1
        Loop
        Nr = 12
'Debug.Assert i + 2 = (Nr + 1) * 6
#If SUPPORT_LEVEL = 2 Then
    Case 160
        i = 5
        j = 0
        CopyMemory fkey(0), Pass(0), 4& * i
        Do
            CopyMemory s(0), fkey(i - 1), 4&
            fkey(i) = fkey(i - 5) Xor (Te4(s(0)) And &HFF000000) _
                                  Xor (Te4(s(3)) And &HFF0000) _
                                  Xor (Te4(s(2)) And &HFF00&) _
                                  Xor (Te4(s(1)) And &HFF&) _
                                  Xor rco(j)
            fkey(i + 1) = fkey(i - 4) Xor fkey(i)
            fkey(i + 2) = fkey(i - 3) Xor fkey(i + 1)
            If j = 14 Then Exit Do
            fkey(i + 3) = fkey(i - 2) Xor fkey(i + 2)
            fkey(i + 4) = fkey(i - 1) Xor fkey(i + 3)
            i = i + 5
            j = j + 1
        Loop
        Nr = 12
'Debug.Assert i + 3 = (Nr + 1) * 6
#End If
    Case 192
        i = 6
        CopyMemory fkey(0), Pass(0), 4& * i
        For j = 0 To 11
            CopyMemory s(0), fkey(i - 1), 4&
            fkey(i) = fkey(i - 6) Xor (Te4(s(0)) And &HFF000000) _
                                  Xor (Te4(s(3)) And &HFF0000) _
                                  Xor (Te4(s(2)) And &HFF00&) _
                                  Xor (Te4(s(1)) And &HFF&) _
                                  Xor rco(j)
            fkey(i + 1) = fkey(i - 5) Xor fkey(i)
            fkey(i + 2) = fkey(i - 4) Xor fkey(i + 1)
            fkey(i + 3) = fkey(i - 3) Xor fkey(i + 2)
            fkey(i + 4) = fkey(i - 2) Xor fkey(i + 3)
            fkey(i + 5) = fkey(i - 1) Xor fkey(i + 4)
            i = i + 6
        Next j
        Nr = 12
'Debug.Assert i = (Nr + 1) * 6
#If SUPPORT_LEVEL = 2 Then
    Case 224 '(Nr+1)*Nb/Nk  (when to exit)
        i = 7
        CopyMemory fkey(0), Pass(0), 4& * i
        For j = 0 To 10
            CopyMemory s(0), fkey(i - 1), 4&
            fkey(i) = fkey(i - 7) Xor (Te4(s(0)) And &HFF000000) _
                                  Xor (Te4(s(3)) And &HFF0000) _
                                  Xor (Te4(s(2)) And &HFF00&) _
                                  Xor (Te4(s(1)) And &HFF&) _
                                  Xor rco(j)
            fkey(i + 1) = fkey(i - 6) Xor fkey(i)
            fkey(i + 2) = fkey(i - 5) Xor fkey(i + 1)
            fkey(i + 3) = fkey(i - 4) Xor fkey(i + 2)
            CopyMemory s(0), fkey(i + 3), 4&
            fkey(i + 4) = fkey(i - 3) Xor (Te4(s(3)) And &HFF000000) _
                                      Xor (Te4(s(2)) And &HFF0000) _
                                      Xor (Te4(s(1)) And &HFF00&) _
                                      Xor (Te4(s(0)) And &HFF&)
            fkey(i + 5) = fkey(i - 2) Xor fkey(i + 4)
            fkey(i + 6) = fkey(i - 1) Xor fkey(i + 5)
            i = i + 7
        Next j
        Nr = 13
'Debug.Assert i = (Nr + 1) * 6
#End If
    Case 256
        i = 8
        j = 0
        CopyMemory fkey(0), Pass(0), 4& * i
        Do
            CopyMemory s(0), fkey(i - 1), 4&
            fkey(i) = fkey(i - 8) Xor (Te4(s(0)) And &HFF000000) _
                                  Xor (Te4(s(3)) And &HFF0000) _
                                  Xor (Te4(s(2)) And &HFF00&) _
                                  Xor (Te4(s(1)) And &HFF&) _
                                  Xor rco(j)
            fkey(i + 1) = fkey(i - 7) Xor fkey(i)
            If j = 10 Then Exit Do
            fkey(i + 2) = fkey(i - 6) Xor fkey(i + 1)
            fkey(i + 3) = fkey(i - 5) Xor fkey(i + 2)
            CopyMemory s(0), fkey(i + 3), 4&
            fkey(i + 4) = fkey(i - 4) Xor (Te4(s(3)) And &HFF000000) _
                                      Xor (Te4(s(2)) And &HFF0000) _
                                      Xor (Te4(s(1)) And &HFF00&) _
                                      Xor (Te4(s(0)) And &HFF&)
            fkey(i + 5) = fkey(i - 3) Xor fkey(i + 4)
            fkey(i + 6) = fkey(i - 2) Xor fkey(i + 5)
            fkey(i + 7) = fkey(i - 1) Xor fkey(i + 6)
            i = i + 8
            j = j + 1
        Loop
        Nr = 14
'Debug.Assert i + 2 = (Nr + 1) * 6
    Case Else
        Err.Raise 1, , "cRijndael.SetCipherKey - Illegal KeyBits Value"
        SetCipherKey = 1
        Exit Function
    End Select
    CreateDecryptionKeys 6

#If SUPPORT_LEVEL = 2 Then
    Case 224

    '224 bit block size
    Select Case KeyBits
    Case 128
        i = 4
        j = 0
        CopyMemory fkey(0), Pass(0), 4& * i
        Do
            CopyMemory s(0), fkey(i - 1), 4&
            fkey(i) = fkey(i - 4) Xor (Te4(s(0)) And &HFF000000) _
                                  Xor (Te4(s(3)) And &HFF0000) _
                                  Xor (Te4(s(2)) And &HFF00&) _
                                  Xor (Te4(s(1)) And &HFF&) _
                                  Xor rco(j)
            fkey(i + 1) = fkey(i - 3) Xor fkey(i)
            If j = 23 Then Exit Do
            fkey(i + 2) = fkey(i - 2) Xor fkey(i + 1)
            fkey(i + 3) = fkey(i - 1) Xor fkey(i + 2)
            i = i + 4
            j = j + 1
        Loop
        Nr = 13
'Debug.Assert i + 2 = (Nr + 1) * 7
    Case 160
        i = 5
        j = 0
        CopyMemory fkey(0), Pass(0), 4& * i
        Do
            CopyMemory s(0), fkey(i - 1), 4&
            fkey(i) = fkey(i - 5) Xor (Te4(s(0)) And &HFF000000) _
                                  Xor (Te4(s(3)) And &HFF0000) _
                                  Xor (Te4(s(2)) And &HFF00&) _
                                  Xor (Te4(s(1)) And &HFF&) _
                                  Xor rco(j)
            fkey(i + 1) = fkey(i - 4) Xor fkey(i)
            fkey(i + 2) = fkey(i - 3) Xor fkey(i + 1)
            If j = 18 Then Exit Do
            fkey(i + 3) = fkey(i - 2) Xor fkey(i + 2)
            fkey(i + 4) = fkey(i - 1) Xor fkey(i + 3)
            i = i + 5
            j = j + 1
        Loop
        Nr = 13
'Debug.Assert i + 3 = (Nr + 1) * 7
    Case 192
        i = 6
        j = 0
        CopyMemory fkey(0), Pass(0), 4& * i
        Do
            CopyMemory s(0), fkey(i - 1), 4&
            fkey(i) = fkey(i - 6) Xor (Te4(s(0)) And &HFF000000) _
                                  Xor (Te4(s(3)) And &HFF0000) _
                                  Xor (Te4(s(2)) And &HFF00&) _
                                  Xor (Te4(s(1)) And &HFF&) _
                                  Xor rco(j)
            fkey(i + 1) = fkey(i - 5) Xor fkey(i)
            If j = 15 Then Exit Do
            fkey(i + 2) = fkey(i - 4) Xor fkey(i + 1)
            fkey(i + 3) = fkey(i - 3) Xor fkey(i + 2)
            fkey(i + 4) = fkey(i - 2) Xor fkey(i + 3)
            fkey(i + 5) = fkey(i - 1) Xor fkey(i + 4)
            i = i + 6
            j = j + 1
        Loop
        Nr = 13
'Debug.Assert i + 2 = (Nr + 1) * 7
    Case 224
        i = 7
        CopyMemory fkey(0), Pass(0), 4& * i
        For j = 0 To 12
            CopyMemory s(0), fkey(i - 1), 4&
            fkey(i) = fkey(i - 7) Xor (Te4(s(0)) And &HFF000000) _
                                  Xor (Te4(s(3)) And &HFF0000) _
                                  Xor (Te4(s(2)) And &HFF00&) _
                                  Xor (Te4(s(1)) And &HFF&) _
                                  Xor rco(j)
            fkey(i + 1) = fkey(i - 6) Xor fkey(i)
            fkey(i + 2) = fkey(i - 5) Xor fkey(i + 1)
            fkey(i + 3) = fkey(i - 4) Xor fkey(i + 2)
            CopyMemory s(0), fkey(i + 3), 4&
            fkey(i + 4) = fkey(i - 3) Xor (Te4(s(3)) And &HFF000000) _
                                      Xor (Te4(s(2)) And &HFF0000) _
                                      Xor (Te4(s(1)) And &HFF00&) _
                                      Xor (Te4(s(0)) And &HFF&)
            fkey(i + 5) = fkey(i - 2) Xor fkey(i + 4)
            fkey(i + 6) = fkey(i - 1) Xor fkey(i + 5)
            i = i + 7
        Next j
        Nr = 13
'Debug.Assert i = (Nr + 1) * 7
    Case 256
        i = 8
        j = 0
        CopyMemory fkey(0), Pass(0), 4& * i
        Do
            CopyMemory s(0), fkey(i - 1), 4&
            fkey(i) = fkey(i - 8) Xor (Te4(s(0)) And &HFF000000) _
                                  Xor (Te4(s(3)) And &HFF0000) _
                                  Xor (Te4(s(2)) And &HFF00&) _
                                  Xor (Te4(s(1)) And &HFF&) _
                                  Xor rco(j)
            If j = 12 Then Exit Do
            fkey(i + 1) = fkey(i - 7) Xor fkey(i)
            fkey(i + 2) = fkey(i - 6) Xor fkey(i + 1)
            fkey(i + 3) = fkey(i - 5) Xor fkey(i + 2)
            CopyMemory s(0), fkey(i + 3), 4&
            fkey(i + 4) = fkey(i - 4) Xor (Te4(s(3)) And &HFF000000) _
                                      Xor (Te4(s(2)) And &HFF0000) _
                                      Xor (Te4(s(1)) And &HFF00&) _
                                      Xor (Te4(s(0)) And &HFF&)
            fkey(i + 5) = fkey(i - 3) Xor fkey(i + 4)
            fkey(i + 6) = fkey(i - 2) Xor fkey(i + 5)
            fkey(i + 7) = fkey(i - 1) Xor fkey(i + 6)
            i = i + 8
            j = j + 1
        Loop
        Nr = 14
'Debug.Assert i + 1 = (Nr + 1) * 7
    Case Else
        Err.Raise 1, , "cRijndael.SetCipherKey - Illegal KeyBits Value"
        SetCipherKey = 1
        Exit Function
    End Select
    CreateDecryptionKeys 7
#End If

    Case 256

    '256 bit block size
    Select Case KeyBits
    Case 128
        i = 4
        CopyMemory fkey(0), Pass(0), 4& * i
        For j = 0 To 28
            CopyMemory s(0), fkey(i - 1), 4&
            fkey(i) = fkey(i - 4) Xor (Te4(s(0)) And &HFF000000) _
                                  Xor (Te4(s(3)) And &HFF0000) _
                                  Xor (Te4(s(2)) And &HFF00&) _
                                  Xor (Te4(s(1)) And &HFF&) _
                                  Xor rco(j)
            fkey(i + 1) = fkey(i - 3) Xor fkey(i)
            fkey(i + 2) = fkey(i - 2) Xor fkey(i + 1)
            fkey(i + 3) = fkey(i - 1) Xor fkey(i + 2)
            i = i + 4
        Next j
        Nr = 14
'Debug.Assert i = (Nr + 1) * 8
#If SUPPORT_LEVEL = 2 Then
    Case 160
        i = 5
        CopyMemory fkey(0), Pass(0), 4& * i
        For j = 0 To 22
            CopyMemory s(0), fkey(i - 1), 4&
            fkey(i) = fkey(i - 5) Xor (Te4(s(0)) And &HFF000000) _
                                  Xor (Te4(s(3)) And &HFF0000) _
                                  Xor (Te4(s(2)) And &HFF00&) _
                                  Xor (Te4(s(1)) And &HFF&) _
                                  Xor rco(j)
            fkey(i + 1) = fkey(i - 4) Xor fkey(i)
            fkey(i + 2) = fkey(i - 3) Xor fkey(i + 1)
            fkey(i + 3) = fkey(i - 2) Xor fkey(i + 2)
            fkey(i + 4) = fkey(i - 1) Xor fkey(i + 3)
            i = i + 5
        Next j
        Nr = 14
'Debug.Assert i = (Nr + 1) * 8
#End If
    Case 192
        i = 6
        CopyMemory fkey(0), Pass(0), 4& * i
        For j = 0 To 18
            CopyMemory s(0), fkey(i - 1), 4&
            fkey(i) = fkey(i - 6) Xor (Te4(s(0)) And &HFF000000) _
                                  Xor (Te4(s(3)) And &HFF0000) _
                                  Xor (Te4(s(2)) And &HFF00&) _
                                  Xor (Te4(s(1)) And &HFF&) _
                                  Xor rco(j)
            fkey(i + 1) = fkey(i - 5) Xor fkey(i)
            fkey(i + 2) = fkey(i - 4) Xor fkey(i + 1)
            fkey(i + 3) = fkey(i - 3) Xor fkey(i + 2)
            fkey(i + 4) = fkey(i - 2) Xor fkey(i + 3)
            fkey(i + 5) = fkey(i - 1) Xor fkey(i + 4)
            i = i + 6
        Next j
        Nr = 14
'Debug.Assert i = (Nr + 1) * 8
#If SUPPORT_LEVEL = 2 Then
    Case 224
        i = 7
        j = 0
        CopyMemory fkey(0), Pass(0), 4& * i
        Do
            CopyMemory s(0), fkey(i - 1), 4&
            fkey(i) = fkey(i - 7) Xor (Te4(s(0)) And &HFF000000) _
                                  Xor (Te4(s(3)) And &HFF0000) _
                                  Xor (Te4(s(2)) And &HFF00&) _
                                  Xor (Te4(s(1)) And &HFF&) _
                                  Xor rco(j)
            If j = 16 Then Exit Do
            fkey(i + 1) = fkey(i - 6) Xor fkey(i)
            fkey(i + 2) = fkey(i - 5) Xor fkey(i + 1)
            fkey(i + 3) = fkey(i - 4) Xor fkey(i + 2)
            CopyMemory s(0), fkey(i + 3), 4&
            fkey(i + 4) = fkey(i - 3) Xor (Te4(s(3)) And &HFF000000) _
                                      Xor (Te4(s(2)) And &HFF0000) _
                                      Xor (Te4(s(1)) And &HFF00&) _
                                      Xor (Te4(s(0)) And &HFF&)
            fkey(i + 5) = fkey(i - 2) Xor fkey(i + 4)
            fkey(i + 6) = fkey(i - 1) Xor fkey(i + 5)
            i = i + 7
            j = j + 1
        Loop
        Nr = 14
'Debug.Assert i + 1 = (Nr + 1) * 8
#End If
    Case 256
        i = 8
        CopyMemory fkey(0), Pass(0), 4& * i
        For j = 0 To 13
            CopyMemory s(0), fkey(i - 1), 4&
            fkey(i) = fkey(i - 8) Xor (Te4(s(0)) And &HFF000000) _
                                  Xor (Te4(s(3)) And &HFF0000) _
                                  Xor (Te4(s(2)) And &HFF00&) _
                                  Xor (Te4(s(1)) And &HFF&) _
                                  Xor rco(j)
            fkey(i + 1) = fkey(i - 7) Xor fkey(i)
            fkey(i + 2) = fkey(i - 6) Xor fkey(i + 1)
            fkey(i + 3) = fkey(i - 5) Xor fkey(i + 2)
            CopyMemory s(0), fkey(i + 3), 4&
            fkey(i + 4) = fkey(i - 4) Xor (Te4(s(3)) And &HFF000000) _
                                      Xor (Te4(s(2)) And &HFF0000) _
                                      Xor (Te4(s(1)) And &HFF00&) _
                                      Xor (Te4(s(0)) And &HFF&)
            fkey(i + 5) = fkey(i - 3) Xor fkey(i + 4)
            fkey(i + 6) = fkey(i - 2) Xor fkey(i + 5)
            fkey(i + 7) = fkey(i - 1) Xor fkey(i + 6)
            i = i + 8
        Next j
        Nr = 14
'Debug.Assert i = (Nr + 1) * 8
    Case Else
        Err.Raise 1, , "cRijndael.SetCipherKey - Illegal KeyBits Value"
        SetCipherKey = 1
        Exit Function
    End Select
    CreateDecryptionKeys 8

    Case Else
        Err.Raise 1, , "cRijndael.SetCipherKey - Illegal BlockBits Value"
        SetCipherKey = 1
        Exit Function
    End Select
#End If
End Function

#If SUPPORT_LEVEL Then
Private Function SetCipherKeyString(PassPhrase As String, KeyBits As Long, BlockBits As Long) As Long
    Dim Pass() As Byte

    Pass = StrConv(PassPhrase, vbFromUnicode)
    ReDim Preserve Pass(31)
    SetCipherKeyString = SetCipherKey(Pass, KeyBits, BlockBits)
End Function
#Else
Private Function SetCipherKeyString(PassPhrase As String, KeyBits As Long) As Long
    Dim Pass() As Byte

    Pass = StrConv(PassPhrase, vbFromUnicode)
    ReDim Preserve Pass(31)
    SetCipherKeyString = SetCipherKey(Pass, KeyBits)
End Function
#End If


'Encrypt a 128 bit block.  plaintext(p ... p+15) is input, ciphertext(q ... q+15) is output.
'plaintext and ciphertext can be the same array.  Will crash if ciphertext(q ... q+15) is not allocated.
Private Sub BlockEncrypt(plaintext() As Byte, ciphertext() As Byte, p As Long, q As Long)
    Dim i     As Long
    Dim k     As Long
    Dim t0    As Long
    Dim t1    As Long
    Dim t2    As Long
    Dim t3    As Long
    Dim s(15) As Byte

    CopyMemory t0, plaintext(p + 0), 4&
    CopyMemory t1, plaintext(p + 4), 4&
    CopyMemory t2, plaintext(p + 8), 4&
    CopyMemory t3, plaintext(p + 12), 4&
    t0 = t0 Xor fkey(0)
    t1 = t1 Xor fkey(1)
    t2 = t2 Xor fkey(2)
    t3 = t3 Xor fkey(3)
    k = 4

    For i = 1 To Nr - 1 'Nr is number of rounds
        CopyMemory s(0), t0, 4&
        CopyMemory s(4), t1, 4&
        CopyMemory s(8), t2, 4&
        CopyMemory s(12), t3, 4&
        t0 = Te0(s(0)) Xor Te1(s(5)) Xor Te2(s(10)) Xor Te3(s(15)) Xor fkey(k + 0)
        t1 = Te0(s(4)) Xor Te1(s(9)) Xor Te2(s(14)) Xor Te3(s(3)) Xor fkey(k + 1)
        t2 = Te0(s(8)) Xor Te1(s(13)) Xor Te2(s(2)) Xor Te3(s(7)) Xor fkey(k + 2)
        t3 = Te0(s(12)) Xor Te1(s(1)) Xor Te2(s(6)) Xor Te3(s(11)) Xor fkey(k + 3)
        k = k + 4
    Next i

    'Final round
    CopyMemory s(0), t0, 4&
    CopyMemory s(4), t1, 4&
    CopyMemory s(8), t2, 4&
    CopyMemory s(12), t3, 4&
    t0 = (Te4(s(0)) And &HFF&) Xor (Te4(s(5)) And &HFF00&) Xor (Te4(s(10)) And &HFF0000) Xor (Te4(s(15)) And &HFF000000) Xor fkey(k + 0)
    t1 = (Te4(s(4)) And &HFF&) Xor (Te4(s(9)) And &HFF00&) Xor (Te4(s(14)) And &HFF0000) Xor (Te4(s(3)) And &HFF000000) Xor fkey(k + 1)
    t2 = (Te4(s(8)) And &HFF&) Xor (Te4(s(13)) And &HFF00&) Xor (Te4(s(2)) And &HFF0000) Xor (Te4(s(7)) And &HFF000000) Xor fkey(k + 2)
    t3 = (Te4(s(12)) And &HFF&) Xor (Te4(s(1)) And &HFF00&) Xor (Te4(s(6)) And &HFF0000) Xor (Te4(s(11)) And &HFF000000) Xor fkey(k + 3)
    CopyMemory ciphertext(q + 0), t0, 4&
    CopyMemory ciphertext(q + 4), t1, 4&
    CopyMemory ciphertext(q + 8), t2, 4&
    CopyMemory ciphertext(q + 12), t3, 4&
End Sub

'Decrypt a 128 bit block.  ciphertext(q ... q+15) is input, plaintext(p ... p+15) is output.
'plaintext and ciphertext can be the same array.  Will crash if plaintext(p ... p+15) is not allocated.
Private Sub BlockDecrypt(plaintext() As Byte, ciphertext() As Byte, p As Long, q As Long)
    Dim i     As Long
    Dim k     As Long
    Dim t0    As Long
    Dim t1    As Long
    Dim t2    As Long
    Dim t3    As Long
    Dim s(15) As Byte

    CopyMemory t0, ciphertext(q + 0), 4&
    CopyMemory t1, ciphertext(q + 4), 4&
    CopyMemory t2, ciphertext(q + 8), 4&
    CopyMemory t3, ciphertext(q + 12), 4&
    t0 = t0 Xor rkey(0)
    t1 = t1 Xor rkey(1)
    t2 = t2 Xor rkey(2)
    t3 = t3 Xor rkey(3)
    k = 4

    For i = 1 To Nr - 1 'Nr is number of rounds
        CopyMemory s(0), t0, 4&
        CopyMemory s(4), t1, 4&
        CopyMemory s(8), t2, 4&
        CopyMemory s(12), t3, 4&
        t0 = Td0(s(0)) Xor Td1(s(13)) Xor Td2(s(10)) Xor Td3(s(7)) Xor rkey(k + 0)
        t1 = Td0(s(4)) Xor Td1(s(1)) Xor Td2(s(14)) Xor Td3(s(11)) Xor rkey(k + 1)
        t2 = Td0(s(8)) Xor Td1(s(5)) Xor Td2(s(2)) Xor Td3(s(15)) Xor rkey(k + 2)
        t3 = Td0(s(12)) Xor Td1(s(9)) Xor Td2(s(6)) Xor Td3(s(3)) Xor rkey(k + 3)
        k = k + 4
    Next i

    'Final round
    CopyMemory s(0), t0, 4&
    CopyMemory s(4), t1, 4&
    CopyMemory s(8), t2, 4&
    CopyMemory s(12), t3, 4&
    t0 = (Td4(s(0)) And &HFF&) Xor (Td4(s(13)) And &HFF00&) Xor (Td4(s(10)) And &HFF0000) Xor (Td4(s(7)) And &HFF000000) Xor rkey(k + 0)
    t1 = (Td4(s(4)) And &HFF&) Xor (Td4(s(1)) And &HFF00&) Xor (Td4(s(14)) And &HFF0000) Xor (Td4(s(11)) And &HFF000000) Xor rkey(k + 1)
    t2 = (Td4(s(8)) And &HFF&) Xor (Td4(s(5)) And &HFF00&) Xor (Td4(s(2)) And &HFF0000) Xor (Td4(s(15)) And &HFF000000) Xor rkey(k + 2)
    t3 = (Td4(s(12)) And &HFF&) Xor (Td4(s(9)) And &HFF00&) Xor (Td4(s(6)) And &HFF0000) Xor (Td4(s(3)) And &HFF000000) Xor rkey(k + 3)
    CopyMemory plaintext(p + 0), t0, 4&
    CopyMemory plaintext(p + 4), t1, 4&
    CopyMemory plaintext(p + 8), t2, 4&
    CopyMemory plaintext(p + 12), t3, 4&
End Sub

#If SUPPORT_LEVEL Then
#If SUPPORT_LEVEL = 2 Then
'Encrypt a 160 bit block.  plaintext(p ... p+19) is input, ciphertext(q ... q+19) is output.
'plaintext and ciphertext can be the same array.  Will crash if ciphertext(q ... q+19) is not allocated.
Private Sub Block160Encrypt(plaintext() As Byte, ciphertext() As Byte, p As Long, q As Long)
    Dim i     As Long
    Dim k     As Long
    Dim t0    As Long
    Dim t1    As Long
    Dim t2    As Long
    Dim t3    As Long
    Dim t4    As Long
    Dim s(19) As Byte

    CopyMemory t0, plaintext(p + 0), 4&
    CopyMemory t1, plaintext(p + 4), 4&
    CopyMemory t2, plaintext(p + 8), 4&
    CopyMemory t3, plaintext(p + 12), 4&
    CopyMemory t4, plaintext(p + 16), 4&
    t0 = t0 Xor fkey(0)
    t1 = t1 Xor fkey(1)
    t2 = t2 Xor fkey(2)
    t3 = t3 Xor fkey(3)
    t4 = t4 Xor fkey(4)
    k = 5

    For i = 1 To Nr - 1 'Nr is number of rounds
        CopyMemory s(0), t0, 4&
        CopyMemory s(4), t1, 4&
        CopyMemory s(8), t2, 4&
        CopyMemory s(12), t3, 4&
        CopyMemory s(16), t4, 4&
        t0 = Te0(s(0)) Xor Te1(s(5)) Xor Te2(s(10)) Xor Te3(s(15)) Xor fkey(k + 0)
        t1 = Te0(s(4)) Xor Te1(s(9)) Xor Te2(s(14)) Xor Te3(s(19)) Xor fkey(k + 1)
        t2 = Te0(s(8)) Xor Te1(s(13)) Xor Te2(s(18)) Xor Te3(s(3)) Xor fkey(k + 2)
        t3 = Te0(s(12)) Xor Te1(s(17)) Xor Te2(s(2)) Xor Te3(s(7)) Xor fkey(k + 3)
        t4 = Te0(s(16)) Xor Te1(s(1)) Xor Te2(s(6)) Xor Te3(s(11)) Xor fkey(k + 4)
        k = k + 5
    Next i

    'Final round
    CopyMemory s(0), t0, 4&
    CopyMemory s(4), t1, 4&
    CopyMemory s(8), t2, 4&
    CopyMemory s(12), t3, 4&
    CopyMemory s(16), t4, 4&
    t0 = (Te4(s(0)) And &HFF&) Xor (Te4(s(5)) And &HFF00&) Xor (Te4(s(10)) And &HFF0000) Xor (Te4(s(15)) And &HFF000000) Xor fkey(k + 0)
    t1 = (Te4(s(4)) And &HFF&) Xor (Te4(s(9)) And &HFF00&) Xor (Te4(s(14)) And &HFF0000) Xor (Te4(s(19)) And &HFF000000) Xor fkey(k + 1)
    t2 = (Te4(s(8)) And &HFF&) Xor (Te4(s(13)) And &HFF00&) Xor (Te4(s(18)) And &HFF0000) Xor (Te4(s(3)) And &HFF000000) Xor fkey(k + 2)
    t3 = (Te4(s(12)) And &HFF&) Xor (Te4(s(17)) And &HFF00&) Xor (Te4(s(2)) And &HFF0000) Xor (Te4(s(7)) And &HFF000000) Xor fkey(k + 3)
    t4 = (Te4(s(16)) And &HFF&) Xor (Te4(s(1)) And &HFF00&) Xor (Te4(s(6)) And &HFF0000) Xor (Te4(s(11)) And &HFF000000) Xor fkey(k + 4)
    CopyMemory ciphertext(q + 0), t0, 4&
    CopyMemory ciphertext(q + 4), t1, 4&
    CopyMemory ciphertext(q + 8), t2, 4&
    CopyMemory ciphertext(q + 12), t3, 4&
    CopyMemory ciphertext(q + 16), t4, 4&
End Sub

'Decrypt a 160 bit block.  ciphertext(q ... q+19) is input, plaintext(p ... p+19) is output.
'plaintext and ciphertext can be the same array.  Will crash if plaintext(p ... p+19) is not allocated.
Private Sub Block160Decrypt(plaintext() As Byte, ciphertext() As Byte, p As Long, q As Long)
    Dim i     As Long
    Dim k     As Long
    Dim t0    As Long
    Dim t1    As Long
    Dim t2    As Long
    Dim t3    As Long
    Dim t4    As Long
    Dim s(19) As Byte

    CopyMemory t0, ciphertext(q + 0), 4&
    CopyMemory t1, ciphertext(q + 4), 4&
    CopyMemory t2, ciphertext(q + 8), 4&
    CopyMemory t3, ciphertext(q + 12), 4&
    CopyMemory t4, ciphertext(q + 16), 4&
    t0 = t0 Xor rkey(0)
    t1 = t1 Xor rkey(1)
    t2 = t2 Xor rkey(2)
    t3 = t3 Xor rkey(3)
    t4 = t4 Xor rkey(4)
    k = 5

    For i = 1 To Nr - 1 'Nr is number of rounds
        CopyMemory s(0), t0, 4&
        CopyMemory s(4), t1, 4&
        CopyMemory s(8), t2, 4&
        CopyMemory s(12), t3, 4&
        CopyMemory s(16), t4, 4&
        t0 = Td0(s(0)) Xor Td1(s(17)) Xor Td2(s(14)) Xor Td3(s(11)) Xor rkey(k + 0)
        t1 = Td0(s(4)) Xor Td1(s(1)) Xor Td2(s(18)) Xor Td3(s(15)) Xor rkey(k + 1)
        t2 = Td0(s(8)) Xor Td1(s(5)) Xor Td2(s(2)) Xor Td3(s(19)) Xor rkey(k + 2)
        t3 = Td0(s(12)) Xor Td1(s(9)) Xor Td2(s(6)) Xor Td3(s(3)) Xor rkey(k + 3)
        t4 = Td0(s(16)) Xor Td1(s(13)) Xor Td2(s(10)) Xor Td3(s(7)) Xor rkey(k + 4)
        k = k + 5
    Next i

    'Final round
    CopyMemory s(0), t0, 4&
    CopyMemory s(4), t1, 4&
    CopyMemory s(8), t2, 4&
    CopyMemory s(12), t3, 4&
    CopyMemory s(16), t4, 4&
    t0 = (Td4(s(0)) And &HFF&) Xor (Td4(s(17)) And &HFF00&) Xor (Td4(s(14)) And &HFF0000) Xor (Td4(s(11)) And &HFF000000) Xor rkey(k + 0)
    t1 = (Td4(s(4)) And &HFF&) Xor (Td4(s(1)) And &HFF00&) Xor (Td4(s(18)) And &HFF0000) Xor (Td4(s(15)) And &HFF000000) Xor rkey(k + 1)
    t2 = (Td4(s(8)) And &HFF&) Xor (Td4(s(5)) And &HFF00&) Xor (Td4(s(2)) And &HFF0000) Xor (Td4(s(19)) And &HFF000000) Xor rkey(k + 2)
    t3 = (Td4(s(12)) And &HFF&) Xor (Td4(s(9)) And &HFF00&) Xor (Td4(s(6)) And &HFF0000) Xor (Td4(s(3)) And &HFF000000) Xor rkey(k + 3)
    t4 = (Td4(s(16)) And &HFF&) Xor (Td4(s(13)) And &HFF00&) Xor (Td4(s(10)) And &HFF0000) Xor (Td4(s(7)) And &HFF000000) Xor rkey(k + 4)
    CopyMemory plaintext(p + 0), t0, 4&
    CopyMemory plaintext(p + 4), t1, 4&
    CopyMemory plaintext(p + 8), t2, 4&
    CopyMemory plaintext(p + 12), t3, 4&
    CopyMemory plaintext(p + 16), t4, 4&
End Sub
#End If

'Encrypt a 192 bit block.  plaintext(p ... p+23) is input, ciphertext(q ... q+23) is output.
'plaintext and ciphertext can be the same array.  Will crash if ciphertext(q ... q+23) is not allocated.
Private Sub Block192Encrypt(plaintext() As Byte, ciphertext() As Byte, p As Long, q As Long)
    Dim i     As Long
    Dim k     As Long
    Dim t0    As Long
    Dim t1    As Long
    Dim t2    As Long
    Dim t3    As Long
    Dim t4    As Long
    Dim t5    As Long
    Dim s(23) As Byte

    CopyMemory t0, plaintext(p + 0), 4&
    CopyMemory t1, plaintext(p + 4), 4&
    CopyMemory t2, plaintext(p + 8), 4&
    CopyMemory t3, plaintext(p + 12), 4&
    CopyMemory t4, plaintext(p + 16), 4&
    CopyMemory t5, plaintext(p + 20), 4&
    t0 = t0 Xor fkey(0)
    t1 = t1 Xor fkey(1)
    t2 = t2 Xor fkey(2)
    t3 = t3 Xor fkey(3)
    t4 = t4 Xor fkey(4)
    t5 = t5 Xor fkey(5)
    k = 6

    For i = 1 To Nr - 1 'Nr is number of rounds
        CopyMemory s(0), t0, 4&
        CopyMemory s(4), t1, 4&
        CopyMemory s(8), t2, 4&
        CopyMemory s(12), t3, 4&
        CopyMemory s(16), t4, 4&
        CopyMemory s(20), t5, 4&
        t0 = Te0(s(0)) Xor Te1(s(5)) Xor Te2(s(10)) Xor Te3(s(15)) Xor fkey(k + 0)
        t1 = Te0(s(4)) Xor Te1(s(9)) Xor Te2(s(14)) Xor Te3(s(19)) Xor fkey(k + 1)
        t2 = Te0(s(8)) Xor Te1(s(13)) Xor Te2(s(18)) Xor Te3(s(23)) Xor fkey(k + 2)
        t3 = Te0(s(12)) Xor Te1(s(17)) Xor Te2(s(22)) Xor Te3(s(3)) Xor fkey(k + 3)
        t4 = Te0(s(16)) Xor Te1(s(21)) Xor Te2(s(2)) Xor Te3(s(7)) Xor fkey(k + 4)
        t5 = Te0(s(20)) Xor Te1(s(1)) Xor Te2(s(6)) Xor Te3(s(11)) Xor fkey(k + 5)
        k = k + 6
    Next i

    'Final round
    CopyMemory s(0), t0, 4&
    CopyMemory s(4), t1, 4&
    CopyMemory s(8), t2, 4&
    CopyMemory s(12), t3, 4&
    CopyMemory s(16), t4, 4&
    CopyMemory s(20), t5, 4&
    t0 = (Te4(s(0)) And &HFF&) Xor (Te4(s(5)) And &HFF00&) Xor (Te4(s(10)) And &HFF0000) Xor (Te4(s(15)) And &HFF000000) Xor fkey(k + 0)
    t1 = (Te4(s(4)) And &HFF&) Xor (Te4(s(9)) And &HFF00&) Xor (Te4(s(14)) And &HFF0000) Xor (Te4(s(19)) And &HFF000000) Xor fkey(k + 1)
    t2 = (Te4(s(8)) And &HFF&) Xor (Te4(s(13)) And &HFF00&) Xor (Te4(s(18)) And &HFF0000) Xor (Te4(s(23)) And &HFF000000) Xor fkey(k + 2)
    t3 = (Te4(s(12)) And &HFF&) Xor (Te4(s(17)) And &HFF00&) Xor (Te4(s(22)) And &HFF0000) Xor (Te4(s(3)) And &HFF000000) Xor fkey(k + 3)
    t4 = (Te4(s(16)) And &HFF&) Xor (Te4(s(21)) And &HFF00&) Xor (Te4(s(2)) And &HFF0000) Xor (Te4(s(7)) And &HFF000000) Xor fkey(k + 4)
    t5 = (Te4(s(20)) And &HFF&) Xor (Te4(s(1)) And &HFF00&) Xor (Te4(s(6)) And &HFF0000) Xor (Te4(s(11)) And &HFF000000) Xor fkey(k + 5)

    CopyMemory ciphertext(q + 0), t0, 4&
    CopyMemory ciphertext(q + 4), t1, 4&
    CopyMemory ciphertext(q + 8), t2, 4&
    CopyMemory ciphertext(q + 12), t3, 4&
    CopyMemory ciphertext(q + 16), t4, 4&
    CopyMemory ciphertext(q + 20), t5, 4&
End Sub

'Decrypt a 192 bit block.  ciphertext(q ... q+23) is input, plaintext(p ... p+23) is output.
'plaintext and ciphertext can be the same array.  Will crash if plaintext(p ... p+23) is not allocated.
Private Sub Block192Decrypt(plaintext() As Byte, ciphertext() As Byte, p As Long, q As Long)
    Dim i     As Long
    Dim k     As Long
    Dim t0    As Long
    Dim t1    As Long
    Dim t2    As Long
    Dim t3    As Long
    Dim t4    As Long
    Dim t5    As Long
    Dim s(23) As Byte

    CopyMemory t0, ciphertext(q + 0), 4&
    CopyMemory t1, ciphertext(q + 4), 4&
    CopyMemory t2, ciphertext(q + 8), 4&
    CopyMemory t3, ciphertext(q + 12), 4&
    CopyMemory t4, ciphertext(q + 16), 4&
    CopyMemory t5, ciphertext(q + 20), 4&
    t0 = t0 Xor rkey(0)
    t1 = t1 Xor rkey(1)
    t2 = t2 Xor rkey(2)
    t3 = t3 Xor rkey(3)
    t4 = t4 Xor rkey(4)
    t5 = t5 Xor rkey(5)
    k = 6

    For i = 1 To Nr - 1 'Nr is number of rounds
        CopyMemory s(0), t0, 4&
        CopyMemory s(4), t1, 4&
        CopyMemory s(8), t2, 4&
        CopyMemory s(12), t3, 4&
        CopyMemory s(16), t4, 4&
        CopyMemory s(20), t5, 4&
        t0 = Td0(s(0)) Xor Td1(s(21)) Xor Td2(s(18)) Xor Td3(s(15)) Xor rkey(k + 0)
        t1 = Td0(s(4)) Xor Td1(s(1)) Xor Td2(s(22)) Xor Td3(s(19)) Xor rkey(k + 1)
        t2 = Td0(s(8)) Xor Td1(s(5)) Xor Td2(s(2)) Xor Td3(s(23)) Xor rkey(k + 2)
        t3 = Td0(s(12)) Xor Td1(s(9)) Xor Td2(s(6)) Xor Td3(s(3)) Xor rkey(k + 3)
        t4 = Td0(s(16)) Xor Td1(s(13)) Xor Td2(s(10)) Xor Td3(s(7)) Xor rkey(k + 4)
        t5 = Td0(s(20)) Xor Td1(s(17)) Xor Td2(s(14)) Xor Td3(s(11)) Xor rkey(k + 5)
        k = k + 6
    Next i

    'Final round
    CopyMemory s(0), t0, 4&
    CopyMemory s(4), t1, 4&
    CopyMemory s(8), t2, 4&
    CopyMemory s(12), t3, 4&
    CopyMemory s(16), t4, 4&
    CopyMemory s(20), t5, 4&
    t0 = (Td4(s(0)) And &HFF&) Xor (Td4(s(21)) And &HFF00&) Xor (Td4(s(18)) And &HFF0000) Xor (Td4(s(15)) And &HFF000000) Xor rkey(k + 0)
    t1 = (Td4(s(4)) And &HFF&) Xor (Td4(s(1)) And &HFF00&) Xor (Td4(s(22)) And &HFF0000) Xor (Td4(s(19)) And &HFF000000) Xor rkey(k + 1)
    t2 = (Td4(s(8)) And &HFF&) Xor (Td4(s(5)) And &HFF00&) Xor (Td4(s(2)) And &HFF0000) Xor (Td4(s(23)) And &HFF000000) Xor rkey(k + 2)
    t3 = (Td4(s(12)) And &HFF&) Xor (Td4(s(9)) And &HFF00&) Xor (Td4(s(6)) And &HFF0000) Xor (Td4(s(3)) And &HFF000000) Xor rkey(k + 3)
    t4 = (Td4(s(16)) And &HFF&) Xor (Td4(s(13)) And &HFF00&) Xor (Td4(s(10)) And &HFF0000) Xor (Td4(s(7)) And &HFF000000) Xor rkey(k + 4)
    t5 = (Td4(s(20)) And &HFF&) Xor (Td4(s(17)) And &HFF00&) Xor (Td4(s(14)) And &HFF0000) Xor (Td4(s(11)) And &HFF000000) Xor rkey(k + 5)
    CopyMemory plaintext(p + 0), t0, 4&
    CopyMemory plaintext(p + 4), t1, 4&
    CopyMemory plaintext(p + 8), t2, 4&
    CopyMemory plaintext(p + 12), t3, 4&
    CopyMemory plaintext(p + 16), t4, 4&
    CopyMemory plaintext(p + 20), t5, 4&
End Sub

#If SUPPORT_LEVEL = 2 Then
'Encrypt a 224 bit block.  plaintext(p ... p+27) is input, ciphertext(q ... q+27) is output.
'plaintext and ciphertext can be the same array.  Will crash if ciphertext(q ... q+27) is not allocated.
Private Sub Block224Encrypt(plaintext() As Byte, ciphertext() As Byte, p As Long, q As Long)
    Dim i     As Long
    Dim k     As Long
    Dim t0    As Long
    Dim t1    As Long
    Dim t2    As Long
    Dim t3    As Long
    Dim t4    As Long
    Dim t5    As Long
    Dim t6    As Long
    Dim s(27) As Byte

    CopyMemory t0, plaintext(p + 0), 4&
    CopyMemory t1, plaintext(p + 4), 4&
    CopyMemory t2, plaintext(p + 8), 4&
    CopyMemory t3, plaintext(p + 12), 4&
    CopyMemory t4, plaintext(p + 16), 4&
    CopyMemory t5, plaintext(p + 20), 4&
    CopyMemory t6, plaintext(p + 24), 4&
    t0 = t0 Xor fkey(0)
    t1 = t1 Xor fkey(1)
    t2 = t2 Xor fkey(2)
    t3 = t3 Xor fkey(3)
    t4 = t4 Xor fkey(4)
    t5 = t5 Xor fkey(5)
    t6 = t6 Xor fkey(6)
    k = 7

    For i = 1 To Nr - 1 'Nr is number of rounds
        CopyMemory s(0), t0, 4&
        CopyMemory s(4), t1, 4&
        CopyMemory s(8), t2, 4&
        CopyMemory s(12), t3, 4&
        CopyMemory s(16), t4, 4&
        CopyMemory s(20), t5, 4&
        CopyMemory s(24), t6, 4&
        t0 = Te0(s(0)) Xor Te1(s(5)) Xor Te2(s(10)) Xor Te3(s(19)) Xor fkey(k + 0)
        t1 = Te0(s(4)) Xor Te1(s(9)) Xor Te2(s(14)) Xor Te3(s(23)) Xor fkey(k + 1)
        t2 = Te0(s(8)) Xor Te1(s(13)) Xor Te2(s(18)) Xor Te3(s(27)) Xor fkey(k + 2)
        t3 = Te0(s(12)) Xor Te1(s(17)) Xor Te2(s(22)) Xor Te3(s(3)) Xor fkey(k + 3)
        t4 = Te0(s(16)) Xor Te1(s(21)) Xor Te2(s(26)) Xor Te3(s(7)) Xor fkey(k + 4)
        t5 = Te0(s(20)) Xor Te1(s(25)) Xor Te2(s(2)) Xor Te3(s(11)) Xor fkey(k + 5)
        t6 = Te0(s(24)) Xor Te1(s(1)) Xor Te2(s(6)) Xor Te3(s(15)) Xor fkey(k + 6)
        k = k + 7
    Next i

    'Final round
    CopyMemory s(0), t0, 4&
    CopyMemory s(4), t1, 4&
    CopyMemory s(8), t2, 4&
    CopyMemory s(12), t3, 4&
    CopyMemory s(16), t4, 4&
    CopyMemory s(20), t5, 4&
    CopyMemory s(24), t6, 4&
    t0 = (Te4(s(0)) And &HFF&) Xor (Te4(s(5)) And &HFF00&) Xor (Te4(s(10)) And &HFF0000) Xor (Te4(s(19)) And &HFF000000) Xor fkey(k + 0)
    t1 = (Te4(s(4)) And &HFF&) Xor (Te4(s(9)) And &HFF00&) Xor (Te4(s(14)) And &HFF0000) Xor (Te4(s(23)) And &HFF000000) Xor fkey(k + 1)
    t2 = (Te4(s(8)) And &HFF&) Xor (Te4(s(13)) And &HFF00&) Xor (Te4(s(18)) And &HFF0000) Xor (Te4(s(27)) And &HFF000000) Xor fkey(k + 2)
    t3 = (Te4(s(12)) And &HFF&) Xor (Te4(s(17)) And &HFF00&) Xor (Te4(s(22)) And &HFF0000) Xor (Te4(s(3)) And &HFF000000) Xor fkey(k + 3)
    t4 = (Te4(s(16)) And &HFF&) Xor (Te4(s(21)) And &HFF00&) Xor (Te4(s(26)) And &HFF0000) Xor (Te4(s(7)) And &HFF000000) Xor fkey(k + 4)
    t5 = (Te4(s(20)) And &HFF&) Xor (Te4(s(25)) And &HFF00&) Xor (Te4(s(2)) And &HFF0000) Xor (Te4(s(11)) And &HFF000000) Xor fkey(k + 5)
    t6 = (Te4(s(24)) And &HFF&) Xor (Te4(s(1)) And &HFF00&) Xor (Te4(s(6)) And &HFF0000) Xor (Te4(s(15)) And &HFF000000) Xor fkey(k + 6)
    CopyMemory ciphertext(q + 0), t0, 4&
    CopyMemory ciphertext(q + 4), t1, 4&
    CopyMemory ciphertext(q + 8), t2, 4&
    CopyMemory ciphertext(q + 12), t3, 4&
    CopyMemory ciphertext(q + 16), t4, 4&
    CopyMemory ciphertext(q + 20), t5, 4&
    CopyMemory ciphertext(q + 24), t6, 4&
End Sub

'Decrypt a 224 bit block.  ciphertext(q ... q+27) is input, plaintext(p ... p+27) is output.
'plaintext and ciphertext can be the same array.  Will crash if plaintext(p ... p+27) is not allocated.
Private Sub Block224Decrypt(plaintext() As Byte, ciphertext() As Byte, p As Long, q As Long)
    Dim i     As Long
    Dim k     As Long
    Dim t0    As Long
    Dim t1    As Long
    Dim t2    As Long
    Dim t3    As Long
    Dim t4    As Long
    Dim t5    As Long
    Dim t6    As Long
    Dim s(27) As Byte

    CopyMemory t0, ciphertext(q + 0), 4&
    CopyMemory t1, ciphertext(q + 4), 4&
    CopyMemory t2, ciphertext(q + 8), 4&
    CopyMemory t3, ciphertext(q + 12), 4&
    CopyMemory t4, ciphertext(q + 16), 4&
    CopyMemory t5, ciphertext(q + 20), 4&
    CopyMemory t6, ciphertext(q + 24), 4&
    t0 = t0 Xor rkey(0)
    t1 = t1 Xor rkey(1)
    t2 = t2 Xor rkey(2)
    t3 = t3 Xor rkey(3)
    t4 = t4 Xor rkey(4)
    t5 = t5 Xor rkey(5)
    t6 = t6 Xor rkey(6)
    k = 7

    For i = 1 To Nr - 1 'Nr is number of rounds
        CopyMemory s(0), t0, 4&
        CopyMemory s(4), t1, 4&
        CopyMemory s(8), t2, 4&
        CopyMemory s(12), t3, 4&
        CopyMemory s(16), t4, 4&
        CopyMemory s(20), t5, 4&
        CopyMemory s(24), t6, 4&
        t0 = Td0(s(0)) Xor Td1(s(25)) Xor Td2(s(22)) Xor Td3(s(15)) Xor rkey(k + 0)
        t1 = Td0(s(4)) Xor Td1(s(1)) Xor Td2(s(26)) Xor Td3(s(19)) Xor rkey(k + 1)
        t2 = Td0(s(8)) Xor Td1(s(5)) Xor Td2(s(2)) Xor Td3(s(23)) Xor rkey(k + 2)
        t3 = Td0(s(12)) Xor Td1(s(9)) Xor Td2(s(6)) Xor Td3(s(27)) Xor rkey(k + 3)
        t4 = Td0(s(16)) Xor Td1(s(13)) Xor Td2(s(10)) Xor Td3(s(3)) Xor rkey(k + 4)
        t5 = Td0(s(20)) Xor Td1(s(17)) Xor Td2(s(14)) Xor Td3(s(7)) Xor rkey(k + 5)
        t6 = Td0(s(24)) Xor Td1(s(21)) Xor Td2(s(18)) Xor Td3(s(11)) Xor rkey(k + 6)
        k = k + 7
    Next i

    'Final round
    CopyMemory s(0), t0, 4&
    CopyMemory s(4), t1, 4&
    CopyMemory s(8), t2, 4&
    CopyMemory s(12), t3, 4&
    CopyMemory s(16), t4, 4&
    CopyMemory s(20), t5, 4&
    CopyMemory s(24), t6, 4&
    t0 = (Td4(s(0)) And &HFF&) Xor (Td4(s(25)) And &HFF00&) Xor (Td4(s(22)) And &HFF0000) Xor (Td4(s(15)) And &HFF000000) Xor rkey(k + 0)
    t1 = (Td4(s(4)) And &HFF&) Xor (Td4(s(1)) And &HFF00&) Xor (Td4(s(26)) And &HFF0000) Xor (Td4(s(19)) And &HFF000000) Xor rkey(k + 1)
    t2 = (Td4(s(8)) And &HFF&) Xor (Td4(s(5)) And &HFF00&) Xor (Td4(s(2)) And &HFF0000) Xor (Td4(s(23)) And &HFF000000) Xor rkey(k + 2)
    t3 = (Td4(s(12)) And &HFF&) Xor (Td4(s(9)) And &HFF00&) Xor (Td4(s(6)) And &HFF0000) Xor (Td4(s(27)) And &HFF000000) Xor rkey(k + 3)
    t4 = (Td4(s(16)) And &HFF&) Xor (Td4(s(13)) And &HFF00&) Xor (Td4(s(10)) And &HFF0000) Xor (Td4(s(3)) And &HFF000000) Xor rkey(k + 4)
    t5 = (Td4(s(20)) And &HFF&) Xor (Td4(s(17)) And &HFF00&) Xor (Td4(s(14)) And &HFF0000) Xor (Td4(s(7)) And &HFF000000) Xor rkey(k + 5)
    t6 = (Td4(s(24)) And &HFF&) Xor (Td4(s(21)) And &HFF00&) Xor (Td4(s(18)) And &HFF0000) Xor (Td4(s(11)) And &HFF000000) Xor rkey(k + 6)
    CopyMemory plaintext(p + 0), t0, 4&
    CopyMemory plaintext(p + 4), t1, 4&
    CopyMemory plaintext(p + 8), t2, 4&
    CopyMemory plaintext(p + 12), t3, 4&
    CopyMemory plaintext(p + 16), t4, 4&
    CopyMemory plaintext(p + 20), t5, 4&
    CopyMemory plaintext(p + 24), t6, 4&
End Sub
#End If

'Encrypt a 256 bit block.  plaintext(p ... p+31) is input, ciphertext(q ... q+31) is output.
'plaintext and ciphertext can be the same array.  Will crash if ciphertext(q ... q+31) is not allocated.
Private Sub Block256Encrypt(plaintext() As Byte, ciphertext() As Byte, p As Long, q As Long)
    Dim i     As Long
    Dim k     As Long
    Dim t0    As Long
    Dim t1    As Long
    Dim t2    As Long
    Dim t3    As Long
    Dim t4    As Long
    Dim t5    As Long
    Dim t6    As Long
    Dim t7    As Long
    Dim s(31) As Byte

    CopyMemory t0, plaintext(p + 0), 4&
    CopyMemory t1, plaintext(p + 4), 4&
    CopyMemory t2, plaintext(p + 8), 4&
    CopyMemory t3, plaintext(p + 12), 4&
    CopyMemory t4, plaintext(p + 16), 4&
    CopyMemory t5, plaintext(p + 20), 4&
    CopyMemory t6, plaintext(p + 24), 4&
    CopyMemory t7, plaintext(p + 28), 4&
    t0 = t0 Xor fkey(0)
    t1 = t1 Xor fkey(1)
    t2 = t2 Xor fkey(2)
    t3 = t3 Xor fkey(3)
    t4 = t4 Xor fkey(4)
    t5 = t5 Xor fkey(5)
    t6 = t6 Xor fkey(6)
    t7 = t7 Xor fkey(7)
    k = 8

    For i = 1 To Nr - 1 'Nr is number of rounds
        CopyMemory s(0), t0, 4&
        CopyMemory s(4), t1, 4&
        CopyMemory s(8), t2, 4&
        CopyMemory s(12), t3, 4&
        CopyMemory s(16), t4, 4&
        CopyMemory s(20), t5, 4&
        CopyMemory s(24), t6, 4&
        CopyMemory s(28), t7, 4&
        t0 = Te0(s(0)) Xor Te1(s(5)) Xor Te2(s(14)) Xor Te3(s(19)) Xor fkey(k + 0)
        t1 = Te0(s(4)) Xor Te1(s(9)) Xor Te2(s(18)) Xor Te3(s(23)) Xor fkey(k + 1)
        t2 = Te0(s(8)) Xor Te1(s(13)) Xor Te2(s(22)) Xor Te3(s(27)) Xor fkey(k + 2)
        t3 = Te0(s(12)) Xor Te1(s(17)) Xor Te2(s(26)) Xor Te3(s(31)) Xor fkey(k + 3)
        t4 = Te0(s(16)) Xor Te1(s(21)) Xor Te2(s(30)) Xor Te3(s(3)) Xor fkey(k + 4)
        t5 = Te0(s(20)) Xor Te1(s(25)) Xor Te2(s(2)) Xor Te3(s(7)) Xor fkey(k + 5)
        t6 = Te0(s(24)) Xor Te1(s(29)) Xor Te2(s(6)) Xor Te3(s(11)) Xor fkey(k + 6)
        t7 = Te0(s(28)) Xor Te1(s(1)) Xor Te2(s(10)) Xor Te3(s(15)) Xor fkey(k + 7)
        k = k + 8
    Next i

    'Final round
    CopyMemory s(0), t0, 4&
    CopyMemory s(4), t1, 4&
    CopyMemory s(8), t2, 4&
    CopyMemory s(12), t3, 4&
    CopyMemory s(16), t4, 4&
    CopyMemory s(20), t5, 4&
    CopyMemory s(24), t6, 4&
    CopyMemory s(28), t7, 4&
    t0 = (Te4(s(0)) And &HFF&) Xor (Te4(s(5)) And &HFF00&) Xor (Te4(s(14)) And &HFF0000) Xor (Te4(s(19)) And &HFF000000) Xor fkey(k + 0)
    t1 = (Te4(s(4)) And &HFF&) Xor (Te4(s(9)) And &HFF00&) Xor (Te4(s(18)) And &HFF0000) Xor (Te4(s(23)) And &HFF000000) Xor fkey(k + 1)
    t2 = (Te4(s(8)) And &HFF&) Xor (Te4(s(13)) And &HFF00&) Xor (Te4(s(22)) And &HFF0000) Xor (Te4(s(27)) And &HFF000000) Xor fkey(k + 2)
    t3 = (Te4(s(12)) And &HFF&) Xor (Te4(s(17)) And &HFF00&) Xor (Te4(s(26)) And &HFF0000) Xor (Te4(s(31)) And &HFF000000) Xor fkey(k + 3)
    t4 = (Te4(s(16)) And &HFF&) Xor (Te4(s(21)) And &HFF00&) Xor (Te4(s(30)) And &HFF0000) Xor (Te4(s(3)) And &HFF000000) Xor fkey(k + 4)
    t5 = (Te4(s(20)) And &HFF&) Xor (Te4(s(25)) And &HFF00&) Xor (Te4(s(2)) And &HFF0000) Xor (Te4(s(7)) And &HFF000000) Xor fkey(k + 5)
    t6 = (Te4(s(24)) And &HFF&) Xor (Te4(s(29)) And &HFF00&) Xor (Te4(s(6)) And &HFF0000) Xor (Te4(s(11)) And &HFF000000) Xor fkey(k + 6)
    t7 = (Te4(s(28)) And &HFF&) Xor (Te4(s(1)) And &HFF00&) Xor (Te4(s(10)) And &HFF0000) Xor (Te4(s(15)) And &HFF000000) Xor fkey(k + 7)
    CopyMemory ciphertext(q + 0), t0, 4&
    CopyMemory ciphertext(q + 4), t1, 4&
    CopyMemory ciphertext(q + 8), t2, 4&
    CopyMemory ciphertext(q + 12), t3, 4&
    CopyMemory ciphertext(q + 16), t4, 4&
    CopyMemory ciphertext(q + 20), t5, 4&
    CopyMemory ciphertext(q + 24), t6, 4&
    CopyMemory ciphertext(q + 28), t7, 4&
End Sub

'Decrypt a 256 bit block.  ciphertext(q ... q+31) is input, plaintext(p ... p+31) is output.
'plaintext and ciphertext can be the same array.  Will crash if plaintext(p ... p+31) is not allocated.
Private Sub Block256Decrypt(plaintext() As Byte, ciphertext() As Byte, p As Long, q As Long)
    Dim i     As Long
    Dim k     As Long
    Dim t0    As Long
    Dim t1    As Long
    Dim t2    As Long
    Dim t3    As Long
    Dim t4    As Long
    Dim t5    As Long
    Dim t6    As Long
    Dim t7    As Long
    Dim s(31) As Byte

    CopyMemory t0, ciphertext(q + 0), 4&
    CopyMemory t1, ciphertext(q + 4), 4&
    CopyMemory t2, ciphertext(q + 8), 4&
    CopyMemory t3, ciphertext(q + 12), 4&
    CopyMemory t4, ciphertext(q + 16), 4&
    CopyMemory t5, ciphertext(q + 20), 4&
    CopyMemory t6, ciphertext(q + 24), 4&
    CopyMemory t7, ciphertext(q + 28), 4&
    t0 = t0 Xor rkey(0)
    t1 = t1 Xor rkey(1)
    t2 = t2 Xor rkey(2)
    t3 = t3 Xor rkey(3)
    t4 = t4 Xor rkey(4)
    t5 = t5 Xor rkey(5)
    t6 = t6 Xor rkey(6)
    t7 = t7 Xor rkey(7)
    k = 8

    For i = 1 To Nr - 1 'Nr is number of rounds
        CopyMemory s(0), t0, 4&
        CopyMemory s(4), t1, 4&
        CopyMemory s(8), t2, 4&
        CopyMemory s(12), t3, 4&
        CopyMemory s(16), t4, 4&
        CopyMemory s(20), t5, 4&
        CopyMemory s(24), t6, 4&
        CopyMemory s(28), t7, 4&
        t0 = Td0(s(0)) Xor Td1(s(29)) Xor Td2(s(22)) Xor Td3(s(19)) Xor rkey(k + 0)
        t1 = Td0(s(4)) Xor Td1(s(1)) Xor Td2(s(26)) Xor Td3(s(23)) Xor rkey(k + 1)
        t2 = Td0(s(8)) Xor Td1(s(5)) Xor Td2(s(30)) Xor Td3(s(27)) Xor rkey(k + 2)
        t3 = Td0(s(12)) Xor Td1(s(9)) Xor Td2(s(2)) Xor Td3(s(31)) Xor rkey(k + 3)
        t4 = Td0(s(16)) Xor Td1(s(13)) Xor Td2(s(6)) Xor Td3(s(3)) Xor rkey(k + 4)
        t5 = Td0(s(20)) Xor Td1(s(17)) Xor Td2(s(10)) Xor Td3(s(7)) Xor rkey(k + 5)
        t6 = Td0(s(24)) Xor Td1(s(21)) Xor Td2(s(14)) Xor Td3(s(11)) Xor rkey(k + 6)
        t7 = Td0(s(28)) Xor Td1(s(25)) Xor Td2(s(18)) Xor Td3(s(15)) Xor rkey(k + 7)
        k = k + 8
    Next i

    'Final round
    CopyMemory s(0), t0, 4&
    CopyMemory s(4), t1, 4&
    CopyMemory s(8), t2, 4&
    CopyMemory s(12), t3, 4&
    CopyMemory s(16), t4, 4&
    CopyMemory s(20), t5, 4&
    CopyMemory s(24), t6, 4&
    CopyMemory s(28), t7, 4&
    t0 = (Td4(s(0)) And &HFF&) Xor (Td4(s(29)) And &HFF00&) Xor (Td4(s(22)) And &HFF0000) Xor (Td4(s(19)) And &HFF000000) Xor rkey(k + 0)
    t1 = (Td4(s(4)) And &HFF&) Xor (Td4(s(1)) And &HFF00&) Xor (Td4(s(26)) And &HFF0000) Xor (Td4(s(23)) And &HFF000000) Xor rkey(k + 1)
    t2 = (Td4(s(8)) And &HFF&) Xor (Td4(s(5)) And &HFF00&) Xor (Td4(s(30)) And &HFF0000) Xor (Td4(s(27)) And &HFF000000) Xor rkey(k + 2)
    t3 = (Td4(s(12)) And &HFF&) Xor (Td4(s(9)) And &HFF00&) Xor (Td4(s(2)) And &HFF0000) Xor (Td4(s(31)) And &HFF000000) Xor rkey(k + 3)
    t4 = (Td4(s(16)) And &HFF&) Xor (Td4(s(13)) And &HFF00&) Xor (Td4(s(6)) And &HFF0000) Xor (Td4(s(3)) And &HFF000000) Xor rkey(k + 4)
    t5 = (Td4(s(20)) And &HFF&) Xor (Td4(s(17)) And &HFF00&) Xor (Td4(s(10)) And &HFF0000) Xor (Td4(s(7)) And &HFF000000) Xor rkey(k + 5)
    t6 = (Td4(s(24)) And &HFF&) Xor (Td4(s(21)) And &HFF00&) Xor (Td4(s(14)) And &HFF0000) Xor (Td4(s(11)) And &HFF000000) Xor rkey(k + 6)
    t7 = (Td4(s(28)) And &HFF&) Xor (Td4(s(25)) And &HFF00&) Xor (Td4(s(18)) And &HFF0000) Xor (Td4(s(15)) And &HFF000000) Xor rkey(k + 7)
    CopyMemory plaintext(p + 0), t0, 4&
    CopyMemory plaintext(p + 4), t1, 4&
    CopyMemory plaintext(p + 8), t2, 4&
    CopyMemory plaintext(p + 12), t3, 4&
    CopyMemory plaintext(p + 16), t4, 4&
    CopyMemory plaintext(p + 20), t5, 4&
    CopyMemory plaintext(p + 24), t6, 4&
    CopyMemory plaintext(p + 28), t7, 4&
End Sub
#End If


'Encrypt an arbritrary size array in blocks.  plaintext(0 ... n-1) is input,
'ciphertext(0 ... m-1) is output, where m is n padded to a multiple of (BlockBits\8) bytes.
'plaintext and ciphertext must be distinct, and ciphertext must be a redimensionable array.
'If appendsize is non-zero, a one byte length field is appended to the plaintext
'before encrypting so the original length can be retrieved after decryption.
#If SUPPORT_LEVEL Then
Private Function ArrayEncrypt(plaintext() As Byte, ciphertext() As Byte, appendsize As Long, BlockBits As Long) As Long
#Else
Private Function ArrayEncrypt(plaintext() As Byte, ciphertext() As Byte, appendsize As Long) As Long
#End If
    Dim i           As Long
    Dim m           As Long
    Dim N           As Long
#If SUPPORT_LEVEL = 0 Then
    Const BlockSize As Long = 16 'bytes
#Else
    Dim BlockSize   As Long

    Select Case BlockBits
    Case 128:  BlockSize = 16
    Case 192:  BlockSize = 24
    Case 256:  BlockSize = 32
#If SUPPORT_LEVEL = 2 Then
    Case 160:  BlockSize = 20
    Case 224:  BlockSize = 28
#End If
    Case Else: Err.Raise 1, , "cRijndael.ArrayEncrypt - Illegal BlockBits value"
    End Select
#End If

    If LBound(plaintext) <> 0 Then Err.Raise 1, , "cRijndael.ArrayEncrypt - plaintext must be zero based array"

    N = UBound(plaintext) + 1
    If appendsize = 0 Then
#If SUPPORT_LEVEL Then
        m = ((N + BlockSize - 1) \ BlockSize) * BlockSize
#Else
        m = (N + BlockSize - 1) And &HFFFFFFF0  'BlockSize=16 specific
#End If
        ReDim ciphertext(m - 1)
    Else
#If SUPPORT_LEVEL Then
        m = ((N + BlockSize) \ BlockSize) * BlockSize
#Else
        m = (N + BlockSize) And &HFFFFFFF0      'BlockSize=16 specific
#End If
        ReDim ciphertext(m - 1)
        ciphertext(m - 1) = N Mod BlockSize
    End If

#If SUPPORT_LEVEL Then
    Select Case BlockBits
    Case 128
#End If

    For i = 0 To N - BlockSize Step BlockSize
        BlockEncrypt plaintext, ciphertext, i, i
    Next i

#If SUPPORT_LEVEL Then
    Case 192
    For i = 0 To N - BlockSize Step BlockSize
        Block192Encrypt plaintext, ciphertext, i, i
    Next i
    Case 256
    For i = 0 To N - BlockSize Step BlockSize
        Block256Encrypt plaintext, ciphertext, i, i
    Next i
#If SUPPORT_LEVEL = 2 Then
    Case 160
    For i = 0 To N - BlockSize Step BlockSize
        Block160Encrypt plaintext, ciphertext, i, i
    Next i
    Case 224
    For i = 0 To N - BlockSize Step BlockSize
        Block224Encrypt plaintext, ciphertext, i, i
    Next i
#End If
    End Select
#End If

    If (N Mod BlockSize) <> 0 Then CopyMemory ciphertext(i), plaintext(i), N Mod BlockSize

#If SUPPORT_LEVEL Then
    Select Case BlockBits
    Case 128
#End If

    If i <> m Then BlockEncrypt ciphertext, ciphertext, i, i

#If SUPPORT_LEVEL Then
    Case 192
    If i <> m Then Block192Encrypt ciphertext, ciphertext, i, i
    Case 256
    If i <> m Then Block256Encrypt ciphertext, ciphertext, i, i
#If SUPPORT_LEVEL = 2 Then
    Case 160
    If i <> m Then Block160Encrypt ciphertext, ciphertext, i, i
    Case 224
    If i <> m Then Block224Encrypt ciphertext, ciphertext, i, i
#End If
    End Select
#End If

End Function

'Decrypts an array encrypted with the ArrayEncrypt function
#If SUPPORT_LEVEL Then
Private Function ArrayDecrypt(plaintext() As Byte, ciphertext() As Byte, appendsize As Long, BlockBits As Long) As Long
#Else
Private Function ArrayDecrypt(plaintext() As Byte, ciphertext() As Byte, appendsize As Long) As Long
#End If
    Dim i           As Long
    Dim m           As Long
    Dim N           As Long
#If SUPPORT_LEVEL = 0 Then
    Const BlockSize As Long = 16 'bytes
#Else
    Dim BlockSize   As Long

    Select Case BlockBits
    Case 128:  BlockSize = 16
    Case 192:  BlockSize = 24
    Case 256:  BlockSize = 32
#If SUPPORT_LEVEL = 2 Then
    Case 160:  BlockSize = 20
    Case 224:  BlockSize = 28
#End If
    Case Else: Err.Raise 1, , "cRijndael.ArrayDecrypt - Illegal BlockBits value"
    End Select
#End If

    If LBound(ciphertext) <> 0 Then Err.Raise 1, , "cRijndael.ArrayDecrypt - ciphertext must be zero based array"

    N = UBound(ciphertext) + 1
    If ((N Mod BlockSize) = 0) Then
        ReDim plaintext(N - 1)

#If SUPPORT_LEVEL Then
        Select Case BlockBits
        Case 128
#End If

        For i = 0 To N - BlockSize Step BlockSize
            BlockDecrypt plaintext, ciphertext, i, i
        Next i

#If SUPPORT_LEVEL Then
        Case 192
        For i = 0 To N - BlockSize Step BlockSize
            Block192Decrypt plaintext, ciphertext, i, i
        Next i
        Case 256
        For i = 0 To N - BlockSize Step BlockSize
            Block256Decrypt plaintext, ciphertext, i, i
        Next i
#If SUPPORT_LEVEL = 2 Then
        Case 160
        For i = 0 To N - BlockSize Step BlockSize
            Block160Decrypt plaintext, ciphertext, i, i
        Next i
        Case 224
        For i = 0 To N - BlockSize Step BlockSize
            Block224Decrypt plaintext, ciphertext, i, i
        Next i
#End If
        End Select
#End If

        If appendsize Then
            If plaintext(N - 1) < BlockSize Then
                N = N - BlockSize + plaintext(N - 1)
                If N > 0 Then ReDim Preserve plaintext(N - 1)
            Else
                'MsgBox "warning - incorrect length field"
                ArrayDecrypt = 1
            End If
        End If
    Else
        'MsgBox "ciphertext size not a multiple of block size"
        ArrayDecrypt = -1
    End If

End Function




'File encryption.  The ciphertext file will be padded to a multiple of (BlockBits\8) bytes.
'One byte is appended to the ciphertext to indicate the length of the final block.
'FileDecrypt recovers this length field and restores the original plaintext file length.
'PlaintextFileName and CiphertextFileName must be distinct, PlaintextFileName must exist,
'CiphertextFileName must not exist, etc.  Be sure to do file checking before calling this.
#If SUPPORT_LEVEL Then
Public Function FileEncrypt(PlaintextFileName As String, CiphertextFileName As String, BlockBits As Long) As Long
#Else
Public Function FileEncrypt(PlaintextFileName As String, CiphertextFileName As String) As Long
#End If
    Dim FileNum     As Integer
    Dim FileNum2    As Integer
    Dim i           As Long
    Dim m           As Long 'ciphertext file size
    Dim N           As Long 'plaintext file size
    Dim Data()      As Byte
#If SUPPORT_LEVEL = 0 Then
    Const BlockSize As Long = 16 'bytes
    Const MaxBlocks As Long = MaxFileChunkSize \ BlockSize
#Else
    Dim BlockSize   As Long
    Dim MaxBlocks   As Long

    Select Case BlockBits
    Case 128:  BlockSize = 16
    Case 192:  BlockSize = 24
    Case 256:  BlockSize = 32
#If SUPPORT_LEVEL = 2 Then
    Case 160:  BlockSize = 20
    Case 224:  BlockSize = 28
#End If
    Case Else: Err.Raise 1, , "cRijndael.FileEncrypt - Illegal BlockBits value"
    End Select
    MaxBlocks = MaxFileChunkSize \ BlockSize
#End If

    N = FileLen(PlaintextFileName)
#If SUPPORT_LEVEL Then
    m = ((N + BlockSize) \ BlockSize) * BlockSize
#Else
    m = (N + BlockSize) And (-BlockSize) 'BlockSize=16 specific
#End If

    FileNum = FreeFile
    Open PlaintextFileName For Binary Access Read As FileNum
    FileNum2 = FreeFile
    Open CiphertextFileName For Binary Access Write As FileNum2

    'For large files, encrypt in pieces no larger than MaxFileChunkSize
    If m > MaxBlocks * BlockSize Then
        ReDim Data(MaxBlocks * BlockSize - 1)
        Do
            Get #FileNum, , Data
#If SUPPORT_LEVEL Then
            Select Case BlockBits
            Case 128
#End If
            For i = 0 To (MaxBlocks - 1) * BlockSize Step BlockSize
                BlockEncrypt Data, Data, i, i
            Next i
#If SUPPORT_LEVEL Then
            Case 192
            For i = 0 To (MaxBlocks - 1) * BlockSize Step BlockSize
                Block192Encrypt Data, Data, i, i
            Next i
            Case 256
            For i = 0 To (MaxBlocks - 1) * BlockSize Step BlockSize
                Block256Encrypt Data, Data, i, i
            Next i
#If SUPPORT_LEVEL = 2 Then
            Case 160
            For i = 0 To (MaxBlocks - 1) * BlockSize Step BlockSize
                Block160Encrypt Data, Data, i, i
            Next i
            Case 224
            For i = 0 To (MaxBlocks - 1) * BlockSize Step BlockSize
                Block224Encrypt Data, Data, i, i
            Next i
#End If
            End Select
#End If
            Put #FileNum2, , Data
            m = m - MaxBlocks * BlockSize
        Loop While m > MaxBlocks * BlockSize
    End If

    'Encrypt the last piece of the file
    ReDim Data(m - 1)
    Get #FileNum, , Data
    Data(m - 1) = N Mod BlockSize
#If SUPPORT_LEVEL Then
    Select Case BlockBits
    Case 128
#End If
    For i = 0 To m - BlockSize Step BlockSize
        BlockEncrypt Data, Data, i, i
    Next i
#If SUPPORT_LEVEL Then
    Case 192
    For i = 0 To m - BlockSize Step BlockSize
        Block192Encrypt Data, Data, i, i
    Next i
    Case 256
    For i = 0 To m - BlockSize Step BlockSize
        Block256Encrypt Data, Data, i, i
    Next i
#If SUPPORT_LEVEL = 2 Then
    Case 160
    For i = 0 To m - BlockSize Step BlockSize
        Block160Encrypt Data, Data, i, i
    Next i
    Case 224
    For i = 0 To m - BlockSize Step BlockSize
        Block224Encrypt Data, Data, i, i
    Next i
#End If
    End Select
#End If
    Put FileNum2, , Data

    Close FileNum
    Close FileNum2
End Function

'File decryption.
#If SUPPORT_LEVEL Then
Public Function FileDecrypt(PlaintextFileName As String, CiphertextFileName As String, BlockBits As Long) As Long
#Else
Public Function FileDecrypt(PlaintextFileName As String, CiphertextFileName As String) As Long
#End If
    Dim FileNum     As Integer
    Dim FileNum2    As Integer
    Dim i           As Long
    Dim m           As Long 'ciphertext file size
    Dim N           As Long 'plaintext file size
    Dim Data()      As Byte
#If SUPPORT_LEVEL = 0 Then
    Const BlockSize As Long = 16 'bytes
    Const MaxBlocks As Long = MaxFileChunkSize \ BlockSize
#Else
    Dim BlockSize   As Long
    Dim MaxBlocks   As Long

    Select Case BlockBits
    Case 128:  BlockSize = 16
    Case 192:  BlockSize = 24
    Case 256:  BlockSize = 32
#If SUPPORT_LEVEL = 2 Then
    Case 160:  BlockSize = 20
    Case 224:  BlockSize = 28
#End If
    Case Else: Err.Raise 1, , "cRijndael.FileDecrypt - Illegal BlockBits value"
    End Select
    MaxBlocks = MaxFileChunkSize \ BlockSize
#End If

    m = FileLen(CiphertextFileName)
#If SUPPORT_LEVEL Then
    If (m = 0) Or ((m Mod BlockSize) <> 0) Then
#Else
    If (m = 0) Or ((m And (BlockSize - 1)) <> 0) Then 'BlockSize=16 specific
#End If
        'MsgBox "File Size Error - ciphertext file not a multiple of block size"
        FileDecrypt = 1
    Else
        FileNum = FreeFile
        Open CiphertextFileName For Binary Access Read As FileNum
        FileNum2 = FreeFile
        Open PlaintextFileName For Binary Access Write As FileNum2

        'For large files, decrypt in pieces no larger than MaxFileChunkSize
        If m > MaxBlocks * BlockSize Then
            ReDim Data(MaxBlocks * BlockSize - 1)
            Do
                Get #FileNum, , Data
#If SUPPORT_LEVEL Then
                Select Case BlockBits
                Case 128
#End If
                For i = 0 To (MaxBlocks - 1) * BlockSize Step BlockSize
                    BlockDecrypt Data, Data, i, i
                Next i
#If SUPPORT_LEVEL Then
                Case 192
                For i = 0 To (MaxBlocks - 1) * BlockSize Step BlockSize
                    Block192Decrypt Data, Data, i, i
                Next i
                Case 256
                For i = 0 To (MaxBlocks - 1) * BlockSize Step BlockSize
                    Block256Decrypt Data, Data, i, i
                Next i
#If SUPPORT_LEVEL = 2 Then
                Case 160
                For i = 0 To (MaxBlocks - 1) * BlockSize Step BlockSize
                    Block160Decrypt Data, Data, i, i
                Next i
                Case 224
                For i = 0 To (MaxBlocks - 1) * BlockSize Step BlockSize
                    Block224Decrypt Data, Data, i, i
                Next i
#End If
                End Select
#End If
                Put #FileNum2, , Data
                m = m - MaxBlocks * BlockSize
            Loop While m > MaxBlocks * BlockSize
        End If

        'Decrypt the last piece of the file
        ReDim Data(m - 1)
        Get #FileNum, , Data
#If SUPPORT_LEVEL Then
        Select Case BlockBits
        Case 128
#End If
        For i = 0 To m - BlockSize Step BlockSize
            BlockDecrypt Data, Data, i, i
        Next i
#If SUPPORT_LEVEL Then
        Case 192
        For i = 0 To m - BlockSize Step BlockSize
            Block192Decrypt Data, Data, i, i
        Next i
        Case 256
        For i = 0 To m - BlockSize Step BlockSize
            Block256Decrypt Data, Data, i, i
        Next i
#If SUPPORT_LEVEL = 2 Then
        Case 160
        For i = 0 To m - BlockSize Step BlockSize
            Block160Decrypt Data, Data, i, i
        Next i
        Case 224
        For i = 0 To m - BlockSize Step BlockSize
            Block224Decrypt Data, Data, i, i
        Next i
#End If
        End Select
#End If
        'Recover length field
        If Data(m - 1) < BlockSize Then
            N = m - BlockSize + CLng(Data(m - 1))
        Else
#If SUPPORT_LEVEL Then
            'MsgBox "warning - incorrect length field in decrypted file." & vbCrLf & "Wrong key, keysize, or blocksize?"
#Else
            'MsgBox "warning - incorrect length field in decrypted file." & vbCrLf & "Wrong key or keysize?"
#End If
            N = m
        End If
        If N > 0 Then
            ReDim Preserve Data(N - 1)
            Put FileNum2, , Data
        End If

        Close FileNum
        Close FileNum2
    End If
End Function


Private Sub Class_Initialize()
#If COMPILE_CONSTANTS = 0 Then
    Dim i          As Long
    Dim y          As Byte
    Dim s(7)       As Byte
    Dim ib         As Byte
    Dim ptab(255)  As Byte
    Dim ltab(255)  As Byte

    'use 3 as primitive root to generate power and log tables
    ltab(0) = 0
    ltab(1) = 0
    ltab(3) = 1
    ptab(0) = 1
    ptab(1) = 3
    For i = 2 To 255 'ptab(i) = ptab(i - 1) Xor Xtime(ptab(i - 1))
        If (ptab(i - 1) And &H80) Then
            ptab(i) = ptab(i - 1) Xor ((ptab(i - 1) And 127) * 2) Xor &H1B
        Else
            ptab(i) = ptab(i - 1) Xor (ptab(i - 1) * 2)
        End If
        ltab(ptab(i)) = i
    Next i

    'affine transformation:- each bit is xored with itself shifted one bit
    Te4(0) = &H63636363
    Td4(&H63) = 0
    For i = 1 To 255
        y = ptab(255 - ltab(i)) 'multiplicative inverse
        ib = y
        If ib And &H80 Then ib = (ib And 127) * 2 Or 1 Else ib = ib * 2 'RotateLeftByte
        y = y Xor ib
        If ib And &H80 Then ib = (ib And 127) * 2 Or 1 Else ib = ib * 2
        y = y Xor ib
        If ib And &H80 Then ib = (ib And 127) * 2 Or 1 Else ib = ib * 2
        y = y Xor ib
        If ib And &H80 Then ib = (ib And 127) * 2 Or 1 Else ib = ib * 2
        y = y Xor ib Xor &H63

        s(0) = y
        s(1) = s(0)
        s(2) = s(0)
        s(3) = s(0)
        CopyMemory Te4(i), s(0), 4&

        s(0) = i
        s(1) = s(0)
        s(2) = s(0)
        s(3) = s(0)
        CopyMemory Td4(y), s(0), 4&
    Next i

    y = 1
    For i = 0 To UBound(rco)
        rco(i) = y
        If (y And &H80) Then 'y = Xtime(y)
            y = ((y And 127) * 2) Xor &H1B
        Else
            y = y * 2
        End If
    Next i

    'calculate forward and reverse tables
    For i = 0 To 255
        y = Te4(i) And &HFF&
        's(3) = y Xor Xtime(y)
'        s(0) = Xtime(y)
        If (y And &H80) Then
            s(0) = ((y And 127) * 2) Xor &H1B
            s(3) = y Xor s(0)
        Else
            s(0) = y * 2
            s(3) = y Xor s(0)
        End If
        s(2) = y
        s(1) = y
        CopyMemory s(4), s(0), 4&
        CopyMemory Te0(i), s(0), 4&
        CopyMemory Te1(i), s(3), 4&
        CopyMemory Te2(i), s(2), 4&
        CopyMemory Te3(i), s(1), 4&
        y = Td4(i) And &HFF&
        If y = 0 Then 'x.y= AntiLog(Log(x) + Log(y))
            s(3) = 0
            s(2) = 0
            s(1) = 0
            s(0) = 0
        Else
            s(3) = ptab((CLng(ltab(&HB)) + CLng(ltab(y))) Mod 255)
            s(2) = ptab((CLng(ltab(&HD)) + CLng(ltab(y))) Mod 255)
            s(1) = ptab((CLng(ltab(&H9)) + CLng(ltab(y))) Mod 255)
            s(0) = ptab((CLng(ltab(&HE)) + CLng(ltab(y))) Mod 255)
        End If
        CopyMemory s(4), s(0), 4&
        CopyMemory Td0(i), s(0), 4&
        CopyMemory Td1(i), s(3), 4&
        CopyMemory Td2(i), s(2), 4&
        CopyMemory Td3(i), s(1), 4&
    Next i
#Else
    Te0(0) = &HA56363C6:   Te0(1) = &H847C7CF8:   Te0(2) = &H997777EE:   Te0(3) = &H8D7B7BF6:   Te0(4) = &HDF2F2FF:    Te0(5) = &HBD6B6BD6:   Te0(6) = &HB16F6FDE:   Te0(7) = &H54C5C591
    Te0(8) = &H50303060:   Te0(9) = &H3010102:    Te0(10) = &HA96767CE:  Te0(11) = &H7D2B2B56:  Te0(12) = &H19FEFEE7:  Te0(13) = &H62D7D7B5:  Te0(14) = &HE6ABAB4D:  Te0(15) = &H9A7676EC
    Te0(16) = &H45CACA8F:  Te0(17) = &H9D82821F:  Te0(18) = &H40C9C989:  Te0(19) = &H877D7DFA:  Te0(20) = &H15FAFAEF:  Te0(21) = &HEB5959B2:  Te0(22) = &HC947478E:  Te0(23) = &HBF0F0FB
    Te0(24) = &HECADAD41:  Te0(25) = &H67D4D4B3:  Te0(26) = &HFDA2A25F:  Te0(27) = &HEAAFAF45:  Te0(28) = &HBF9C9C23:  Te0(29) = &HF7A4A453:  Te0(30) = &H967272E4:  Te0(31) = &H5BC0C09B
    Te0(32) = &HC2B7B775:  Te0(33) = &H1CFDFDE1:  Te0(34) = &HAE93933D:  Te0(35) = &H6A26264C:  Te0(36) = &H5A36366C:  Te0(37) = &H413F3F7E:  Te0(38) = &H2F7F7F5:   Te0(39) = &H4FCCCC83
    Te0(40) = &H5C343468:  Te0(41) = &HF4A5A551:  Te0(42) = &H34E5E5D1:  Te0(43) = &H8F1F1F9:   Te0(44) = &H937171E2:  Te0(45) = &H73D8D8AB:  Te0(46) = &H53313162:  Te0(47) = &H3F15152A
    Te0(48) = &HC040408:   Te0(49) = &H52C7C795:  Te0(50) = &H65232346:  Te0(51) = &H5EC3C39D:  Te0(52) = &H28181830:  Te0(53) = &HA1969637:  Te0(54) = &HF05050A:   Te0(55) = &HB59A9A2F
    Te0(56) = &H907070E:   Te0(57) = &H36121224:  Te0(58) = &H9B80801B:  Te0(59) = &H3DE2E2DF:  Te0(60) = &H26EBEBCD:  Te0(61) = &H6927274E:  Te0(62) = &HCDB2B27F:  Te0(63) = &H9F7575EA
    Te0(64) = &H1B090912:  Te0(65) = &H9E83831D:  Te0(66) = &H742C2C58:  Te0(67) = &H2E1A1A34:  Te0(68) = &H2D1B1B36:  Te0(69) = &HB26E6EDC:  Te0(70) = &HEE5A5AB4:  Te0(71) = &HFBA0A05B
    Te0(72) = &HF65252A4:  Te0(73) = &H4D3B3B76:  Te0(74) = &H61D6D6B7:  Te0(75) = &HCEB3B37D:  Te0(76) = &H7B292952:  Te0(77) = &H3EE3E3DD:  Te0(78) = &H712F2F5E:  Te0(79) = &H97848413
    Te0(80) = &HF55353A6:  Te0(81) = &H68D1D1B9:  Te0(82) = &H0&:        Te0(83) = &H2CEDEDC1:  Te0(84) = &H60202040:  Te0(85) = &H1FFCFCE3:  Te0(86) = &HC8B1B179:  Te0(87) = &HED5B5BB6
    Te0(88) = &HBE6A6AD4:  Te0(89) = &H46CBCB8D:  Te0(90) = &HD9BEBE67:  Te0(91) = &H4B393972:  Te0(92) = &HDE4A4A94:  Te0(93) = &HD44C4C98:  Te0(94) = &HE85858B0:  Te0(95) = &H4ACFCF85
    Te0(96) = &H6BD0D0BB:  Te0(97) = &H2AEFEFC5:  Te0(98) = &HE5AAAA4F:  Te0(99) = &H16FBFBED:  Te0(100) = &HC5434386: Te0(101) = &HD74D4D9A: Te0(102) = &H55333366: Te0(103) = &H94858511
    Te0(104) = &HCF45458A: Te0(105) = &H10F9F9E9: Te0(106) = &H6020204:  Te0(107) = &H817F7FFE: Te0(108) = &HF05050A0: Te0(109) = &H443C3C78: Te0(110) = &HBA9F9F25: Te0(111) = &HE3A8A84B
    Te0(112) = &HF35151A2: Te0(113) = &HFEA3A35D: Te0(114) = &HC0404080: Te0(115) = &H8A8F8F05: Te0(116) = &HAD92923F: Te0(117) = &HBC9D9D21: Te0(118) = &H48383870: Te0(119) = &H4F5F5F1
    Te0(120) = &HDFBCBC63: Te0(121) = &HC1B6B677: Te0(122) = &H75DADAAF: Te0(123) = &H63212142: Te0(124) = &H30101020: Te0(125) = &H1AFFFFE5: Te0(126) = &HEF3F3FD:  Te0(127) = &H6DD2D2BF
    Te0(128) = &H4CCDCD81: Te0(129) = &H140C0C18: Te0(130) = &H35131326: Te0(131) = &H2FECECC3: Te0(132) = &HE15F5FBE: Te0(133) = &HA2979735: Te0(134) = &HCC444488: Te0(135) = &H3917172E
    Te0(136) = &H57C4C493: Te0(137) = &HF2A7A755: Te0(138) = &H827E7EFC: Te0(139) = &H473D3D7A: Te0(140) = &HAC6464C8: Te0(141) = &HE75D5DBA: Te0(142) = &H2B191932: Te0(143) = &H957373E6
    Te0(144) = &HA06060C0: Te0(145) = &H98818119: Te0(146) = &HD14F4F9E: Te0(147) = &H7FDCDCA3: Te0(148) = &H66222244: Te0(149) = &H7E2A2A54: Te0(150) = &HAB90903B: Te0(151) = &H8388880B
    Te0(152) = &HCA46468C: Te0(153) = &H29EEEEC7: Te0(154) = &HD3B8B86B: Te0(155) = &H3C141428: Te0(156) = &H79DEDEA7: Te0(157) = &HE25E5EBC: Te0(158) = &H1D0B0B16: Te0(159) = &H76DBDBAD
    Te0(160) = &H3BE0E0DB: Te0(161) = &H56323264: Te0(162) = &H4E3A3A74: Te0(163) = &H1E0A0A14: Te0(164) = &HDB494992: Te0(165) = &HA06060C:  Te0(166) = &H6C242448: Te0(167) = &HE45C5CB8
    Te0(168) = &H5DC2C29F: Te0(169) = &H6ED3D3BD: Te0(170) = &HEFACAC43: Te0(171) = &HA66262C4: Te0(172) = &HA8919139: Te0(173) = &HA4959531: Te0(174) = &H37E4E4D3: Te0(175) = &H8B7979F2
    Te0(176) = &H32E7E7D5: Te0(177) = &H43C8C88B: Te0(178) = &H5937376E: Te0(179) = &HB76D6DDA: Te0(180) = &H8C8D8D01: Te0(181) = &H64D5D5B1: Te0(182) = &HD24E4E9C: Te0(183) = &HE0A9A949
    Te0(184) = &HB46C6CD8: Te0(185) = &HFA5656AC: Te0(186) = &H7F4F4F3:  Te0(187) = &H25EAEACF: Te0(188) = &HAF6565CA: Te0(189) = &H8E7A7AF4: Te0(190) = &HE9AEAE47: Te0(191) = &H18080810
    Te0(192) = &HD5BABA6F: Te0(193) = &H887878F0: Te0(194) = &H6F25254A: Te0(195) = &H722E2E5C: Te0(196) = &H241C1C38: Te0(197) = &HF1A6A657: Te0(198) = &HC7B4B473: Te0(199) = &H51C6C697
    Te0(200) = &H23E8E8CB: Te0(201) = &H7CDDDDA1: Te0(202) = &H9C7474E8: Te0(203) = &H211F1F3E: Te0(204) = &HDD4B4B96: Te0(205) = &HDCBDBD61: Te0(206) = &H868B8B0D: Te0(207) = &H858A8A0F
    Te0(208) = &H907070E0: Te0(209) = &H423E3E7C: Te0(210) = &HC4B5B571: Te0(211) = &HAA6666CC: Te0(212) = &HD8484890: Te0(213) = &H5030306:  Te0(214) = &H1F6F6F7:  Te0(215) = &H120E0E1C
    Te0(216) = &HA36161C2: Te0(217) = &H5F35356A: Te0(218) = &HF95757AE: Te0(219) = &HD0B9B969: Te0(220) = &H91868617: Te0(221) = &H58C1C199: Te0(222) = &H271D1D3A: Te0(223) = &HB99E9E27
    Te0(224) = &H38E1E1D9: Te0(225) = &H13F8F8EB: Te0(226) = &HB398982B: Te0(227) = &H33111122: Te0(228) = &HBB6969D2: Te0(229) = &H70D9D9A9: Te0(230) = &H898E8E07: Te0(231) = &HA7949433
    Te0(232) = &HB69B9B2D: Te0(233) = &H221E1E3C: Te0(234) = &H92878715: Te0(235) = &H20E9E9C9: Te0(236) = &H49CECE87: Te0(237) = &HFF5555AA: Te0(238) = &H78282850: Te0(239) = &H7ADFDFA5
    Te0(240) = &H8F8C8C03: Te0(241) = &HF8A1A159: Te0(242) = &H80898909: Te0(243) = &H170D0D1A: Te0(244) = &HDABFBF65: Te0(245) = &H31E6E6D7: Te0(246) = &HC6424284: Te0(247) = &HB86868D0
    Te0(248) = &HC3414182: Te0(249) = &HB0999929: Te0(250) = &H772D2D5A: Te0(251) = &H110F0F1E: Te0(252) = &HCBB0B07B: Te0(253) = &HFC5454A8: Te0(254) = &HD6BBBB6D: Te0(255) = &H3A16162C

    Te1(0) = &H6363C6A5:   Te1(1) = &H7C7CF884:   Te1(2) = &H7777EE99:   Te1(3) = &H7B7BF68D:   Te1(4) = &HF2F2FF0D:   Te1(5) = &H6B6BD6BD:   Te1(6) = &H6F6FDEB1:   Te1(7) = &HC5C59154
    Te1(8) = &H30306050:   Te1(9) = &H1010203:    Te1(10) = &H6767CEA9:  Te1(11) = &H2B2B567D:  Te1(12) = &HFEFEE719:  Te1(13) = &HD7D7B562:  Te1(14) = &HABAB4DE6:  Te1(15) = &H7676EC9A
    Te1(16) = &HCACA8F45:  Te1(17) = &H82821F9D:  Te1(18) = &HC9C98940:  Te1(19) = &H7D7DFA87:  Te1(20) = &HFAFAEF15:  Te1(21) = &H5959B2EB:  Te1(22) = &H47478EC9:  Te1(23) = &HF0F0FB0B
    Te1(24) = &HADAD41EC:  Te1(25) = &HD4D4B367:  Te1(26) = &HA2A25FFD:  Te1(27) = &HAFAF45EA:  Te1(28) = &H9C9C23BF:  Te1(29) = &HA4A453F7:  Te1(30) = &H7272E496:  Te1(31) = &HC0C09B5B
    Te1(32) = &HB7B775C2:  Te1(33) = &HFDFDE11C:  Te1(34) = &H93933DAE:  Te1(35) = &H26264C6A:  Te1(36) = &H36366C5A:  Te1(37) = &H3F3F7E41:  Te1(38) = &HF7F7F502:  Te1(39) = &HCCCC834F
    Te1(40) = &H3434685C:  Te1(41) = &HA5A551F4:  Te1(42) = &HE5E5D134:  Te1(43) = &HF1F1F908:  Te1(44) = &H7171E293:  Te1(45) = &HD8D8AB73:  Te1(46) = &H31316253:  Te1(47) = &H15152A3F
    Te1(48) = &H404080C:   Te1(49) = &HC7C79552:  Te1(50) = &H23234665:  Te1(51) = &HC3C39D5E:  Te1(52) = &H18183028:  Te1(53) = &H969637A1:  Te1(54) = &H5050A0F:   Te1(55) = &H9A9A2FB5
    Te1(56) = &H7070E09:   Te1(57) = &H12122436:  Te1(58) = &H80801B9B:  Te1(59) = &HE2E2DF3D:  Te1(60) = &HEBEBCD26:  Te1(61) = &H27274E69:  Te1(62) = &HB2B27FCD:  Te1(63) = &H7575EA9F
    Te1(64) = &H909121B:   Te1(65) = &H83831D9E:  Te1(66) = &H2C2C5874:  Te1(67) = &H1A1A342E:  Te1(68) = &H1B1B362D:  Te1(69) = &H6E6EDCB2:  Te1(70) = &H5A5AB4EE:  Te1(71) = &HA0A05BFB
    Te1(72) = &H5252A4F6:  Te1(73) = &H3B3B764D:  Te1(74) = &HD6D6B761:  Te1(75) = &HB3B37DCE:  Te1(76) = &H2929527B:  Te1(77) = &HE3E3DD3E:  Te1(78) = &H2F2F5E71:  Te1(79) = &H84841397
    Te1(80) = &H5353A6F5:  Te1(81) = &HD1D1B968:  Te1(82) = &H0&:        Te1(83) = &HEDEDC12C:  Te1(84) = &H20204060:  Te1(85) = &HFCFCE31F:  Te1(86) = &HB1B179C8:  Te1(87) = &H5B5BB6ED
    Te1(88) = &H6A6AD4BE:  Te1(89) = &HCBCB8D46:  Te1(90) = &HBEBE67D9:  Te1(91) = &H3939724B:  Te1(92) = &H4A4A94DE:  Te1(93) = &H4C4C98D4:  Te1(94) = &H5858B0E8:  Te1(95) = &HCFCF854A
    Te1(96) = &HD0D0BB6B:  Te1(97) = &HEFEFC52A:  Te1(98) = &HAAAA4FE5:  Te1(99) = &HFBFBED16:  Te1(100) = &H434386C5: Te1(101) = &H4D4D9AD7: Te1(102) = &H33336655: Te1(103) = &H85851194
    Te1(104) = &H45458ACF: Te1(105) = &HF9F9E910: Te1(106) = &H2020406:  Te1(107) = &H7F7FFE81: Te1(108) = &H5050A0F0: Te1(109) = &H3C3C7844: Te1(110) = &H9F9F25BA: Te1(111) = &HA8A84BE3
    Te1(112) = &H5151A2F3: Te1(113) = &HA3A35DFE: Te1(114) = &H404080C0: Te1(115) = &H8F8F058A: Te1(116) = &H92923FAD: Te1(117) = &H9D9D21BC: Te1(118) = &H38387048: Te1(119) = &HF5F5F104
    Te1(120) = &HBCBC63DF: Te1(121) = &HB6B677C1: Te1(122) = &HDADAAF75: Te1(123) = &H21214263: Te1(124) = &H10102030: Te1(125) = &HFFFFE51A: Te1(126) = &HFDDA1:) = &H768617: Te0DA96: Te Te1(122) = &HE45C5Cd4Te Te1iTe1(98) = &HAAAA4FE5: ) = &HE(35) 1) 1: T&H6E Te1(1  yC5[ = &H2F2F5E71:  148):  14= &AA: Te1E907)) = 2) =3wBBh &H23Ee1(49) = &09
  Next 09
  856A: Te0(87) = &H5B5BB6EDF: Te0n Te1(0A0F0: f0(156) = &H79DEDEA7: TDA(154) 3) =BDA666CC:450A87:  Te (s(1CFor i = t &H57C C C Cr i = t &H&H&H PORT_LEV7C C C Cr= &H76861  Te (s(1yH49CECE87:DA: Te0(F5E71:  1 &HF) = &H791F
   7C C C Cr i = t 1(11CC:450ptab(i -t959531'e1(92) = &H4A4A94DE:  Te1(93) = &H4C4C98D4:  Te1(94) = &H5858B0E8:  Te1(95) = &HCFCF854A
    TF95) = &Hs D854A
57C C8'76861  T Cr=h= &HB7e1(36) = &H36366C5A:  Te1(37) = &H3F3F7E41:  Te1(38) = &HF7F7F502:  Te1(39) = &HCCCC834F
    Te1(40) = &H3434685C:  Te1(41) = &HA5A551F4:  Te1(42) = &HE5E5D134:  Te1(43) = &HF1F1F908:  Te1(44) = &H7171E293:  Te1(45) = &HD8D8AB73:  Te1(46) = &H31316253:  Te1tab(39) =Bdael.A40) =  6868D0
B:  Te0(97) =DA = &H114= &AA: T854 = &H3F3Fv2 = &H57C4C493
    Te1&H16FBFBED: (A6A657: Te0(198) = &HC7B4B4h54) 3) =BD, 42B4B4h54) 3) =BD, 42B4BFBFBBd4e0(2201D= &HE9AEh&H58C1BloBFBED: (A6A657:  Te15E1(114) = Fe1(123) 5     y = y Xor t) = &H451(44) = &H687949433
C  Te1(44) = &HB0B
   5[HE3E3DD3E:  Te1(78) = &H2F2F5E71:  Te1(793) = &HE0&HE3ADA
    T05F2BhE2E5C:= &DA = &H11 C39D5E: 0A1:) = &H768617: Te0DA96: Te Te1(122) = &HE45C5Cd4Te Te1iTe1(98) = &HAAAA4FE5: ) = &HE(35) 1FAAAA4FE5o TeA1:) = &H768617:
    m =========  T05h284: T9:  Te1(23) Te15E1(1 A
57C C5kr            1(23) TN R82850:(47)  = &H768617:
   82)      = &Hh9) = &r
2Bhe1(7) 18N0 = &H9iC5C Te (s(1CFAAAA4FE59iC5C Fs(1CFor iF4FE59iC5uDF C Cr i =FTe1(10) = &H9iC5C Te (s(1CFAAAA4FE59iC5C Fs(1CFor iF4FE59iC5uDF C Cr i =FTe1(10) = &H9iC5C Te (s(1CFAAAA4FE59iC5C Fs(1CFor iF4FE59iC5uDF C CrCDd449433
5h284: T9:(1CF40D9D9A7_ B4h54) 3) =BD, 4 (s(1CFBs(1CFAAAA4FE59iC5C Fs(1CFor i(244) = &HDABFBF65: Te0(245) = &H31E6E6D7: Te) =E59iC5uDAAAA4FEI    = &Hh9DF24FEI    = rETe1(38) =FD:  Te1( k95) = &HsF5D134:  2: Te0(143) = &H957373E6
    Te0(144) = &HA06060C0: Te0(145) = &H98818119: Te0(146) = &HD14F4F9E: Te0(147) = &H7FDCDCA3: Te0(148) = &H66222244: Te0(149) = &H7E2A2A5D &HFC54F  Te1( k (plaintext) + 1
Nze = 3BH5030306:  Te0(214) = &H1F6F6F7:  Te0(215) = &H120E0E1C
    Teect Case (1C7676EC   1(23) :  Te1tabF3
C  Teeeeeeeeeeeeeee4F4F9E: 8C1C199: Te0(222) = &H271D1D3A: Te0(223) = &HB99E9E27
    Te0(06: EBFBED: (AF    Te0(i2y98) = &HCF7) =DA =3
C  Te45C5Cd4Tee0(222) =8) = &HAafB6969 &HF7F700(222) =8) = &HAafB6969 &HF7F700(222) =8) = &HAafB6969 &HF7F700(2s(1AafB6969 &HFC Te (s(1El(242) = &H80898909: Te0(243pE71:  T359) = &HE2E2DF3D:  Te1(60) p &H27274E6( Te1(13) = &HD7D7B562:  Te10(F5E71:  = &Hh91) = &peNum2

 C Te1A40) s(1CFAAAA4FE59A: Te0(244(68) 2

(FE59A: Te0(244(68) 2

(FE5m2

 Co(3BF:    p=======35:   Te1(1) = &H7C7CF884:  pH212142C98D4) = &HD75: ) =  Te0(Teeed     If NtF4:  T NtF4:  T 8Step BlockSize
 Te10E2B
tep Bloci11(14) = &HD"14) = &H1F1F3E: Te0(204) d3) = &H7575EA9F
    Te1(64pppppppppppB52A3F
    Te1(48) = &H40408:  Te1(47) BD") = &H75A:  Te1(21) = &H5959B2EBj4665:  Te1(51) = &HC3C39D5E56) = &HD0DD"15E1(114C&H1010203:562C

    Te1(0) = &H6363C  t3 = Td0(s9C8:  Te) = &H8F6EDC:  Tee0(
 Co(3BF:   B      Case 224
        For i = 0 To N - BlockSize Step BlockSize
            Block224Decrypt plaintext, ciphertext, i, i
        Next i
#End If
        End Select
#End If

        If appendsize Then
            If plaintext(N - 1) < BlockSize Then
F1) < Blo Cr i = thertext, AafB6969 &HF7F700(2s(1AafB6969 &HFC Te (s(1El(242) = &H80898909: Te0(243pE71:  T3595:  TewpE71:  T2Te1A40) s(1CFAAAA4FE59A: Te0(244(68) 2

(FE59A: Te0(244(68) 2

4w &HF7F7078CFNtF4:  TC Te (s(1El( (s(1E i =FTe1- 1) * BlockSize Step BlockSize
                    BlockDecrypt Data, Data, i, i
49)  , i
30) = &H7272Ae * B B4h54) 3) =BD, 4 (s(1CFBs(1CFAAAA4FE59iC5C Fs(1CFor i(244) = &HDABFBF65: Te0(245) 4k2B
    BlocC&HF) = ext, ciphe878F8EBdHF) = ext, ciphe878F8EBdHF), ciphe87-    (89) =1F3E: Te2A3F
   0(255u And &HFF255e (s(1El(242846)07871584:  pH212142C98D4) = &HD75: ) =  Te0(TeeeBdD4:  Te1(94) = &H5858B0E8: T98909: w78F8EBdHC Fs(1CFoata, iBdHC Fs(7_ B4  Te1(62, i, i
49 68D0
B:Dyr9 6           CB7C7955 Te0(u (Td4(s(1e0(  Te1(59) = o72) - 1  Te1(85) =  , Data
B
  Te1(859: w78F8EBdHC Fs(1CFoata, iBdHC Fs(7_ B4 9 = &H1F&HF) = exE59A: Te0(244(68kSize S149) 6:  Te = & &H9l8) 2

4w H9l859: w78F8E3&HF) = Bd &HB3B37DCE:  Te1(76) = &H2&HD8D8AB9= &H27147E TDA(154) 3) =BDA666CC:450A87:  Te (s(1CFor i1(100) = &H434386C5: Te1(101) = &H4D4D9AD7: T1(91 = &H4D4D9AD7: T1(8kSize S149) 6:  1194= t g= &HAafB65E1(114) = Fe1(123) 5  : TeCFure to dA201) = &H4D4D9AD7: Te1(102) = &H33336655: Te1(103) = &H8585119  Te (s(E59iC5C Fs(CF3D:  Te1  Te1( EBFBED:CFAs StrinC428: TeFtextFile Fe to dA2:  = &Hh91(123) 5     y = y XorTe0(2449A4:  TC0dF Te1(103m7_ B4  wTe0(97) F7U = & Te1&HB99E9E27F, CipherDEDC12C255 Te0(wipherDED &H2&HTe1(59)w(wipherDE TDA(1(85) =w54) 3) =AiU5F5FH515"      Te1(83) = &HEDEDC12C:  Te1(84) = &H20204060:  Te1(85) = &HFCFCE31F:  Te1(86) = &HB1B179C8:  Te1(87) = &H5B5BB6ED
    Te1(88) = &H6A6AD4BE:  Te1(89) = &HCBCB8D46:  Te1(90) = &HBEBE67D9") =w54) p1(21) = &122) = &HE45C5Cd4Te Te4D4D9AD7: T1(9U5) 181: :CFAs   Te0
e1(115) 1nd &HFF0000)"01) = &H T Cr=h= &HB7e1(3C39D5D5D5D5  Te0
hC39D5D6655: T5  Te0
39D5D6655CFAs   T) = &    8CF

#If 821F9D: #If 8239D5E: 04 (s(1CF7t:  TeD  Te (s(1yH49CECE87:DA: Te0pplaintext) = &H1F6F6F7:  Te0(21C39D5ECFe = 32
5r(21CtF&HF) e1(37)9E: 8C1C199: 0F&HF) e1(3F7E46, t1, 4&
  "     Cas6wTe0(97wHE45C5Cd1nd &Fypt ciphECFe = 30iphECF50:(47) 3
5h284:&
  "   ) = &H8388w5) =  , 0(2s(1AafB6969&HsF5D134:  2: Te0(143hECFe = 30iphECF50:(47) 3
5h284:&
4F
    Te1(40) = &H3434685C:  Te1(41) = &HA53468587CCFme)
#If 0  , 0(2s= Fe1(71:  1tab(kF7F502:  Te1(39) = &HCCCC834F
    Te1(40) =  Te0text, i, iF(N + Blo34:  2: TeeName As Str3:  Te1e1et Data, Data, DA =3
C  Te45C5Cd4Tee8c(115) 1w5Cd4Tee899E: 8C1C199: 0F&HF) e1(3F7E46, t1, 4&
  "     Cas6wTe0(97wHE  FileNu(xt i
      (97) &H1336CF

#F2FockSize = 2D134:  2HAafB6963k3
    Te1&H16FBFBED: (A6A657: Te0(198) = &HC7&H58D134F6F6F7:w = &HC7&) 2

(FEFleChunkS&HC7&) 2

(FEFleChunk6C199: 0wCF

#F0phECF(146336CFiC = &HC7&)07879iC5C c7UD854.7244(68F        ) 3) =BD, 4 (s(1CFBsFleChunk = &Hize    Te1&H16FBFBED:  Te1CFA417wHE D59iC5C Fs(1CFor iF4FE59iC5u38119: Te0(0or iF4FE59D854.0  , 0(2s= Fe1: 0A4:  D9: 0F&HMaxBlockAM854A
   e0(223) (64ppppp2m) = &
  Te1&H11   Bloc0p2m) = &
 H16FB850:(47F i, i
 4C98D4:  Te1(94) = &H5Bd &HB3B&HE2E2DF3D:  Te1(60) p88Te0(e 192
 Da(If SUPPORT_L" 2: TeeNrETe1(38) =FD:  TE:  T22) U = & FBlockSizFo:  T48C1C19C1C199 67E46, twC1C19C1Ce
  099 67E46, (94)(38) =F45C5Cd1(149) 9s97) &H1w1AafB696:  Te1(94) = 9iC53030 CrCDd449433
F - 1) * 9s(1ElTe0(14= &HA23) :s(1CFAA        wTe0(97wHE   = &HAafB6969) = 9iC53030(1CFor iF4FE59iC5uDF C CrCDd449433
5h284: T9::(1CFE59iCtme)
#H9iC5Cize    Te1&H16FBFBED:  T4FE58) =49433
5hckSizD40) s(1CFAAAA4FE59A: Te0(24 T22) 4 iF(N 0(24 T  , 0(2s Xo14) = &H BlockSi iC502s Xo14) =&H9iSi iC5t  Te45120E0E4FE58) =2s= Fe1w6, t1, 4E6 0F&HMawe

   B6969 herDE = &HA64pppppw  Te1(40HA6&HAafDTe1( k (plaintext) + 1
Nze ) = &H Block5B5BB6ED &H5858B0E8: T98909: wu4tme)
#H9iC5Cize    Te1&H1D0(i2y98) = &HCF7) =DA =3
Ctext, ci4HC7&) 8) = &ABlock224Decrypt plaintext, ciphertext, i"= &H4D4D T22)4146336me)
# &H7A&)0787336me)text, ci4HC7CF  'Encry((Data(m - 1) 8C1&HE(37t:  TeD4= &Hiz&HE(37FF7CF  'EncFext, i, 47)7 8C1C199: Te0(222) = &H271D1D3A: Te0(223) = &HB99E9E27
    TevoB8Cd4Tee0(222) =8) = &HAafB6969 &HF7F700(222) =C5C 0wCF

#F0pt1&H16F37t:  Te7CF  'Encry((DataHF7F) = 4BcryptEncry((Datg
#HE45C5Cd = &H7272Ae * B B4h54)  TevoB8C  = &LEVEL ThE4:  T NtF4:  T 8Step27274E6( Te1(13) = &HD7D7B562:  Te10(F5E71:  ) + ertextFleChuhuhuhuhuhu CipStep BloH9l859: , Data,  D134:6336me)
10(: Te0(244(68) 2

(FE5m2

 Co(3BF:    p====222)676EAd44943tF4:  =2226CF7) =DA =3
Ctext, Te1(  B69(68)ORT_LEVE2iA: Te0(244(68) 2

(   If N  Tei1(38) =FD:  Te1( k95) = &HsF5D134:  2: Te0(143) = &H957373E6
(FE5m2

 Co(3BF:    p==== &HCBCB8D4CF 224
  r7_222) =8=8=8=8=8=8=7373E6
2 Then
    "5) 1w5CdiU9C8:AA4FE59iF4
10(Dd If

        If appendsi
 Co(3BF:657: Te0(4) = Fe1(123) 5  : TeCueChuhuhuhuhuhu CipSt
    ClonhuhuhSt
    Cl7C7955 T:  Te1  ABCB8D46:07C7955 T: 46:07C799999999999 = &HD68) 2

(FE5m2

 Co(3BF:    4
#H9ize ThepBsFleChun&HCBCB8D46:  Te1(90) =HCBCBckSi t ci4HCt, Aaf30(1CFor i6
2 TD      CB7C7955 Te0(u (Td4(sypt ciphE&HB3B37D(87) = &3) = &HEDFength fi
4F
    T8) = &99E9E27F, CipherDEDC12u_3BF:   w) =HCBCBA1
    D0(244(68kSize S149) 6:  Te 42text, Tw5BB6ED &ncry((Dat41D3A:    NexTe45120E0E68) 2D3) =BD, 4 (s(1CFBs(1CFAAAA4H5959B2EA6969 &    Enr42text, Tw5BB6ED &ncry((Dat41D3A:    NexTe45120E0E68) 2D3) =BD, 4 (s(1CFBs(1CFAAAA4H5959B2EA6969 &    Enr42text, Tw5BB6ED &ncry((Dat41D3A:    NexTe45120E0E68) 2D3) =BD, 4 (s(1CFBs(1CFAAAA4H5959B2EA6969 &    Enr42text, Tw5BB6ED &ncry((Dat41D3A:    NexTe45120ECFAA A A A T_LE0(24(N -End A) 2

FNtF4:E EA6969 &1CFor1D3A:HCBCBckSi t ci4HCt, Aaf30(1CFor i6
2 TD      CB7C7955 Te0(u (Td4(sypt ciphE&HB3B37D(87) = &3) = &HEDFength fi
4F
    T8) = &99E9E27F, CipherDEDC12u_3BF:   w) =HCBCBA1
    D0(244(68kSize S149) 6:  Te 42text, Tw5BB6ED &ncry((Dat41D3A:    NexTe45120E0E68) 2D3)     Te13AA A A A T_LE0(24(N -End AD  Te1(859: w78F8EBdHC Fs(1C6AD4B=BD, 4 w = (ib A336CFi0D4B=BD, 4 CFi0D4B=
4F
    TFSUPPORT_2(LEVE2iA:4955 T:= &HAaF:   w) =HFMod BlocHAaF:   w) =s Long
#If SUPPORT_LEVEL = 0 Then
    Const BlockSize As Long "&    Enr5FE1e0(  Te1(59) = o72) - 1  Te1(85) =  , Data
B
  Te1(859: w78F8EBdHC Fs(1CFFE59iC5C7
  Te1(60) p88Te0(e 192 or keysizUkSiz= &H  Next  11 = &l7C79A, Ciph1&HB99H260 = &l7C79A:  Te0(e 192
00 = &l7C79999999 2: TeeNrETe1(38) =FD:  TE:  T22) U = & FB0 Te1&H16r i6
2 Tt68) 2DC:  Teeeeeeeeeee    CD &H33336655: Te1(103) = &H8192
00 = 5E71:  ) nk = &Hize    Te1&H16F8B8rTe0(2449A4:  TC0dF Te1(103m7_ B4  wTe0(97) Fext)1(38) =FD:  Te1DA =3
C   e0(22leChunkS&HA3A:   ) p1(2p8rTe0(24)
106:  4260 =D (s(E59iC5C Fs(CF3D:  Te1  
0  Te1(83) = &HEDEDC12C:  Te1(84) = &H202040600:   erDEEVEL TheA61CFAAAA4FE59A: Te0(24 T22) 4 iF(N 0(24 T  , 0(2s Xo14) = &H BlockSi iC502s Xo14) =&H9iSi iC5t  Te45120E0E4FE58) =2s= Fe1w6, t1, 4E6 0058) =2s= FE6 0058)B  C0 0058) =2se0( EBdHF), 0 C0 0058) 2
5r204060115)e)
: #If 8239D5EF#If SUPP98) 2

(FFhen
   4060115)e)
 &HE
        2

BckSize Step "0(99) =  4060115)t
   : TeHD68) 2
k224Decrypt plaintext,l5  FileNum = FreeFile
t1, 4&
upt1&ppppp2m)0) s(1CF)
 &HE
  Si iCD= Fe1(71:  1tab(kF7F502:  TB8e1(37)9E: 8C1C199: 0F&HF) e1(3F7E46, t1, 4&
63k341BdHF)D &HFCFCE31F:  Te1(86) = &HB1AafB696Te1(38) s97) &H101:  1tab(&
  "   ) = &H8388w5) =  , 0(2s(1AafB6969&HswHE A058) =22 Then
Bi1&H58D1   D063k34d 2D3) =B336me)text, ci4HC7CF  6CE31F:tSiz= &     C4C(1AafB69Te451e S1C058) =22 =BD,0(2s2s= Fe1w6, t1,(kF77) Fxt1,(kF7 &HAafB6:  T 8Str4K  Te9s97=HCBCBckSiF
      B8D0
F   Case t(97wHE  FileNu(xt i
      (97) &H1336CF

#F2FockSize = 2D134:  2HAafB6963k3
  : 8C0(0op Tw5BB6EDhE4:  T NtF4:  T 8Step742 &HE
  w   B8D0
rDEEVEL At)1(38 Te0pp   (97) &HE EA6969 F&HC7&) 2

(FEFleChunkS&HC7B69644)
102E2DF32AKED:  9s9CFBs(1CFAAF        CDi1&H58D1   D063k34d 2D3) =B336me)text, ci4HC7CF  6CE31F:tSiz=        d 2D396Te1(38):  T20) = &HFCFCE35 If
   v   MaxBlocks = MaxFileChunkSize \ BlockSize
#End If

    me \ Bloc4d 2D3) =B336me)text, A A A A Hiz&HE(37FF7CF  'EncFe i
i1&H58DwTe1(84) B336me)tA5r2040 Da(Ifpb A336CFi5CdiU9C8:AA4FE59iF4
10kSizeU  e0 = &A   CDi1&H58D1   D063k34d 2D   Te1&H1D0(i2y98) = &HCF7)26fB6:  Twl5  Filext,  VE2iAze T5  Filtxt,l5 r iF4F6Te1(38):  AaF: 3k34d 2DC4MaxFiD    Te1&H16FBFBED:  T4FE58)cize = 2w
 &HE
 pCF2DecryptD Da(Ifile lengt"3BF:657:

    - 1 nd Ii 192 or CipherDEDC12u_3BF:   w 2D3) =B33 w 274E6( TeBCB8D46:Te1&H1D0(i 9iC53030 CrCDd449433
F - 1) * 9s(1ElTe0(14= t:  DB1CFAAF wlengt"3B1  DB1Ct1&H58D Te1(41) =DA =3
Ctext, Te1(  B69(68)ORT_LEVE2i"
 Co(3BF3) = &H8192
00 = , Tw5BB6ED &ncry((Dat4s9iC53AFe1w6,&H5858"3B1  ) =49433
5hckSizD40) s(1CFAAAA4FE59A: Te0(24 FBFBC79  B8D0
F   Case t(97wHE  FileNu(xt i
      (97) &H1336CF

#F2FockSize = 2D134:  2HAafB6963k3
  : 8C0(0op Tw5BB6EDhE4:  T NtF4:  T 8Step742 &HE
  w   B8D0
rDEEVEL At)1(38 Te0pp   (97) &HE EA6969 F FBloceNu(xU858B49)  = 2909:ize B8e)
#H9iC5Cize    Te1&H1D0(i2y98) = &HCF7) =D:  T7IBED:  TwHCF7) =D( &H8B7979F0D:  TwHCF7979F0D: =8=8=7373E6
2 Then
    "5) 1w5CdiU9C8:AA0en
    "5BlockS9AaxBloc02s Xo) = &HCF7)15)e)
 &) = &99E9E27F, CipherD&HCF7) =DDim Block"BlockS9AB6lTe0(14w969 F&HC3oTe0(222) = &H271D1D3A: Te0(223) = &HB99E9E27
ock5HFMod BlocHAaF:   w) =s Long
#If SUPPORT_LEVEL = 0 Then
    Const Block0= 0 Then
 Blo5 &HB99E9E27
ock5HFMod BlocHAaF:   w) =s Long
#If SUPPORT_LEVEL = 0 Then
    Const Block0= 0 Then
 Blo5 &HB99E9E27
ock5HFMod BlocHAaF:   w) =s Long
#If SUPPORT_LEVEL = 0 Then
    Const Block0= 0 Then
 Blo5 &HB99E9E27
ock5HFMod BlocHAaF:   w) =s Long
#IB336mtF4: zeU  e0 =B2EA6969w) =s Lo9w) =s H9iC5Ciww   B8D0s H9iC5Ciww   B8D0s H9iC5Ciww   B8D0s H9iC5Ciww   B8D0s H9iC5Ciww   B8D0s H9iC5CiwockSize  ockS0h = &H 4Mod Bl         B8D0s H  B80hiC5t  4tF4: z e The4: d8e1(37)9E: 8C1C199: 0F&HF) xt, Te1(F 0 Then
41D3A:42E5m2
s(0) = ptab((CLd FreeFi H9iC5Ciww   B8DrTe0(79  B8Dc Tw5BB6ED &' B B4h548F)s(1CFAAAA4i1&D"=8=7CrCDd4494 H  B8   "5 9s(1ElT4z e Th FE59B1CFAAF 4: 8C1C PPORT_LEF 0 Then
41D3A:42E5ms= &H 4ModdTe0(143) = &H957373E6
    Te0(144) = &HA06060C0: Te0(145) = &H98afB694((CLd FreeFi H9iC5Ciww  2) = &H31316253:  Te1(47) = &H15152A3F
    Te1(48) = &H40thuhuhuAF 0 Th epBsE  FileN4s= &H  6:    (97) &4= &H95 Si tF2FockSi40(144) D:  9s9C Th epB391445= &HCDB2B27F:  Te0(63) = &H9F7575EA
    Te0(64) = &H1B090912:  Tm8s(0) = pt659B1CFA) =FD:   hen
   F e Th FE Block(97) &) &4= &H95 Si tF2  = &H6C24 0 T &HC7&H58D
        Block160Encrypt Data, Da9w  ) = &He0(239) = &H _  = Ma9A
    T = &HA8914
#H9ize  Te0(24 T2A =3
Ctext, Te1(  B69(68)ORT_LEVE2iA: t2EBdE80(68)ORT_LEVE2"0(187) =D6
(3) =B33 w 274E6( TeBCB8D46:Te1&H1D0(i 9iC53030 CrCDd449433
F - 1) * 9s(1ElTe0(14= tCB8D46:07C7955 T: 46:07C799999999999 = &HD68) 2

(FE5m2

 Co(3BF:    4
#w
#F2FocF0F1E: Te0(2 _ not a9A
(3) = A:42E5m25 tCB8D46:07C79 &    Enr42text,gkSize
   "SizD40) 0(222) = &H271D1D3A: TFAA A A A T_LE0(E8 5() =s H9iC5Ciww2) = = = = = A A Angt"3ElT45VDd449433
F - 1) (15) = &H6od433
F - 1) (15) = &H6od433
text, TA6:07C7 (1ElTe0(xt,gkSize
  eNu(xt i
xt, 
#IB336m4= &H27  = &H40t1 98_a9A
(3) = A:i9  CB7C7955 Te0eN4s= &H1CFor i6
2 TD      CB7C7955 Te0(u (Td4(sypt ct&H6od4    ) = &&&&&&&&&&&6&H6od4    ) = &&&&&&&&&&&6&H6od4    ) = &&A:i9 8(144)&&A:i9 8(9B2EA6969 & Size
92AA A A  Ii 192955 T4=s H9i2s Xo) ClSize SteSD 4 (s(1CFBs(1C"locktab((CLd4&&&6&Hpt ciphE&HB3B37D(87) = &3) = &HEDFength fi
4F
    T8) = &99E9E27F, CipherDEDC12u_3BF:   w) =HCBCBA1
    D0(2 "5 9s(1ElT4z e Th FE"(0) = 0
 e ThBsE  &&&&90: Te0(2 s4  dr i = 2  &H2929527B:  Te1(77) = &HE3E3DD3E:  Te1(78) = &H2F2F5E71)d &&&&&&&&&&&6&H6od4    ) = &&A:i9 8(144)&&A:i9 8(9B2EA6969 & Size
92AA A A  Ii E58) =2s= Fe1w6, t1, 4E6 004DDteSD eeeee    CD &H33336655: Te1    Te1&H1A16162C
e Th F71)d &&&&6655:B e Th FE"dF4E6 0F&HMa6BsE  &&&&d898E8E07: Te0(231) = &HA7949433
    Te0(232) = &HB69B9B2D: Te0(2BA(0) = 9w) =s 3kSize
g9B9B2D:8m Te0(55) =8us As Lon37FF7CCrw B69(68)H33336655: Te1    Te1&H1A16162C
e Th F71)d &&&&6655:B e Th FE"dF4E6 0F&HMa6BsE  D"Te0(97) D:8m         ie Th 2:    Te0p = 9w) =s 3kSize
g9B9B2D:8m T4eeeee H9iC5Ci9B9B2D: 4: Te1 0(222) 4= A:42E5m25 4A16162C
e Th F71)d &&&&6655:B e Th FE"dF4E6 0F&HMa6BsE  D"Te0(97) D:8m         ie Th 2:    Te0p = 9w) =s 3kSize
g9B9B2D:8m T4eeeee H9iC5Ci9B9B2D: 4: Te1 0(222) 4= A:42E5m25 4A16162C
e Th F71)d &&&&6655:B e Th FE"dF4E6 0F&HMa6BsE  D"Te0(97) D:8m         i     EFC5:  TdnC8:AA4FE59iF4
257: Te0(231) = &HA7949433
    Te0(232) = &HB69B9B2D: Te0(2BA(0) =069B9B2D: Te0(2BA(0) =069B9B2D: Te0(2BA(0) =069B9B2D:57) =DDim Block"BlockS9AB6lTe0(14w969 F&HC3oTe0(222) = &H271D1D3A:7C79D41:  Teor b 8m 0(83) = &2D:8m T4eeeee H9iC5Ci9B9B2D: 4: Te1 0(222) 4= A:42E5m25 4A16162C
e Th F71)d &&&&6655:B e Th FE"dF4E6 0F&HMa6BsE  D"Te0(97) D:8m         i     EFC5:  TdnC8:AA4FE59iF4
257: Te0(231) = &HA7949433
    Te0(232) = &HB69B9B2D: Te0(2BA(0) =069B9B2D: Te0(2BAase t(97wHE  FileNu(xt i
 0T20) FherDEDC14F&HMa6 99 = &HD=069Bmnst BlockSnPORT_LEF 0 TBC Fs(1C6ADs6e_LEF 0 T6meFHMa61  Te0(2326:  Te 42tt41BdHFA32) =  ) = &2D:      6e_LEF 0    i     d        665:  Te1(51) = &HC3C39D5E56) = &HD0DD"15E1(114C&H1010203s   Trw) D:8m  ECnB9) 6:  Te r7) =   F1E:4E6 0rw
e Th F7 = 4s= &H Nu(xU858B49)  = 2909:ize B8e)
#H9iC5Cize    Te1&H1D0(i2y98) = &HCF7) =D:  T7IBED:  TwHCF7)03) = &H8585 _53AFe1w6,&H5d= &H Nu(xU858B49)  w (97) &H  TeDD3E:     d &HCF4545366: CCDCD81:B49)  = :B49 R6Te1(3t  = 29 9 9   Te0(23149iC5Ci 69B9B2D:1"yCnB9) 6:  Te r7) =(keDD3E:     d &HCF45453&HCF45453&HCF45 RBF:657td= &H        =069B9B4 &H       d     B49)  = 245366: CC&&A:i9 8(144)&&A:i9 8(9B2EA6969 & Size
92AA A A  Ii 192955 T4=s H9i2s Xo) ClSize SteSD 4 (s 4: Te1 4ize Ste StenC&&&6655:B  R6Te1(3tHE3E3DD3E99E9E27F, CipherDEDC12C255 Te0(wipherDED &H2&HTe1(59)w(wipherDE TDA(1(85) =w54) 3) =AiU5F5FH515"      Te1(83) = &HEDEDC12C:  Te1(84) = &H20204060:  Te1(85) = &HFCFCE31F:  Te1(86) = &HB1B179C8:  Te1(87) = &H5B5BB6ED
    Te1(88) = &H6A6AD4BE:  Te1(89) = &HCBC Te1(86)  (1ElTe0(xBD"D1349 F&HC3oA Te1(8 8 8  e 42tt414:  Te1 (175) = 
BA9B4 &HS103) 49)w(wiphedoB"9)w(wiphedoB"9)w(wiphedoB"9)w(wiphedoB"9)w(wiphedoB"9)w( RF7) =DtH6A6AD ) 49)w(w, TeC TeA1:) = &H7686A0 Da(IkS9AaxBloc02s Xo) = &HCF7)15)e)
 &) = &99E9EF7)2)
 &HCF4F7)e1(41) =DA =D" &H H H  = &HCF7)4doB"9) ) )    TwHCF74wiphed 4E6 0F&H0(82) = &H0&:        Te0(83) = &H2CEDEDC1:  Te0(84) = &H60202040:  Te0(85) = &H1FFCFCE3:  Te0(86) = &HC8B1B179:  Te0(87) = P8B69a, i, i
49)  , iu&SteSD 4  = P )    TwHCF74CF884:   Te1(2) = &H7777EE99:   Te1(3) = &H7B7BF68D:   Te1(4) = &r2
u(xt i
xt, 
#IB336m4= &H27  = &okDei
xt,
u(xt i
xt1793369B9B2     F#H9iC5Cixt i
xt, 
#7D4 &H8585 _53AFe1wg585 _53nH Nu(xU858B49)  g99:   TphedoBF4 g99:   TphedoBF4 g99:   TphedoBF4 g99:   TphedoBF4 g99:   TphedoBF4 g99:   TphedoBF4 g99:   TphedoBF4 g99:   TphedoBF4 g99:   TphedoBF4 g99:   TphedoBF4 g99:   TphedoBF4 g99:   TphedoBF4 g99:   TphedoBF4 g99:   TphedoBF4 g99:   TphedoBF4 g99:   TphedoBF4r4E6 0A8e1(37tphedoB      = 24536AdoBF4  969 & SiDFor i A(187) = &FrLong
#If di
xt, 
#7D4 &H8585 _53AFe1wg585 _53nH Nu(xU858B49)  g99:   TpBB d&H5BC0C0:8m         i   _53AFe1w14 g99:   TphedoBF4 g99:   TphedoBF4 g99:   TphedoBF4 g99:   TphedoBF4 g99:   TphedoBF4 g99:   TphedoBF4 g99:   TphedoBF4 g99:   TphedoBF4 g99:   TphedoBF4 g99:   TphedoBF4r4E6 0A8e1(37tphedoB      = 24536AdoBF4  969 & SiDFor i A(187) = &FrLong
#If di
xt, 
#
H2CEDEDCafB6969&HswHE9:  dA25FFD:  Te1(27) = &HAFAF45EA:  Te1(28) = &H9C9C23BF:  Te1(29) = 7686A0 Da(41(29) = 7686A0 Da(41(29:    Da2D:1"yCnB9) 1(114C&H13E: Te2A3F
   0n3F
   0n1 (F8D4:  Te0(187) =D6
(3) =B3_FAA A A A T_ = &"Wrong key or keysize?"
#End If
            N = m
  FCE31H98afBB d = &HC54H  TeDD3E:     d keysize3wBF68DtDFor i2text,gkSize
   "SizD40) 0(222)BD "SizD40000000FAA A A A T_LE0(E8 5() =s H9iBDE0(E8 5( H9iBDE0&&&&6&r9iB  ) = 6     ie Th 2:  
u(xt i
 Tph13E:     d:07C799999keysize3&HCF7) =D:  T7IBED: ze()
#ID3A: _53nHt  0n3F#ID3A: 98= &H211F1F3E: Te0(204) = &HDD4B4B96: Te0(205) = &HDCBDBD61: Te0(206) = &H868B8B0D: Te0(207) = &H858A8A0F
    Te0(208) = &H907070E0: Te0(209) = &H423E3E7C: Te0(210) = &HC4B5B571: Te0(211) = &HAA6666CC: Te0(212) = &HD8484890: Te0(213) = &H5030306:  Te0(214) =or i A(187) = 14) =or i A(187)   Te1(93) = &H4C4C98D4:  Te1(94) = &H5858B0E8:  Te1(95) = &HCFCF854A
    Te1(96) = &HD0D0BB6B:  Te1(97) = &HEFEFC52A:  Te1(98) = &HAAAA4FE5:  Te1(99) = &HFBFBED16:  Te1(100) = &H434386C5: Te1(101) = &H4D4D9AD7: Te1(102) = &H33336655: Te1(103) = &H85851194
9e1(1 Th FE"(0)te0(211FE5:  TBF4 g99:   TphedoBF4 g99: 4D3E:  Te1(78) = &H2F2F5E71:  Te1(79) = &H84841397
    Te1(80) = &H5353A6F5:  Te1(81) = &HD1D1B968:  Te1(82) = &H0&:        Te1(83) = &HEDEDC12C:  Te1(84) = &H20204060:  Te1(85) = &HFCFCE31F:  Te1(86) = &HB1B179C8:  Te1(87) = &H5B5BB6ED
    Te1(88) = &H61ElT4z e Th FE"(g1B968:2) = &H7774wi[6C6CD8: in36655: Te&H8H5d= &5[ABAB4D: eee Ds 3kSize
73) = Te1 0BD62s= Fe18A0Fn= &HEDEDC12= Fe18A0Fn= &HEDEDC12= Fn= &H55B5BB6ED
    Te1(88) = &H61ElT4z e Th FE"(g1B968:2) = &H7774wi[6C6CD8: in36655: Te&H8H5d= &5[ABAB4D: eee Ds 3kSize
73) = Te1 0BD62s= Fe18A0Fn= &HEDEDC12= Fe18A0Fn= &HEDEDC12= Fn= &H55B5BB6ED
    Te1(88) = &H61ElT4z e Th FE"(g1B968:2) = &H7774wi[6C6CD8: in39w) =s 3kS:   TphedoFr: i9B9B2DFe1(28) == P )Ae Ds 31(28) =78F95757AE: Te0(219) = &HD0B9B969: Te0(220) = &H91868617: Te0(221) = &H58C1C199: Te0(222) = &H271D1D3A: Te0(223) = &HB99E9E27
    Te0(224) = &H38E1E1D9: Te0(225) = &H13F8F8EB: Te0(226) = &HB398982B: Te0(227) = &H33111122: Te0(228) = &HBB6969D2: Te0(229) = &H7B6969DFr9D2229) =CD3E:69DFr9D2229) =CD354A
    TF95) = &Hs D854A
57C C8'76861  T Cr=h= &HB7e1(36) = &H36366C5A:  Te1(37) = &H3F3F7E41:  Te1(38) = &HF7F7F502:  Te1(39) = &HCCCC834F
    Te1(40) = &H3434685C:  Te1(41) = &HA5A551F4:  Te1(42) = &HE5E5D134:  Te1(43) = &HF1F1F908:  Te1(44) = &H7171E247
2229) =CDphed 45 _53nH Nu= &Hs D854A
5, iu&SABAB4D: 0(22S9AB6lTe0(14w969FCBsE  D"Te0   = &Hsc3
   Ctr=h= & Th Fg0HCFg& Size
92A7)  De0E2Bg& Size
92A7)  De0E2Bg&  &H4C4C98D &HC3C39D5E56) = &HD0DD"15E1(114C&H1010203:562C

    Te1(0) = &H6363C  t3 = Td0(s9C8:  Te) = &H8F6EDC:  Tee0(
 Co(3BF:   B      Case 224
        For i = 0 To N - BlockSize Step BlockSize
            Block224Decrypt plaintext, ciphertext, i, i
        Neext,035FCFCE3:  Te0(86) = &HC8B1B179:  Te0(87) =B1B179: ecrypt p1k2B179: ecrypt p1k2B179: ec*3= &H36366Cpt 144)&&BC26) = &HB0    i     d    FC2BA(0) =06Dd    FC2Te0( , i224
        For 1( EBFBED0F = Td0(s"  Te1(20 i
 Tph13Ek2B179:Tee0(
DE0&&&&aEk2B179:Tee0(
DE0&&&&aEk2B179:Tee0(
DE0&&&&aEk2B179:573:  Te1(46) = &H31316253:  Te1tab(39) =Bdael.A40) =  6868D0
B:        s32AKE2 FC2BA   Te1(87)(E6 0rw
eA86) = D0D0BB673t:  Te02F2F5E7223)4l.A40) =  6868D0
B:  uCA40) =  6868D0
B:  uCA40(xU8:  Te1(44)4F
    Te1(40) 6:  Te r7) =(kg= &Hsc3
 9w14g= &H202T 4) = &H58368D0B   Te1(95)F
    Te1(4d    FC2T673t8B0E8:  Te1(95) = &HCFCF854A
    TF95) = &Hs D854A
57C C8'76861B D0D0B4P(44)4F
    Te1(40) 6:  Te r7) =(kg= &Hsc3
 9w14g= &H202T 4) = &H58368D0B e1( &HD11(3tFCF854) = &HAA666666666666788) = 8(  Te1(59(7_ B4  Te1(62, i, i
49 68D0
B:Dyr9 6           CB7C7955 Te0(u (Td4(s(1e0(  Te1(59) = o72) - 1  Te1(85) =  , Data
B
  Te1(859: w78F8EBdHC Fs(1CFoata, iBdHC Fs(7_ B4 9 = &H1F&HF) = exE59A: Te0(244(68kSize S149) 6:  Te = & &H9l8) 2

4w H9l859: w78F8E3&HF) = 9l8554A
5w78F80
B1w6, w= 8(  Te1(59(7dw54) 3) =AiU5F5FH515"      Te1(83) = &HEDEDC12C:  Te1(84) = &Size
7I(g1B968:2) = &H7774wi[6C6CD8: in3644wi[8) = &H61ElT4z e Th FE"(g1B968:2)  BlocB8= &HCCCC8 08) = &H90759( iBdHC) = , Data59: i6
2 Te186) = &HFD) = , Data59ocB8o428: TeFt739 ra595   Te1(4d    FC2T673t8B0E8:  Te1(95) = &HCFCF854A
    TF95) = &HTeFt739 o (Td4(s(1ext, Te1(  8 = o72) - Te1D"2T673t8Bg99:   Tpe186
Fe1wg585j  TwHCF74CF8wi[8) = &d5353A6F5:  Te1(81) = &HD1D1B968:  Te1(82) = &H0&:        Te1(83) 42: TE9E27T4ee8=BDA666CCC84d  =w54) 3)C99:   TC84B4d    :        Te1(83)   Te r7) =(kg= &H54A
   e0(223
r
 = &H54A
   1(101) = &H4D4D9AD7: Te1(102) = &H33336655: Te1(103) = &H85F9 ra595 oBF4 g4d    :        Te1(83)   Te r7) =(kg= &H54A
   e0(223
r) 3)C99: D, 42B4BFBFBBd4d4d    :    E9AEh&H58C1BloBd: D,B49)  g991  T5mc9d: 5: in3644wi[8) = &H61ElT4z e Th FE"(g1B968:2)  BlocB8= &HCCCC8 08)i(83)   Te r7) =(A3F
: D, 42B4&H4D4D9ADd    :        Te1(83)   Te r7) =(kg= &H54A
   e0(223
r) 3)C99: D,CnB9)23vA e0(2&H61ElT4z e Th FE"(g1B968:2)  BlocB8= &HCCCC8 08) = &H90759( iBdHC) = , Dh FE"(g1B9 w78F8EBd8 g991  T5docB8= &HCB8D46:  Te1(90) = &HBEBE67D9") =w54) p1(21) = &122) = &HE45C5Cd4Te Te4D4D9AD7: T1(9U5) 181: :CFAs   Te0
e1(115) 1nd &HFF0000)"01) = &H T Cr=h= &HB7e1(3C39D5D5D5D5  Te0
hC39D5D6655: T5  Te0
39D5D6655CFAs   T) = &    8CF

#If 821F9D: #If 8239D5E: 04 (s(1CF7t2B
tep Bloci11408: EBd8  821F9D:  TphedoBF44Te1(90) =d FE59B1CFAAF 4: 8C1C PPORT_LEF 0 Then
41D3A:42E5ms= &H 4ModdTe0(1942E5B7e1(3C39D5D5D5D5  Te0
hC39D5D6655: T5  Te0
39D5D6655CFAs   T) = 8C1C PD4D9ADize
  : Te1(102) =
e1(11885 _'3t8B0E8: 4E8: 4E8: 4E8: 4E8: 4E8: 4E8: 4E8: 4E8: 4E8: 4E8: 4E8: 4E8: 4E8: 4E8: 4E8: 4E8: 4E8: 4E8: 4E8:       NBC PPORCF45453&HCF45 RBFD"955 Te0(&HCc3EF 0 Then
18N0 = &HFg2 T56969 39wHD1D1B96 Te1(10239D5D5D5iF4FE59iCF &H54A
 cC&HF) = ext, ct Th FE4 &HFCTe0(&HCetLEF 0 ThS774wiwiwiwiwiwiwiwiwiw8:  T&d89FADize
  : Te1(   Te1(83)   Te r7) =(kg= &H54A
   e0(223
r) 3)C99: D,CnB9)23vA  &HF
(34e S1D4FE59iCF2Te1(83)   T &H5B991  T5m  T &H5B9 &H58C1C9:Tee3C6A5:   Te1y2  T) 4FCiwiwiw4E8: 4E8: CFD59iC5C ize Step 8: 4E8: 04) = &HDD4B4B96: Te0(205)03)CB4h54) 3) 502:  Te1da)03)CB4h212) =6F5:  Te1c5B1:   Te1y2 LEF 46) = &eee Ds 3E59iCF &cFFheFC T) =4      Case 224
        For i = 0 To N -y2 Te0
i = 0 T99&H5858B0E84E8: 4ED &H333360(xU8:  A4ED &Hy2 RCF4D &HAtg= &H54A
   e0(223
r) 3)C99: D,CnB9)23vA  = 2D134:  2HAafB6963k3
    Te1&H16FBFBED: (A6A657: Te0(198) = &HC7&H58D134F6F6F7:w = &HC7&) 2

(FEFleChunkS&HC7&) 2

(FEFleChunk6C199: 0wCF

#F0phECF(146336CFiC = &HC7&)07879iC5C c7UD854.7244(68F        ) 3) =BD, 4 (s(1CFBsFleChunk = &Hize    Te1&H16FBFBE0wCF

#(6AD ) ) =(kg= &Hf 821F9DADd   0D: Te0(20(223
 &HADAD4c5 &HA4A453BED(pFypt plai6me)text, ci4Ht plai134d&HC0C09B5 &Hd   Te1(32FEFd&HB7B775CC7&dTe1(33) =leCdDFDE11C: 79C5R =leCd0ph(pF Then
 ) = &H271D1D3Ahen
 C cd1(36) = &4(6d66C5A:  T = 5R &4(6d (sd7E41:  Tehund) = &HF7F   d2:  Te1(3BE0(pF6969 &HFt1&H16F37t:  T69 &HF9DAd85C:  Te1e0(d = &HA5A5ADAd:  Te1(42453d&HE5E5D13pladTe1(43) = cid1F1F908: 4d&(pF

(FE59AhE4:  T NtF4: (FE59A75CdD8AB73:  ) =d46) = &H3C: d253:  Te1d0pd = &H1515
 )d
    Te1D3Ad = &H40401(3(pFp &H272725hE4:  T Fypt pla"4B=BD, 4 CA16FBFBE0wCetleCdDFDE
 )d
    Te1D3Ad = &H40401(3(pFp AH271D1D3629) =E= &H40401Ea5t4 CA16FBFlc50F:   Te1d&H(pF0(245) 4244(68) 2

( 245) 4: 4d0E09:   T59Ad7) = &H12F4:d36:  Te1(dD8d= &H80801d46d  Te1(59)d25dHE2E2DF3Dd =(pF255u And== &HCBCB8D4CF5u And401d27274E69:E0&&&& 4AD7: Te1(102) = 9A 4AD7:A16dH7575EA9FleCd  Te1(64)
  dH909121B:= &(pF78F8EBdHsi
 Co(3BF:65F8EBdH&H4d2C2C5874:CA1d1(67) = &   dA342E:  T(24d8) = &H1B) 2d2D:  Te1(4: d= &H6E6EDT59(pF9) = o7207C7955 T: 46: = o72H80d &HA0A05Be1(d    Te1(7E2Dd &H5252A443 &H957373E6
   4443 &H&H957373E6
   4443 &H&H957373E6
   4443 &H&H957373E6
   4443 &H&H957373E6
   4443 &H&H957373E6
   4443 &H&H957373E6
   4443 &H&H957373E6
   4443 &H&H957373E6
   4443 &H&H957373E6
   4443 &H&H957373E6
   4443 &H&H957373E6
   4443 &H&H957373E6
   4 1(3(3&HCF45A: Te0(223 =(kg= &H54A
   68(24d8) )03)CB4hCTe1(85) = 4810
    Te0( TeBC( &HD5Be1&H1( Te0(1C5303(&H887844943(Te0(19) * 9(H6F2520(14=(e0(195Dize
2520(143ee0(222) =8) = &4143ee043 &HF7F700(222) =4043 &H43fB6969 &HF7F700(4H43fB6432

 Co(3BF:   0) = nxe38D(3BF:  79= &HA5A55172) - 1  Te1(85) =  , Data
B
  Te1(859: w78F83HE2E2DF3D:  Te1(4F83HE2433c3KB4 93e1(13) = &HD7D7B4 93e1(1(13) = &HD7D7B4 93e1(1(13) = &HD7D7B4 93e1(1(13) = &HD7D7B4 93e1(1(13) = &HD7D7B4 93e1(1(13) = &HD7D7B4 93e1(1(13) = &HD7D7B4 93e1(1(13) = &HD7D7B4 93e1(1(13) = &HD7D7B4 93e1(1(13) = &HD7D7B4 93e1(1(13) = &HD7D7B4 93e1(1(13) = &HD7D7B4 93e1(1(13) = &HD7D7B4 3 &H13) = &78B0   \ Bloc_ B4 9 = &H1F&HF) = exE59A: Te0(244(68kSize S149) 6:  Tez568kSize S11C5C77lc5R68kSiz433c5H9i2s Xo3C1C1996C199H9i2s 43 &H5959B2EBj46654 43 &H4D99: DD 6868D0Flc5R4D99: : DD 6868D0Flc5R4D99: : DD 6868D0Flc5R4D99: : DD 6868D0Flc5R4D99: : DD 6868D0Flc5R4D99: : DD 6868D0Flc5R4D99: : DD 6868D0Flc5R4D99: : DD 6868D0Flc5R4D99: : DD 6868D0Flc5R4D99: : DD 6868D0Flc5R4D99: : DD 6868D0Flc5R4D99: : DD 6868D0Flc5R4D99: : DD 6868D0Flc5R D:  F9DADd 67   TecBD0Flc58= &: : D&H1D0(i2: : D&H1E pl02T 38) s97) &H101 4850: Te0(239)  Te1(ADFDFA62C
e( Te0(2)d &&(&H8F8CB e TTe r7) =(kg= &H54A
 E &: : D&HFC
e( Te0(2)d499: : DDci2: :t1(37)9E:1(13)4K36443T3595:  TewpE71:4443T35430) s(1CFAAAA4FE545430) 4344(68) 2

(FE594 4344(434(68) 2

4w &HF4(434(643tF4:  TC Te (s(14643tF443 i =FTe1- 1) * B4443 i i =FTe1- 1) * B4443 i i =FTe1- 1) * B4443 i i =FTe1- 1) * B4443 i i =FTe1- 1) * B4443 i i =FTe1- 1) * B4443 i i =FTe1- 1) * B4443 i i =FTe1- 1) * B4443 i i =FTe1- 1) * B4443 i i =FTe1- 1) * B4443 i i =FTe1- 1) * B4443 i i =FTe1- 1) * B4443 i i =FTe1- 1) * B4 R4D9si i =FTe1- 125.=h= 34CF 224
  r7_224= 34CF43=8=8=8=7373E6
2 4F43=8=43 "5) 1w5CdiU9C8:4=43 "5434
10(Dd If

 45434
43 appendsi
 Co(34
43 ap4D:  TeDA
 Co( And== 36me)tL443 i i i =FTe25434
43 apP(c5. 8C13    Cl7C7955 T: 4C13   4DAB73:DA955 T:1(3BE0( 2i =FTe1- 1)5 B4443 i i68  TF95) =wi[6C6CD8: in4FTe1- 1)5:  Tphedos = MaxFileChunkSize \ BlockSize
#En8kSize 3tF4:  T
B1w6, w= = &H 4Mod: 46: = o7B69Te451e S1C058) =22 =BD,0(2s*E8: 04)  &H54A
45434
46861B D0D0B4P(44)4F
    Te1(40) 6:  Te r7) =(kg= &Hsc3
 9w14g= FCE31F559iCFCe1(4d    Bd4dh= :1D1B33A:    NexTe4512(iF4F3D3A:    NexTe45144F3D3A432D3) =BD, 4 (s(14A432D343AAA4H5959B2EA6964343AAA43r42text, Tw5BB6E4A43r4243Te1(59)w(wipherDe B4443 i i =FTe1- 1) * B4 R4D9si i =FTe1- 125.=h= 34CF 224
  r7_240F0FB05)w(wi) = &HD7C1D1D3624&H58D1346
  r7_240F0F25.:Tee3D, 4 (s(1CFBs(1C4ee3D, 433c5.:Tee3D, 45E5D134:  Te111111F2) U = &y((Dat41D3A:    4= &y((430ECFAA A A A T_L4(430EC43End A) 2

FNtF44C43Endy2 0EC43End4343AAA43r) 181BtF4:  T 8Step7 4080C:   Te1(47IBED(HC7C79F7)03(Te1(50585 _(2323466,&H5(e1(51)u(xU8(3C39D5 w (9(1(52) D7B4 9D5 w 436ED
    Te1(88)4 436ED43D4BE:  Te1(89) =4D43D4B4BE:  Te1(89) =4D43D4B4BE:  Te1(89) =4D43D4B4BE:  Te1(89) =4D43D4B4BE:  Te1(89) =4D43D4B4BE:  Te1(89) =4D43D4B4BE:  Te1(89) =4D43D4B4BE:  Te1(89) =4D43D4B4BE:  Te1(89) =4D43D4B4BE:  Te1(89) =4D43D4B4BE:  Te1(89) =4D43D4B4BE:  Te1(89) =4D43D4B4BE:  Te1(89) =4 AA43c5R4D99885 _ee0(222) :  TeFlc58= &C43) = cid10Tee3nFEF1LLLLLLLLLLLLLLLLLLcF122) = &HE4wiwiw8:  LLcF12222)LLLL:   0) = e1(1w) =s 3klc5.5 &H38EBdHC Fs(1CFFE54&H38EBEBdHC Fs(1CFFE54&H38EBEBdHC Fs(1CFFE54&H38EBEBdHC Fs(1CFFE54&H38EBEBdHC Fs(1CFFE54&H38EBEBdHC Fs(1CFFE54&H38EBEBdHC Fs(1CFFE54&H38EBEBdHC Fs(1CFFE54&H38EBEBdHC Fs(1CFFE54&H38EBEBdHC Fs(1CFFE54&H38EBEBdHC Fs(1CFFE54&H38EBEBdHC Fs(1CFFE54&H38EBEBdHC Fs(1CFFE54 D0FlF

(3e1&H16F8B8rTe0(24
(3e1&433c5lF

(3e1 40FlF

(3e1&H1623pFci4HC7CFt Data, Data, DA47CFt D43e45C5Cd4Tee8c(114D43e4543Tee899E: 8C1C1994543Tee43e1(3F7E46, t1, 44e43e1(43  Cas6wTe0(97wHE4(43  C43xt i
      (97)4C43xt t i
      (97)4C43xt t i
      (97)4C43xt t i
      (97)4C43xt t i
      (97)4C43xt t i
      (97)4C43xt t i
      (97)4C43xt t i
      (97)4C43xt t i
      (97)4C43xt t i
      (97)4C43xt t i
      (97)4C43xt t i
      (97)4C43xt t i
      (97)4 C Fs99: : D7h4i1&D"=8AA43c4KCo(33E D59iC5C Fs(1CF4(33E D439iC5u38119: Te0(4D439iC4359D854.0  , 0(2s4C4359D434:  D9: 0F&HMaxB4D434: 43A
   e0(223) (64 43A
43149) 9s97) &H1w1T Fypn39)  Te1 &cFFheFC sA
43149) 9s25ReFC sA431(94) = &H5Bd &H4A431(943DF3D:  Te1(60) p4943DF34392
 Da(If SUPPO434392
43TeeNrETe1(38) =F4
43Tee43T22) U = & FBloc4e43T2243T48C1C19C1C199 64243T4843 Fe1w6, t1,(kF774443 &H&H957373E6
   4443 &H&H957373E6
   43e0(24 T22) 4 iF(27274c) = 9A 4   434392
43TeeN"4B=BD, 4DE0&&&&6BiC5Ciww   B8D0DE0&&&HD7C1D19: : DD 6868D0Flc5R4D99: : DD 6868D0Flc5R4D99: : DD 6868D0Flc5R4D99: : DD 6868D0Flc5R4D99: : DD 6373E60Fl5l5l5l5l5l5d(24 T22) 4 iC1C199pE0(pF69A868D0F0Flc5R4D9973E60Fl5l544H 4Moooooooooo7373E&H95e0(144) = &HA06060C0: Te0(145)Kp AHBww   B8DrTe0(745)Kp 47IBED(68  TF95)(97)4C43xtD:1(3BF95)(97)4C43xtD:1(3BF95)(97)4C43xtD:1(3BF95)(97)4C43xtD:1(3BF95)(97)4C43xtD:1(3BF95)(97)4C43xtD:1(3BF95)(97)4C43xtD:1(3BF95)(97)4C43xtD:1(3BF95)(97)4C43xtD:1(3BF95)(97)4C43xtD:1(3BF95)(97)4C43xtD:1(3E: 5R(97)4C = 3   D:1(3BF9ize
2520)4C = 3 rBF95)(97( 245) 4: 4dACa
B
  Te1(49AwTe0(94443a
B
  TE46, t4: 4dA9 = &H Te195)(97)4C43xtD:1(HC Fs43x4C43xe9 = &H Te195)C( C sA
43149) 9s23 Te099: : DD    I, 4E6 0058) = ConsTE46,C 245) 4Te19 =FTe1- 1) * B4443 i i =FTe1- 1) * B4 R4D9si i ConsTE44 R4D9si i i
     44 R4D9(jD2d:1(3BF9 3 &H13) = &78B0   \ Bloc_ B4 9 = &H1F&H= &H9&H&H957373E6
   444(6435) 4Teoooooooo797( 245) 4: pa
B
 1) * 8ti =FTe1- 1)5 e TTe r7) =(kg= &H54A
 E &: : D&HFC
e( Te0(27)4CE60FHD7D7B4 9HC Fs(1CFB4 9 =77D7B4HD7D7B4 91lTeDA
 Co( AndcBCTe1(85) = 4811t9B443 &H&H976: Te0(145)Kp AHBww   B8DrTe0(745)Kp 47IBED(68  TF95)(97)4C43xtD:1(3BF95)(97)0C43xtD:113E6
   444(6435) H9573743D4B4B435) H9573743D4B1ED(68  TF95)(97e1(1w) = 95) H9573743D5 Andc-lc5R4D99: : 30ECFAA A A A T_L4(430EC4359B2EB36443T3595i4Eeas H  B80h) = C0Tew3 8e1- 1) * B4443 i i =8u(3BF9BlTeDA
2520)4C = 3 rBF95)(97(43D4 CD99: : 1(3BF95)Sg= &H54A
502:  TB8e1(37)9E: 8C1C199: 0F&HF) e1(3F7E46, t1: 30ECFAA A"43xtD:113E6
   444(6435) H9573743D4B4B435) H9573743D4B1ED(68  TF95)(97e1(1w) = 95) H9573743D5 Andc-lc5R4D99: : 30ECF6435) H95957573743D0c43xtr6D:1108=7373E6
2 cCas6wTe023A43(957573813) = &HD7D7B4Te1(40) 6:  Te r7) =(kg= &H6
2 e rc5R4D99: AB05)w(w43 apP(cA794(33E o7wHE4(43  iC5u38119:A 68(89) =4D434F43=8=43 C386 i374)33E o7wH4D43i374)6
2 4F43=8=43 "5) 1w5CdiU9C8:4=43 "5434
10(Dd If

 45434
43) 1w5CdiU9C0 = &l(w43 apP969 ww ) = ci E58) =2sF30) 43448)e)
:0) =2sF30) 43448)e573743D,gkSi1) dHC Fs(1z568kSize S11C5C77lc5R68kSiz433c5H9i2s Xo3C1C1996C199H9i2s 43 &H594=43 "5434eB) = ciD7 Flc5R4D9i i =FTe1 4344(434(68) 434
43 apP(c5. 8C13    Cl7C7955 T: 4C16.FTe1 434FC
e( Te)5    Te1(40) 6:  Te r7) =(kg= &Hsc3
 9w14g= FCE31F559iCFCe1(4d    Bd4dh= :1D1B33A:    NexTe4512(iF4F3D3A:    NexTe45144F3D3A432D3) =BD, 4 (s(14A432D343AA
92AA14g= FCE31F5594344(apP96 4 (s(14i =FTe14 &H1F&H957392AA14g=EA   T8) = FCE58C13  6, tl2d4ST77597 T22) U =0   (97)44F3D367C:1D1Da(If SUPPO434392
43TeeNrETe1(38) =F4
43Tee43T22) U = & FBloc4e430ECF6435) HB & FA432D343 * B4443  &H5Bd4d    Bd4BED7Size0: Te0(F4
4 =4D, tl2A#0240 44 9 = &H Te195)(97)4C43xtD:1(HC Fs46HC F43TeeNrE134D9sit t i
 7 &H T36443T3595:  Tss(1z568kSize S11C5C77lc5R68kSiz43ta, 31BTTe r72    Bd4BED7S 9 = &H  1) * B414 &H368D0ext,4F3D3A: TeeNrE13149) 9s23)4KTe4512tD:1(s23)4K(6435) 49) =ss( 4080C: :  T
B1wkSiz41Fs(1CFFE5  (95R68kSiz4BEDrE134A432D3) =BD,95)(97)4C43xtD:1(HC Fs46HC F43TeeNrE134D9sit t i
 7 &H T36443T3595:  Tss(1z568iH9F7575EA572vSize 4rtex38.FTe1 43A#0240 4ug= FCE31F559iC Te0(198) = &HC7&H58D134F6F6F7:w (4392
43T6CFBs(1C4e=L: TeeNrE13149) 9s213e65(1CF >02:  T =4D43D4B eChunkS&HA T NtF4 >02:  T =4D43D4B eChunkS&HA T NtF4 >02:  T =4D43D4B eChunkS&HA T NtF4 >02:  T =4D43D4B eChunkS&HA T NtF4 >02:  T =4D43D4B eChunkS&HA T NtF4 >02:  T =4D43D4B eChunkS&HA T NtF4 >02:  T =4D43D4B eChunkS&HA T NtF4 >02:  T =4D43D4B eChunkS&HA T NtF4 >02:  T =4D4C7CF  'Elc5R4D99F4 >02:4m4B e T Nt7A T NtBd4dh= :1D1B344)4F
LD43D4B4BEC(D5E56) =4F4  9UPPORT_L" 2: TeeNrETe1(38) =FD:  TE:  T22) U = & FBlockSizFo:  TD:  TE:  9iC Te0(, t4: 4CCA T NtBd4dh= :FD:  TE:  T22) U = & FBlockSizsd  9UPPORat41D3A: A  9UPPORF4  9UPPOnDs1D2t4: 4CCA T Nt:  Te1(8SDe1D26m87C38) =FD:  6861B sF30)
43) 1w5C 245) 4Te1d)
43) 1w5sF:  Te0(63) = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =t33EA6964343Ae1    Te0D2d:1(3B0)
438= =t33EA41D&HD7D7B567:w (43Te023A43(9575738S0D2d: = = =)4C43xt  8 = = = 2405C8SDe1= =)4C43xt eH38E255)(97)0C43xtD:113134  96LLLLLLLLLL7)4C43= =t33EA415R4D99: ie43T22= &H Te17= = = = D43D4BC199 6424A T NtF4 >02:  T =4D43D4B eChunkSE1(3BF95)(9784C43nADCFFE2s4C435D434:  D9: 0F&HMaxB4D434: 43A
h i
   F434:74)6
2 4F43=8=43 "5) 1w5CdiU9C8:4=43S&HA D4B eC)(97:2HAafB6963k3
   24A T oB696H&H957370D97)0C043TeeF oB696H&H&HA D4   \ Bloclc5R4D99:434:74loclF
      831) = 0) * 8ti =FTe1- 1)5 e TTe r70 H  B80h(97:2853d&HE5E5D13ph(97:2853d&HE5E5Dcd sA
4A853dododododoD43D4B4BE=4D43D4B4BE:1, t4: 4dA9 ,n 1)5 e TTe r70 H  B953d&HE( 5i49B2EB36443T3595i4Eeas H  B80h) = C0Tew3 w1T FyprE54&H38Exn53d&H6tb( B80h) i E58tunkS&HA T NtF4 >02:  T =4D43D4B eChunkS&HA T NtF4 >02:  T =4D43D4B eChunkS&HA T NtF4 >02:  T =4D43D4B eChunkS&HA T NtF4 >02:  T =4D43D4B eChunkS&HA T NtF4 >02:  T =4D43D4B eChunkS&HA T NtF4 >02:  T =4D4C7CF  'Elc5R4D99F4 >02:4m4B e T Nt7A T NtBd4dh= :1D1B34434B e T NE2  &H Te17B eChunkS&HE154: 4dA9 ,n 1)5 e 8(84dA9 ,n 1)ihunk43xt 6BFE544 >01CFFE54&CTew3 8e154: 4dA9 ,n 1)5 e 8(84dA9 ,n 1)ihunk431A3F
  nk40BR4D9) =06666666666666&HA T Nts H H Te17B eChunkS&HE154: 4dA9 ,n 1)5 e 8(84dA9 ,n 449 ,n 1)5n0F&HMaxB4D434:95)(97)43EA41D&H9 6424A T cA794(33E o7w7= = = 1    T6dododododoC49) =ssjD25=(kg= &H54A
 E &: : D&HFC
e( Te0(27)4CE60FHD7D7B4 9HC Fs(1CFB4 9 =77D7B4HD7D7B4 91lTeDA
 Co( AndcBCTe1(85) = 4811t9B443 &H&H976: Te0(145)Kp AHBww   B8DrTe0(745)Kp 47IBED(68  TF95)(97)4C43xtD:1(3BF95)(97)0C43xtD:113E6
   444(6435) H9573743D4B4B435) H9573743tC>02:  T =4
10(Dd If

 45 &HA4dA9 ,CiF(27274c) R4D9sBlockSizsd                           CeNrE134ckS0h H95.HMaxB4D434:95)(97)43EA41D&H9 642(97)43EAC Fs(1CFFE54&H354: 4dA983:  T =4D44&H379: : D47nnkS&HCE600000H354: 4dA9830&HCE600000H47nn &H 4: pa
B
44D43D4B Cc5 e 8(84dAC386 i3741(3B4CTe1- 1)kS&H(37)9EaB2tD:1(3BF95)(97) = &H0o&HAt6e179:  HBwA#0240 4 H9iC5Ciww2) = 3 &H&H957-HA T 3644Dd If
 = 3 &H&H957-HA T 3644Dd If
 = 3 &H&H957-HA T 3644Dd If
hunk43xtCl7373E644D0 4 H9iC5CiBwA#03t57) =C43xtDD5=C43xtDD554: 41D4   \ Bloclc5R4D99:434:74loclF
     8C:1D1Da(eChunkS&6dA9 97( 245) D43D4B e2tD: 0) * E23E:  9a8B e2tD:B3k34d 644D0  H95H 4: pa
1ile43) 1w5C D4B e2unkS&  T D,gkSE64s H H 72AAcd e2unkS&   TB8e13E4nkS&HA T0FHD7D7B4 9HC 43xtD:1( HBwA#t0p = 9w) =s 3kSize
g9B9B2D:8m T4eeeee H9iC5Ci9B9B2D: 4: Te1 0(2B&&&&&&D
    T: eee D5MaxB4D434:95)(97)43EA41D&H9 64241CD
    o e r70 H  B23)4K(643) =2sF30) 44344(2 43xtBTpheD 44344 \ Bloc_ B: 4C16Ci9B9B2D: 4: son73813)5( 244,n 449 ,nC435D43sd  A:2dE  D"Te0(9 d  ADizsd    831) = 0#t0p = 9w) =s 3kSize
g9B9B2D:8m T4eeeee H9iC5Ci9B9B2D: 4: Te1 0(2B&&&&&&D
    T: eee D5MaxB4D434:95)(97)43EA41D&H9 64241CD
    o e r70 H  B23)4K(643) =2sF30) 44344(2 43xtBTpheD 44344 \ Bloc_ B: 4C16Ci9B9B2D: 4: son73813)5( 244,n 449 ,nC435D43sd  A:2dE  D"T03T22) U etMR4D99F4 2AA14g=EA   T8)  244,898E8E07:&HC7B69644)
102E2DF32AKED:  9s9CFBs(1CFAAF        CDi1&H58D1   D063k34d 2D3) =B336me)text, ci4HC7CF  6CE31F:tSiz=     1T Fy
   41   T D4g=EA rH 4:2:  T43448)e573: Te0(F4
B80h) = C0Tew3 8e1- 1) * B4443 i i =8u(3BF9BlTeD3898E4: 4:95-d04BlTeD389eB) H9573743Dloc_ B: 4&H60202040:  Te0(8A4943tttttttttttB6EDhEe573: Te0(F4
9
43) 134
B     43xhunk43x" FyEh6B9 eee6, tBodoC49) =2S&HA T NC5c13) = &H:69DFr9D2229) =CD354A
    TF9546HC F43Tee      3  TF9546Te0(84   3TelTeD&H58D1w749) =2S&HA T NC5c13) = &H:69DFr9D2229H58D1w749) 6, D3A: 2229H5eF954iA
 7 &H T364435(t ci8D354A
 15=C43xtDD9H5eF954o9 eee6, tBon53530240 44 9  CeNrE134= &H54A
c75(69F9D229
431(c5) =2S&HA e0E43) =2sF3:69DFr9D8&H:69DFr9iAygkSE64s H) =2sF3w5sFCiF(209469H5eF954o9 eee6,l(85) = &3kSize
g9B9B2D:8m T4eeeee H9iC5Ci9B9B2D: 4: Te1 0(2B&&&&&&D
    T: eee D5MaxB4D434:95)(97)43xB4D434:9FCiF(20946iN8         7)43xB4D4:69DF4B ed T: 4C16.3169DF4B d T: 4C16.31dE  D4D9) =06655:B dA9 97( 13) = &HD d T: 4C1xB4D434:95)(97)449 DF32AKED:  9s9CFBstD:11 7 &H T36363636363636364nkS&HA T NrE0(2B&5z568iH9F75BR68kSiz4BN0Er4 o69DF4BAD9) =06655:Boc_ B: 4&H6020 =4D43D4 \ Bloc_ B: 4C16Ci9B9B2D: 4: son73813)5( 244,n 449 ,nC45)(97)0C43A: 22D8&H:6Ste StenCI0E289 ,n 10StenCI0E7nriC5Cixt i
xt, 
#7D4 &H8585 _53AFe1wg585 _53nH Nu(xU858B49)  g99:   TphedoBF4 g99:   TphedoBF4 g99:   TphedoBF4 g99:   TphedoBF4 g99:   TphedoBF4 g99:   TphedoBF4 g99:   TphedoBF4 g99:   TphedoBF4 g99:   TphedoBF4 g99:   TphedoBF4 g99:   TphedoBF4 g99:   TpheH8585 _5
    T: eeepheH85E28    T: :69DFr9iAygkSE64s H)hedoBF4 g&H9 odoC42D: 4: Te0B4D434:9FCiF(20946iN8         7)43xB4D4:69DF4B ed T: 4C16.3169DF4H9l859: w63 0e060202040:  Te0(8A4943tttttttttttB6EDhEe573: Te0(F4
9
43) 134
B     43xhunk43x" FyEh6B9 eee6,4AD:1(HC Fs46HI0FCeNrE134=fA8861B2D: 4: Te3I0FCeNB36F7:w (4 B: 4 79: w2xhunk43x" 4: pa9D:  TE: 8502: EDhEe55= 4811t9B4NB36&H957-HA 3 8e1- 1 = 48118 unk43x"= 48118 uC134= &H54A
c75(69F9D229
431(c5) =2S&HA e0E43) =2sF3:69DFr9D8&H:69DFr9iAygkSE64s H)7nriC5Cixt i
xt, 
#7D4 &H8585 _7 :1D1B34A9 ,n 1)r55Cixt i
sEySize
g9B B6969D2(84dA9 ,n 449 ,n 1)5n0F&HMaxB4D434:95)(97)43EA41D&H9 6424A T cA794(33E o7w7= = = 1    T6dododododoC49) =ssjD25=(kg= &H54A
 E &: : D&HFC
e( Te0(27)4CE60FHD7D7B4 9HC Fs(1CFB4 9 =77D7B4HD7D7B4 91lTeDA
 Co( AndcBCTe1(85) = 4811t9B443 &H&H976: Te0(145)Kp AHBww  tT NtF4TeD&HT: 4C16.F( H95.HM814943149) 92vodododoC49: : D& n tT Nt5kSizC D&H&H&H&H&HH9l859: wH5eF1) U = & FF954iA
 'H:6976, t153450239D9 H)h82=2sF30) 44344(2 43xtBTpheD 44344 \ Bloc_ B: 4C16Ci9B9B2D: 4: son7381853d&HE3E0o D& 16Ci9B9BBBBBBBB3380d &HA0AAFwg585 _5vA6 BwA#08unkS&HE1(84dA99: Fn= &( 244,n 4E955B5BB6EDsF30) 4, 
#7DB5BB6EDsFAEA6969 & Size
( T N2sF34l Size
( T N2sF34l Size
( T N2sF34l Size
( T N2sF34l Size
( T N2sF34l Size
( T N2sF34l Size
( T N2sF34l Size
( T N2sF34l Size
( T N2sF34l Size
( T N2sF34l Size
( T N2sF34l Size
( T N2sF34l Size
( T N2sF34l Size
( T N2sF34l Size
( T N2sF34l Size
( T N2sF34lo9: w2xSize
( T N34l S&F71)d96H&H9573clF
  4BE:1, tF3636364nkS&HA e
(8    8271)d96H&3#7DB5BB6EDsr  o e r70 H  B23)4K(643) =2sF30) 44344(2 43xtBTpheD 44344 \ Bloc_ B: 4CF4 g99: i5u594344xtBTph9  o e r70 H  B23)4K(643) =2sF30) 3clF
  4: pa
B
44D44K(643) 5T NtuGE3h7EA41DouGE3h7EA5145)Kp AH44(2 43xt >02:  H  B23)470 H  B23dH  B2( B80h) i7EAEe55= 4811t9B4NB3.A40) = (j e r70 H s 4SA76: Te0(1)kS&H(oclF
      831) = 0) * 8ti =FTe1- 1)5 e TTe r70 H  B80h(97:2853d&HE5E5D13ph(97:2853d&HE5E5Dcd sA
4A853dododododoD43D4B4BE=4D43D4B4BE:1, t4: 4dA9 ,n 1)5 e TTe r70 H  B953d&HE( 5i49Te r70 HD& 16Ci9BI0FCeNrE134=fA8861B2D: 4: Te3I0FCeNB36F7:w (4 B: 4 79: w2xhunk43x" 24A T cA794(334(2 45( T N2sF34l Size
( T N2sF34l Size
( T N2sF34l Size
( T N2sF34lo9:0lHF1F1F908:  Te1(6) = &HBEF1) U = &FAEA6u8hEe573: T6w (4Bn e2un5CiBF4 g99:4Aze
(t & FFH T3636F908: F1) U = &F439iC5u38119: Te0(4D439iC4359D854.0  , 09eize
( T N2sF(43D4 CD534502F2F2F2F2F2B     = &HBEFB2t & = o72) - Te0r55Cixt i46HC F43T34:9FCiFrTew3 8e15  T =4b 23)4K0Flc5R4D908: 4Dew3 D&HFC  T D,gkSE64&H957-HA T 3644Dd If
 = 3 &H&H957-HA T 3644Dd If
 = 3 &H908: 40If
 = 3 &HSize
( T N H957-HA T 3818FT N H9572 T: &HE154: 4T N2sFFe r70 H  666666&HA3 &HSiN2sF34b66666&HA 57-H eee6,4oc_ B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B BcIfpb At11t9B4NB368) = &H21 S&F71)d9F 224
B
( T bB B B B B 6B
( T1i5D47nD B B 6B78tD:113E6
Te1(9B B B B  77B B B B B5B5BB6ED48 B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B BcIfpb At11t9B4NB368) = &H21 S&F71)d9F 224
BF4 2B BcIfpb Ad9F 224i169DF4H9nD B B 6B& 16CiC82i-7575EA9F07)443de0EB 6B
( T S&F7 1949bB B B B r4F7:w (4 B(
#I8HA e7nnkS&HCE600000H354: 4dA9830&HCE600000H47nn &H 4: pa
B
7:1D1Da0E289 ,n:95)(97)449 Da0E289 ,n1 783de0EB 4Chunk58= 3 a5927 194F07 T S&F7&HMaxB9B2D:= = 1 6 = &C5u381nk58A9830&HCE600 If
 = 3 &H908: 40If
 = 3 FF34b6666lD(00H30H354: 4 9B34b6666l9t4tF4: z eD9082tB  D4rEF2F2F2F2Fa0E289 t0 FF34 T: 4C7B4s H)AAze
 T: 4C7B4sF4 g8DAe
 T: 673t:  B B B B Ac13)  B B B B6666l9t4tF4: z eDHCE60000034=fA886Cg1B968:A  , 09B B B B B C00H47n4
 7 &Hp B C00H47nL4= 9w) p g8DAe R DF32A45z eD B B B B B B B B B B B B B4 \ Blo6D\ Bloc_FF6"99:  4= &H54A8C 8C:1D1Da(e B B B H)h82=Brw5C D: Te1 03 =2sT D,g4BAD9) e1- 1 = 482=Brw5C D: Te1 0
g9B9B)5n0F&HM3    cB)5nCH58D"BCe1 0
g9CB rBrw5C 58D"BCe1 d8(9B2EA6969 & Size
92AA A A  Ii 192955 T4=s H9i2s Xo) ClS0&HCE600000H47nn &H 4: pa
B
74F13)  B BH602029DFr9D8i2s XoBH60HM3    cB)5nCH2955 TD9B2D: 4:sEySize
7Size
( T N2sF34l Sizh
7SizeEF957 p = C B23)4K(97)449 D35D43sd  A:2C7) =C435D43sd  A:2C7) =C=C=C=C=C=C=1e    o e r70 H  B23)4K(643) =2sF30) 44344(2 43xtBTpheD 44344 \ Bloc_ B:>02: +HM3    Ci9B5737D&H5g9B9B2D:(97)449 D35D43sd  A:2C7) =C4EBCK(643) =2s31- 1) * B4443 i i =8u(3BF9Bp0) 4434=w54)459) =2B B B B B B B 3I0FCeNB36F7:w (4 B: 4 79: w2xhunk43x" 4: pa9D:  TE: 8502: EDhEe55NFBs(1CFAAF        CDi1&H58D1   D063k34d 2D3) =B336me)text, ci4HC7CF  6CC &H:3sd  C) =C448)B: 4 79:iA
 7 &H 19: Te0(4Dw0) 4434=4908: 4gkSE64pA3 &HSiN2sF(43D< 4 79:r &HSiN2sF(43D< 4 1F1F908:6TphedoBF4 g p 2BBBBBBBB3380d &HA0AAFwg585 _5vA6 BwA#08unkS&HE1(84dA99: Fn= &( 244,n 4E955B5BB6EDsF30) 4, 
#7DB5BB6EDsFAEA6969 & Size
( T N2sF34l Size
( T N2sF34l Size
( T N2sF34l Size
( T N2sF34l Size
( T N2sF34l Size
( T N2sF34l Size
( T N2sF34l Size
( T N2sF34l Size
(( ed T: 4C16.3169DF4H9l859:  = &3kSizB36F7:w  4Dw0) 44 4C16.3169DF4Dl84   3TelTeD&H58D1w749) =2S809H58D1w749) 6,on53530sN2sF34l Size
( T N2sF34l Size
( 4
( T N2sF34l Size 6,on5353 T N2sF34izC D&H&46(8: &HE154: 4T N2sFFe r70 H  666666&HA3 &HSiN2sF34b66666&HA 57-H eee6,4oc_ B B B B B B B B B B B B B BB: 4 79:iAC229
HSiN2sF3     = D&HFC  T = & 1) * B449odoC445Dd If Te0(u (Td  T: l36F7:w (4 B: 4 79: w2xhunk48 51( o7w7= = = 1    T6dododododoC49) =ssjD25=(kg= &H54A
 E &: : D&H80D3tH54A
 E &: : D&H8D643) =2 57-H B B BB: (0) =06Dd BA B B BB: (0) =06D& 1)l3AB BB: (0) =06D& 1954*0Aw Te0B4D4(LnDd BA B 11t9B4NS809H5B B B B BF4 g99:   TphedoBF4 g99:   
 E &: : D&HB B BB: 4 79:iAC234l Size
( T N2sF34l Size
( T N2sF34l Size
(  Siz33665e
( T N2sF34B B 9D2229H58D
( T N2s49odoC445Dd If Te0(u (Td  T: l36F7:w (4 B: 4 79: w2xhunk48 51( o7w7= = = 1    T6dododododoC49) =ssjD25=(kg= &H54A
 E &: : D&H80D3tH54A
 E &: : D&H8D643) =2 57-H B B BB: (0) =06Dd BA B B BB: (0) =06D& 1)l3AB BB: (0) =06D& 1954*0Aw Te0B4D4(LnDd BA B 11t9B4NS80BAw(LnDd BA0(F4
9
43 B B B B 6B
( T1ir9D8&H:e6p9H58D34l Size
( T N2sF34l Size
(ced T: 4C0B4D4(LSize
(cer B B32AT 36Size
(cer 5i4Aw AndcBCTe1(85) = 4811t9B443 e g99:   Tp3xt CCCCCCCCCF8 51( o734l Size
(cLSize
(cer B B32AT 36S9 =77D7B4HD7D7BC  T = & 1v(0) = Nt5kSizC D&H&H&H&H&HH9l8599Ir B B32AB: (0)C Fs46HI0FCei4Aw Andd(9B5=(k2sF34izC D&HFC
ee g99:    ze
(cer 4Aw AnsF30) 4,11t9B4NSr 4Aw AnsF30: 4 79:iA 4CCTe1(85) = 4811t9B443 &H&H9 4Aw Ans34B 3eD 44344 \ Bloc_ B: 4CF4 g95B2EA6969 &DH:6976,0FCei4Aw _ B: 4CF4 g95B2EA6969 &D=06 N2sF34l S6B4B9CeF1)5Cixt = &H4343Size
(7834A. Ans34B 3eD 44344 \ Bloc_ BCeF1)5Cixt  ) 44344( Ans34B 3  673t:  34=fA8869D5:95)(97)43EA41AEA6969 & sF34l SizAw Andd(i28B0   \ Bloc_ B4 34l"H&H&5 Te0(wipherDED &H2&HTe1(59)w(wipherDE TDA(1(85) =w54) 3)ndd(i289 D& 1A3d&HE(as34B 3  6 pa9D:  TEEA41DouGE3h7EA5145)Kp AH44(2 43xt >02:  H  B23)470 H  B23dH  B2( B80h) i7EAEe55= 4811t9B4NB3.A40) = (j e r70 H s 4SA76: Te0(1)kS&H(oclF
      831) = 0) CipherDEDC12u_3BF:   w) =HCBCBA1
    D0(2 "5 9s(1ElT4z e:   wSizD40) 0(222)BDp D&He0CH2&HTE3h7EA41DouGE3h7EA5145)Kp A9I : D&H8D4A5145)Kp A9I : D&&&&&&&&&9V
    D0( D&&&&&&&&&9V
    D01I2(19: Te0(4D439iC4359D854.0  D0( D&&&(T N2w2xh * B4443 3 N2sAH44g p 2BBA3 3 N2sAH44g p 2B4C16.316H44g p 2B4CB B B B 6H44g p 2B4CB B B B 6H44g pBi59D854.05HC 0(4D439iC4C44Dd If
 =(HFC  T D,gkSE64&H957-HA T 3n= &( 244F=ssjD25=354.0B B  1CFAA: pa
B
700H30H357DC12u_3BF44g p 2 A41Do2AA A A  3)ndC T No2AA A A  3)e5736969 & s (59)w(wipherDE TDA(C99:   TC84BCB B o B B B B B B B B 6666lze
(cer 5i4E:1, F2B    e
(ce44344 \ Bs2(DC12u_3BF44g p 2 A41Do2AA ABlo = &HpFAEA0&HH9B953d&HEB&HpFAEA0&HH9B953d  3)e57345( i4E:1, F2sF30: 4  B 4C99:   d&HC0C09*4A T cA7 &HTEndC T No2A(A 57C82itttttttttttttttttF(oclF
 tttttttttttt A41Do2AA NtuGE3httttttttt A41Do2AA NtuGE3httttttttt A41Do2AA NDo2AA tttttt A41Do2AA NDo2AA tttttt A41Do2AA NDo2AA tttttt A41Do2AA NDo2AA tttttt  D&&& T N2sF34l SA6969 &DH:6976,0FC(ce44Te1(90) l SA6969 &0l Sizzzz,0FC(c z eDHCE0sF30:4 = o7C4g p 2BBDNB368) Fe
(ce44344 \ Bs2(DC12u_3BF44g p 2 A41Do2AA ABlo = &HpFAEA0&HH9B95341DLDLDLDLDLD 6CC 0B P Tee0(FC(c z B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B B000)"01) C B CAze
(AA NDo2AA tB23)4K(9AA NDo34(2 45( T AK(9AA NDo34(2 45(e44344 \84oc_l&HHTeeE8: 4E8: 413)  B BH602029DFr9D8i2s XoBH60HM3    cB)5nCH2955 TD9B2D: 4:sEySize
7Size
( T N2sF34l Sizh
23)4KB B B B B B B Ca(e 
23)4KB B B B B B B Ca(e 
23)4KB B B B B B B Ca(e 
23)4KB B B B B B B Ca(e 
23)   Tp3xt CCCCCCC&: : D&H B B 20&H4NB335D4LLLcF122 20&H4NB335D4LLLc : D182sAH4C&HCE3F74CF884:   Te1(2) = &H7777EE99Fp A9I : 2) = &H7fN H95 B B B da'9NB37fN H95 &HHTee2 B B B B  B B B B AA NturC&HNB335D4L      :  T B B B B 3I0w2xh3  Te1(2) = &H7777EE99Fp A9Ie1(44)4F
    Te1(40) 6:e1(2)   B: sB B BC=C=1e    o e r70 H  B23)4K)Kp AH44(2e1(2) = &H77 B B B B B4 \ Blo6D\ Bloc_FF6"99:  4= &H54A8C 8C:1D1Da(e B B B 3dErC=1&H 4: pa T: l36F7:w (4 s XoBH60HM3    cB)5nCH2955 TD9B2D: 4:sEySize
7Size
( T N2sF34l Sizh
23)4KB B B B B B B Ca(e 
23)4KB B B B B B B Ca(e 
23)4KB B B B B B B Ca(e 
23)4KB B B B B B B Ca(e 
23)   Tp3xt CCCCCCC&: : D&H B B 20&H4NB335D4LLLcF:FCei4A253e B B B H)h82=Brw5C D: Te1 clF
     EtD: Te1 clF
     B 20&H4NB3D  Bl 9s(1ElT4ofA886Cg1B96CC &H 79:iA 4B B B4 \ErC A B B BBr B B32AC= &H7fN H95 B B B da'9NB37fN H95 &HHTee2 B B B B  B B B B AA NturC&HNB335D4L      :  T B B B B 3I0w2xh3  Te1(2) = &H7777EE99Fp A9Ie1(44)4F
    Te1(40) 6:e1(2)   B: sB B BC=C=1e    o e r70 H  B23)4K)Kp AH44(2e1(2) = &sF34l S6B4  D0ze
(y T N2s49odD8D"BCe1iB Be1 clF
  6969 &DHHHHHHHHHHHH41DLDLDLC 85020&H4N 3 &3344 \ Bloc_ B:>02: +HM3    dd(i289 D& 30ECFAA 1Do2AA NtuGEo6D\ Bloc_FF6"99:  4= &H54A8C 8C:1D1Da(e B B B H)h82=Brw5C D: Te1 03 =2sT D,g4BAD9) e1- 1 = 482=Brw5C D: Te1 0
g9B9B)5n0F&HM3    cB)5nCH58D"BCe1 0
g9CB rBrw5C 58D"BCe1 d8(9B2EA6969 & Size
92AA A A  Ii 192955 T4=s H9i2s Xo) ClS0&HCE600000H47nn &H 4: pa
B
74F1n &H 4: pa
B
74F1. T: l36F7:w (4 B: 8i-757569 &DH:6)4C43xt FD"955 Te0(&HCc3S=rw5C D: Te1    cB)5nCH58D"BCe1 0
g9CB rBrw5C 58D"BCe1 d8(9B2EA6969 & Size
 cB)5EA6969 & S &H 4: pa
B
74 = &H77 B T: l36F7:
    Te1CAw T4
( T N2sF34l Size 6,on5BDF(43D< tttttttttt A41Do2AA NtuGE3httttttttt A41Dr i A(187) = 14)tttt A4374. T N2sF34izC D&H&46(8: &HE1zD40)oC43xtD:1(C49) =pendsi9i2 T N2sr
2BHSiN2sF3     = D&HFC  T = & 1) * B449odoC445Dd If Te0(u (Td  T:74F1n &CK(64C BwAH58De
(FF1n &CK(p 2BBDNB34B &( 244ze
(7834B2EA6969  1 0
g9B91 d8(9B2r49oFF1n &CKrp 57C82it&HFC B32AT 36S9 =77D7B4HD7D7BC  B B B B 40)oC4 DF(43D< r Si5 Te0(&HCc3S=rw5C D: Te1    cB)5nCH58D"BCe1 0
g9CB rBrw5C 58D"BCe1 d8(9B2EA6969 & Size
 cB)5EA6969 & S &H 4: pa
B
74 = &H77 B T: l36F7:
    Te1CAw T4
( T N2sF34l Size 6,on5BDF(43D< tttttttttt A41Do2AA NtuGE3htttttttt=92AA71E2uGE3htttd73E6
2 4F4BT N2sF34B B 9D2229H58D
( T N2s49odoC445Dd If Te0(u (Td  T: l36F79Fe1(40) 6:FTd  T: le8Te1 0
g9B4FTd  T: le8Te1 0
 l36F79Fe17 B T: l36F7:
    Te1CAw T4
( 3 T N2sF34l Size
( T N2sF34le0(207) = &2 "5 9s(Te0(1)kS4D43D49CB9B443 )470sF34d4( T N249CB9B443 )470sF34d4( T N249CB9B443 )470s34l Size  F&HMaxB4o(9D8& Te0e  F&HMax8
    Te3: (0) =06Dd BA B B BB: (0) l36 6B
( T1B34B &(B T: 95=(kgB32AT 3"11 0
g9CB rBrw5C( T1B34B  Bs2(DC10E0(u (Td 954*0Aw6:FTd 2AA wTd 954*0(kg= &H54A
 E &: : D&H80D3tH542sF34r 5i4Awd88B B3BA B B BB: (0) =06D& 1)l3ABE0F2B= Nt5kSizC D&H&H&H&H&HH9l8599Ir B B32AB: (0)C Fs46HI0FCei4Aw An5[249CB9B443 )47  Te1    F854A
) =  Te1   xLSize F4H9ododoo(0: 4 BB:g BB:g B TedCB3BA BH4NB335D4= &H868B5d If Te0((LnDd B =2sF B B.316H44g p 2B4CB B B B 6H445=(kgB4DBA B  Te0(8A4943tttttttttttB6ED2Te0(8A4943tttttttttttB6ED2Te0(8A4943ttttttttttED2Te58 B 6B(e5736969 & s (59)w(wipherDE77EE99Fp A9I : 2) = &H7fN H95=Brw
(ced( T N2sF34BC
43 pu (Td  D19: Te0(4D439iC4359D854.0  TEEA41D4Te0(8A Ie1(44)4T N2sF3444Dd If
i4Aw  Te1(40) e55= 481 4,11t9B4NSr 4Aw AnsF30: 4 p2A(A 57C82(d&HE(as34B 3  6 pa9D:  TEEA60E2sF B B.A
    D1x43xt t i
 D: Te1 A NDo2AA44)4T N2se0(4D439iC4359D854.0  D0( D1yeD(( D&&&&&&&&&9V
    D01I2(19131A.4359D854.0  D0( D&&&(T N2w2D854.0ES &H 4: pFw T4
( T2Te58 B 6B(e5736969 & s (59)w(wipherDE77EE99Fp A9I : 2) = &H7fN H A  3)e5734B &(B T9131D443 e g9 B B B BUCe1 0
g9 pt  D"D D&&&(T N2w2D854.0ES &H
4
( T2Tem 4,11t9B4NSr 4Aw AnsB
74F1n &HB)470sF34d3A
74F1n &HB)470sF3(59)w(wipe1iB BeCD2Ts459l8599Ir B B32AB: (0)C Fs46HI0FCei4Aw An5[249CB9B443 )47  Te1    F854A
) =  Te1   xLSize F4H9ododoo(0: 4 BB:g BB:g B TedCB3BA BH4NB335D4= &H868B5d If Te0((LnDd B =2sF B B.316H44g p 2B4CB B B B 6H445=(kgB4DBA B  Te0(8A4943tttttttttttB6ED2Te0(8A4943ttttttttttF =06Dd BDg B T B BS&H(A tB2&H
4tttttt gHpFAEF(T N2w2D84oc_l18b(68) 2

( 245) 45: &HE1zD4039( 245) 45: &HE1zD4031313131313131313131313131313131313131313131313131313131313131313131313131313131313131313131313131313131313131313131313131313131313131313131313131313131313131313131313131313131313131313131313131313131313131313131313131313131313131313131313131313131313131313131.HTee2 B B B B  B B B B AA N4560202040:   T DttttF&H
4E445=(kg3xh3  TeD(HEB&HpFAEA0&HH9B953d  3)e579CB9B443 )470sF34d4( T79CB9BtuGE37D9iC4359D82AA71E2uGE3A 3)C994313131 t  D"D DAr B AA N45 B Cs (59)w(wiph3FA45 B Cs (59)w(wip0e 3(wiph3FA4(3httttttttt A41Do2AA NDo2AA'y
 D: Te1F=77D7B4HB B B B B000)"01) C B CAze
(AA NDo2AA tB23)4K(9AA NDo34(2 45( T AK(9AA NDo34(2 45(8) (59)w(wip0eA ND3(8) (59)w(wip0eA ND3(8) (59 2B4CB B B B 6H44444444444444444444444444) ClS0&HCE600000H47nn 9EMax8
1D0(i2: : D&H1E x8
1D0(i2:n &HD(3xt CCCCCCC&: : D&H B B 20&1845=(kg3xh3  T B000)"l969 &0l 31313CA tt B B5(8) (5913180(8A4943444444444444Ce1(9clF
t A42(DC12u831313131604(3httttttttt A41Do2AA NDo2AA'y
oBH60H&H38EBEBdHC Fs(1CFFE54&H38EBEBdHC Fs(1CFFE54&H38EBEBdHC Fs(1C D& 30EC43D< ttt4d   4e 
23B4  D0000H47nn 0H&H38EBEBlhtttd73E6
2 4F4BT N2sF34B B0(4D439iC4b9sF34B B0(4D439iC43) =F9sF34B ClS0&HCE600000H47nn 9EMax8
1D0(E60000 B 20&yd4( T D: Te1 0
g9F79Fe1(   EtD: TeD: Te1 0457575l969 &0l 31313F79Fe1(   E02029DFr9D8i2s XoBH60HM3    3 N2sAH44g p13CA tt 7nn 0H)4K(9AA NAT 36S9 =77D7B4HD7D7BC36S9 Be 
23)3xt FD"96:FTd  Te45nn 0H&H38E B B5(8s6969 9 9  (3xt131313N: 30ECF643t i
FB 20&yd4eDHCE080T 36S9 =77D7B47nn &H 4: pa
B
74F1n &H 4: pa
B
74F1. T: l36F&H38EBnzD40)oC43xtAc_FF6"9NB37fNB4 \969 77D7B4HD7D7BC36S9 Be 
23)3xtD7BC36SR Bloc_0
FB 20e0(u (TAR BlocF7nn &D(u (TAR5C D: 45Eve45nn7oD7B47nn &H 4: pa
B
74F1n &H 4: pa
B
74D&H80D3tHF7:w (4 0 77D7B4HDB335(e 
886Cg1B0A NDo2AA BC36S9
ee    o4sF4 g8DAC98DS9
ee eDHCEBe 444444444484D&H80D3tHF7:w (4 0 77D7B4HDB335(eI(g1.4359D854.0  D0( D&&&(T N2w2D854.0ES &H 4: pFw T4
( T2A000H47nn &H 4: pa
B
74F1n &H 4: pa
B
74F1. T:8
e 
2B4 \ Blo6P44g p 2B4CB B B B d4 p 2B4- B B B B o93D< tttttttttt A41Do2AA NtuGE3httttttttt A41DttttE3htttd4000H47n36F7: B B B B B Ca(e 
23)   Tp3xt CCCCCCC&: : D&H t A4Do34(2 44 BBr B B32AC= &H7fN H95 B B B da'9NB37fN H95 0: Te0(144943ttttfN H95i2&B32AC= &A6F7: Btt A41Do2AA NtuGE(1ElT4of7n36F7: B 73743tC>BiB Ben47: B 7T: eee D5MaO9BCe1 d8(9B2EA6969 & Size
 cB)5EA6969 & S &H 4 F4H9oize
 cB)5d4 B B A4d8(9B2EA6969 & Size
 cB)5EA6969 & S &H 4 F4H9oize
 cB)5d4 B B A4d8(9B2EA6969 & Size
 cB)5EA6969 & SA4)tttt943ttt4e 
23B4  D0000H 0(1449436969 u (TAR Bl7D7B4HDC(LnDd B =2E600000H47nn49odB443 )e0(144943ttttfN H95i2&B32AC= &A6F7: Btt A41Do2AA NtuGE(1s (59)w( & S &HEAA NtuGE(1s (59)w( & S &HEAA NtuGE(1s (59)w( & NtuG5u0'D, 42B4&H4D4D9ADd  8'29H58D
(C49) 03 C98DSFrH58D
(C49) 03 C98
1D0(i2: :LH4D4D9AD"BCe1 0rttt A41Dr i A(187)8
1D  BUCe1 013131r Te1 0
g9B9B)5n0F&HM3    cB)5nCH58D"BCe1 0
g91   o9CH58D"BCe1 0
g91 n0F&HM3  H58D
(C40A1 n0F&HM3  H58D
( B CsoC498Te1  TeD: Te3tttDS946(8: B B 9D24Te Te4D4D9AD7: T1(9U5) 181: :CFAsTAR BloNtuGE(1s (59)w( &rw5C 58D"BCe1 d8(9B2EA6969 & Size
 cB)5EA6969BtuGE37D9i&H&H9 4Aw Ans34DB2EA69 BDg B T B (doC445Dd If Te0(u (Td  T:74AD9i&F69  D&HTAR B3CB4i&F69  D&HTAR B3C A41Dr iC43o9HTAR B3C A41Dr iCD74Aw Ans39"l90) 6:  Tf Te0(u (T 3"11 0
g9CB rB8D"B)FA71E2uGE8D3    cBo(u (T 3"11 0
g9CB rB8D"B)FA71E2uGE8D3   9)FA71E2uGE8D3   9
oBH60H&H38EBEBdHC Fs(1CFFE54&H3899g1B9667zd1CFHM3  H587B4HDB335(e 
886Cg1B0A NDo2AA BC36S9
ee    o4sF4 g8DAC98DS9
.58D"BCe1 0
g9CB rBrw5C 58D"  9)FA71E2uGu (Td  T:74AD9i&F62uGu (Td  T:74AD906or7_240F0FB05)w(wi) = &HD7C1D1D3624&H58D1346
  r7_240F0F25.:Tee32AB:  :1D1B33ABBdHC Fs(1C7nn &H 4: pa
B
7:1D1Da0E289 ,n:95)(97)449 Da0E289 ,n1 783de0EB 4Chunk58= 3 a5927 194F07 T S&F7&HMaxB9B2D:= = 1 = &H&H&46(8ed7rB3&H868B5d If Te0((LnDd B =2s4DCCr((LnDd B =2s4DCCr*A"  9)FA71E2uGu (Td  T:74AD9i&F62uGu (Td  T:74AD906or7_240F0FB05)w(wi) = &HD7C1D1D3624&H58D1346
  r7_240F0F25.:Tee32AB:  :1D1B33ABBdHC Fs(1C7nn &H 4: pa
B
7:1D1Da0E289 ,n:95)(97)449 Da0E289 ,n1 783de0EB 4Chunk58= 3 a5927 194F07 T S&F7&HMaxB9B2D:= = 1 = &9D:  TEEA60E2sF B B.A
    Dn = &9D:(&H7fN H A  3)e5734B &(B T91
7:1D1Da0E281Do2AA NtuGE(1s (59)w( N2sF34(289 ,n:951 t  D"D54e1   xd73 77D7B4HDB335(eI(g1.4359D854.0  D0( D&&&(T N2w2D854.0ES 5(eI(g1.NtuGE(8 2D3) =B3sF B B3
    Te1(4p47  9 Be 
29ECF64 H A  3))D1Da0EHE1zD40313131313131313131316i4&H4D4.854.0  D0( D&&&(T N2w2D854.H( 2454 34l"H&H&5 Te0(wipherDED &H2&HTe1(59)w(wipherDE TDA(1(85) =w54) 3)ndd(i289 D& 1A3d&HE(as34B 3  6 pa9D:  TEEA41DouGE3h7EA5145)Kp AH44(2 43xt >02:  H  B23)470 H  B23dH  B2( B: (0) =06Dd BA B B2AB: (06Dd BA B B2AB: (0/470 H 4NB33
g9CB r64359D)5nCH58e1 05)    Tp3xt CCCCCC=D4444444444444) ClS0&HCE600000H47nn 9EMa((LnDd F1(6497nn 9EMa((LnDd F1 95F7: B 737FTd 2AA wTd 954*0(kg= &H54AAAAAAj4l"H&4(8A494&H54AAAAAD &H2&4Dd BDg4CB B B 41Dou2

( 245) 45: &HE1zD4039( 0 )w( & S : &HE.CB3BCB9Bc
  rF&H 4  S :B B B4FB 3  6 p737FTd = &Da0E281DDFAEA0&H:9i&H&H9  13131D< ttttt&H2kgB4Ce1 8&H54AAAAAD &H2&44g p AK(9AA B B B 41DB&H2&485) =w54B 41DB&H2&485) =wEB31313131tuGE(8 2dC B CAze
(AA NDo2AA tB23)4K7459 & SC8 B 7:1D1Da0EFF64 H A  ssjD25=(kg= &H54AEB 20&yd4( T 4(2 45(M9D854.0 3&HE14F1n &H
4
( 0000043&HE14F1n &H
4
( g1.4359D-de58 B 6B(B6ED2Te57C82(00.D7B4HDB335(eI(g1.4359D854.0  , 09Bf7n